/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#include "apologies.h"
#include "serverWiper.h"
#include "cvars.h"

#include <stdlib.h>

ApologyManager::ApologyManager()
{
}

ApologyManager::~ApologyManager()
{
    apologies.clear();
}

void ApologyManager::loadApologies(shString const& filename)
{
    apologies.clear();

    shString pathAndFilename = "cfg/sw/text/";
    pathAndFilename += filename;
    if(!ServerWiper::getInstance()->getFilesystem()->FileExists(pathAndFilename.c_str(), "MOD"))
    {
        META_LOG(g_PLAPI,"Could not open the apology file : %s", filename.c_str());
        return;
    }

    FileHandle_t file = ServerWiper::getInstance()->getFilesystem()->Open(pathAndFilename.c_str(), "r", "MOD");

    int apologyCount=0;
    for(;!ServerWiper::getInstance()->getFilesystem()->EndOfFile(file);apologyCount++)
    {
        char text[1024];
        ServerWiper::getInstance()->getFilesystem()->ReadLine(text, sizeof(text), file);
        int text_size = Q_strlen(text);
        for(int i=0;i<text_size;i++)
        {
            if(text[i]=='/')
            {
                text[i]='\0';
                i=text_size;
            }
        }
        text[text_size-1] = '\0';
		for(int i=text_size-1;i>0;i--)
		{
			if(text[i] == '\n' || text[i] == '\r' || text[i] == '\0')
				text[i] = '\0';
			else
				break;
		}
        addApology(getLowerString(text));
    }

	ServerWiper::getInstance()->getFilesystem()->Close(file);

    META_CONPRINTF("Read %d apologies\n", apologyCount);
}

bool ApologyManager::isApology(shString const& text, shString & apology)
{
    shString newText = getLowerString(text);

    //Put spaces before and after
    shString tmp = " ";
    tmp += newText;
    tmp += " ";
    newText = tmp;

    for(int i = 0;i<apologies.size();i++)
    {
        shString trimedApology = apologies[i];
        trimedApology.trim();
        if(isPatternInString(newText, apologies[i]) || newText == trimedApology)
        {
            apology = apologies[i];
            return true;
        }
    }

    return false;
}

void ApologyManager::addApology(shString const& apology)
{
    shString tmp = " ";
    tmp += apology;
    tmp += " ";
    apologies.push_back(tmp);
}

