/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#include "server_wiper_mm.h"
#include "cvars.h"
#include "time.h"
#include "url.h"
#include "language.h"
#include "help.h"

void onServerWiperStatusChange(ConVar *var, char const *pOldString)
{
    ServerWiper::getInstance()->statusChanged(CVAR_BOOL(g_ServerWiperEnabled));
}

void onLanguageChange(ConVar *var, char const *pOldString)
{
    LanguageManager::getInstance()->loadLanguageFile(g_Language.GetString());
    HelpManager::getInstance()->loadHelpMessages();
}

ConVar g_ServerWiperVersion("sw_version", SERVER_WIPER_VERSION, FCVAR_REPLICATED|FCVAR_NOTIFY, "ServerWiper version");
ConVar g_ServerWiperEnabled("sw_enabled", "1", FCVAR_REPLICATED|FCVAR_NOTIFY, "1 if the ServerWiper is enabled, 0 else", onServerWiperStatusChange);
ConVar g_ShortServerName("sw_short_server_name", "myserver", 0, "short server name, used for sending bans to an url");
ConVar g_Language("sw_language", "en", 0, "ServerWiper default language", onLanguageChange);
ConVar g_TeamAttack("sw_ta", "1", FCVAR_NOTIFY, "0 = no team attack detection\n1 = team attack detection");
ConVar g_TeamKill("sw_tk", "1", FCVAR_NOTIFY, "0 = no team kill detection\n1 = team kill detection");
ConVar g_TeamFlash("sw_tf", "1", FCVAR_NOTIFY, "0 = no team flashing detection\n1 = team flashing detection");
ConVar g_Insult("sw_insult", "1", FCVAR_NOTIFY, "0 = no insult detection\n1 = insult detection");
ConVar g_ApologyTeamAttack("sw_apologies_ta", "1", FCVAR_NOTIFY, "0 = no apologies detection for team attack\n1 = apologies detection for team attack");
ConVar g_ApologyTeamKill("sw_apologies_tk", "1", FCVAR_NOTIFY, "0 = no apologies detection for team kill\n1 = apologies detection for team kill");
ConVar g_ApologyTeamFlash("sw_apologies_tf", "1", FCVAR_NOTIFY, "0 = no apologies detection for team flashs\n1 = apologies detection for team flashs");
ConVar g_ApologyInsult("sw_apologies_insult", "1", FCVAR_NOTIFY, "0 = no apologies detection for insults\n1 = apologies detection for insults");
ConVar g_BombPlant("sw_bomb_plant", "1", FCVAR_NOTIFY, "0 = no bomb planting detection\n1 = bomb planting detection");
ConVar g_BombDefuse("sw_bomb_defuse", "1", FCVAR_NOTIFY, "0 = no bomb defusing detection\n1 = bomb defusing detection");
ConVar g_HostageAttack("sw_hostage_attack", "1", FCVAR_NOTIFY, "0 = no hostage attack detection\n1 = hostage attack detection");
ConVar g_HostageKill("sw_hostage_kill", "1", FCVAR_NOTIFY, "0 = no hostage kill detection\n1 = hostage kill detection");
ConVar g_HostageFollow("sw_hostage_follow", "1", FCVAR_NOTIFY, "0 = no hostage follow detection\n1 = hostage follow detection");
ConVar g_HostageRescue("sw_hostage_rescue", "1", FCVAR_NOTIFY, "0 = no hostage rescuing detection\n1 = hostage rescuing detection");
ConVar g_Greetings("sw_greetings", "1", FCVAR_NOTIFY, "0 = no greetings detection\n1 = greetings detection");
ConVar g_MessageBoxTime("sw_message_box_time", "7", 0, "time in seconds until message boxes disappears (insults, etc.)");
ConVar g_DebugLevel("sw_debug_level", "1", 0, "debug level, 0 = none, 1 = normal");
ConVar g_Advert("sw_advert", "1", 0, "0 = no advert\n1 = advert (thanks :p)");
ConVar g_AdvertDelay("sw_advert_delay", "600", 0, "delay in seconds between two advert");
ConVar g_HelpMessages("sw_help_messages", "1", 0, "0 = no help messages\n1 = help messages");
ConVar g_HelpMessagesDelay("sw_help_messages_delay", "265", 0, "delay in seconds between two help messages");
ConVar g_Register("sw_register", "1", 0, "allows or not registration with the global server (for statistical usage only)\n0 = no registration\n1 = registration");
//ConVar g_UseMessageBoxes("sw_use_message_boxes", "1", 0, "0 = don't use messageboxes, say only\n1 = use messageboxes");//TODO
ConVar g_SendBanURL("sw_send_bans_url", "", 0, "url to send bans to, use the empty string to prevent ban sending");
ConVar g_SendAdminBans("sw_send_admin_bans", "1", 0, "also send bans made by admins to the url, 1 = yes, 0 = no");
ConVar g_StatusCommand("sw_status_command", "status", 0, "chat command to show the status");
ConVar g_AllowStatus("sw_allow_status", "1", 0, "allow or not the status command, 1 = yes, 0 = no");
//ConVar g_SettingsCommand("sw_settings_command", "swsettings", 0, "chat command to show the settings");
//ConVar g_AllowSettings("sw_allow_settings", "1", 0, "allow or not the settings command, 1 = yes, 0 = no");
ConVar g_HideSayCommands("sw_hide_chat_commands", "0", 0, "hide or not chat commands, 1 = yes, 0 = no");
ConVar g_HideSayInsults("sw_hide_chat_insults", "0", 0, "hide or not chat messages with insults, 1 = yes, 0 = no");
ConVar g_DisableChat("sw_disable_chat", "0", 0, "disable or not the chat (does not block Mani Admin Plugin admin say), 1 = yes, 0 = no");
ConVar g_WelcomeMessage("sw_welcome_message", "1", 0, "show or not the welcome message when a player joins the server, 1 = yes, 0 = no");
ConVar g_ShowStatusOnDeath("sw_show_status_on_death", "0", 0, "show or not the player's status when he dies, 1 = yes, 0 = no");
ConVar g_CheckPlayersName("sw_check_players_name", "1", 0, "check or not the player's name for insults, 1 = yes, 0 = no");
ConVar g_BanRecoveryPlaytime("sw_ban_recovery_playtime", "60", 0, "playtime required, in minutes, to regain 1 ban point");
ConVar g_ForgiveTkMenu("sw_forgive_tk_menu", "1", 0, "show a menu to enable forgiveness, 1 = yes, 0 = no");
ConVar g_ForgiveTkMenuTime("sw_forgive_tk_menu_time", "30", 0, "how many time, in seconds, a player has to forgive or not a team killer");
ConVar g_SavePlayersAtRoundStart("sw_save_players_at_round_start", "1", 0, "save or not players status in the database at round start, 1 = yes, 0 = no");
ConVar g_UseDelaysToLimitApologies("sw_use_delays_to_limit_apologies", "0", 0, "use or not time delays to limit player apologies, 1 = yes, 0 = no");
ConVar g_DetectManiAdminPluginBans("sw_detect_mani_admin_bans", "1", 0, "detect or not bans made with Mani's Admin Plugin, 1 = yes, 0 = no");
ConVar g_DetectSourceModBans("sw_detect_sourcemod_bans", "1", 0, "detect or not bans made with SourceMod, 1 = yes, 0 = no");
ConVar g_RemindPlayersWithBadBehaviour("sw_remind_players", "1", 0, "remind or not players when they got kicked and retry, 1 = yes, 0 = no");
ConVar g_SoundToPlayWhenReminding("sw_remind_players_sound", "ambient/sheep.wav", 0, "sound to play when reminding, set to the empty string to disable");
ConVar g_ShowSteamIdOnConnect("sw_show_steamid_on_connect", "1", 0, "show the steamid of new players, 1 = yes, 0 = no");
ConVar g_TeamMembersTagCheck("sw_team_members_tag_check", "0", 0, "check if team members have the team tag, if no they get kicked, 1 = yes, 0 = no");
ConVar g_TeamMembersTag("sw_team_members_tag", "", 0, "team tag, used when sw_team_members_tag is set to 1");
ConVar g_DatabaseCleanDelay("sw_database_clean_delay", "100", 0, "delay in days to remove players from the database, set it to 0 to disable");

ConVar g_PointLossRatio("sw_point_loss_ratio", "1", 0, "multiply all the point loss by a ratio when a player does many bad things in a row, 1 = yes, 0 = no");
ConVar g_PointLossRatioDelay("sw_point_loss_ratio_delay", "120", 0, "delay in seconds in whitch the point loss when making a bad action are multiplied with a ratio");
ConVar g_PointLossRatioMax("sw_point_loss_ratio_max", "3", 0, "maximum ratio for sw_point_loss_ratio_delay");

ConVar g_PunishAtRoundStart("sw_punish_at_round_start", "1", 0, "punish more at round start, 1 = yes, 0 = no");
ConVar g_PunishAtRoundStartDelay("sw_punish_at_round_start_delay", "10", 0, "duration in seconds of the round start");
ConVar g_PunishAtRoundStartRatio("sw_punish_at_round_start_ratio", "2", 0, "ratio for punishment at round start");

ConVar g_PunishLeavers("sw_punish_leavers", "1", 0, "punish more if a player make crap and leave, 1 = yes, 0 = no");
ConVar g_PunishLeaversDelay("sw_punish_leavers_delay", "15", 0, "duration in seconds between the bad action and the leaving");
ConVar g_PunishLeaversBanPoints("sw_punish_leavers_ban_points", "-10", 0, "ban points for leavers punishment");

ConVar g_AllowedNamesCheck("sw_allowed_names_check", "0", 0, "check the names to look the % of allowed chars in it and kick players with bad nicknames, 1 = yes, 0 = no");
ConVar g_AllowedCharactersInName("sw_allowed_chars_in_names", "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ", 0, "the characters which are allowed in names");
ConVar g_MinAllowedCharactersInName("sw_min_allowed_chars_in_names", "30", 0, "minimum percentage of allowed chars in names");

ConVar g_MaxBehavPoints("sw_max_behav_points", "100", 0, "maximum number of behaviour points");
ConVar g_MaxBanPoints("sw_max_ban_points", "25", 0, "maximum number of ban points");
ConVar g_BehavPointsKickSteps("sw_behav_points_kick_steps", "4", 0, "number of kick steps related to behaviour points");

ConVar g_CountTAWhenDead("sw_count_ta_when_dead", "0", 0, "count (1) or not (0) team attacks when the attacker is dead (he grenades)");
ConVar g_CountTKWhenDead("sw_count_tk_when_dead", "0", 0, "count (1) or not (0) team kills when the killer is dead (he grenades)");
ConVar g_CountTFWhenDead("sw_count_tf_when_dead", "0", 0, "count (1) or not (0) team flashs when the flasher is dead (flash grenades)");
ConVar g_CountHostageAttackWhenDead("sw_count_hostage_attack_when_dead", "0", 0, "count (1) or not (0) hostage attacks when the attacker is dead (he grenades)");
ConVar g_CountHostageKillWhenDead("sw_count_hostage_kill_when_dead", "0", 0, "count (1) or not (0) hostage kill when the killer is dead (he grenades)");

ConVar g_TeamAttackDelay("sw_ta_delay", "1", 0, "delay in seconds between two team attack detection");
ConVar g_HostageAttackDelay("sw_hostage_attack_delay", "1", 0, "delay in seconds between two hostage attack detection");
ConVar g_TeamFlashApologyDelay("sw_tf_apology_delay", "120", 0, "delay in seconds between two team flash apologies");
ConVar g_TeamAttackApologyDelay("sw_ta_apology_delay", "120", 0, "delay in seconds between two team attack apologies");
ConVar g_TeamKillApologyDelay("sw_tk_apology_delay", "240", 0, "delay in seconds between two team kill apologies");
ConVar g_InsultApologyDelay("sw_insult_apology_delay", "240", 0, "delay in seconds between two insult apologies");
ConVar g_GreetingsDelay("sw_greetings_delay", "480", 0, "delay in seconds between two greetings");
ConVar g_InvalidCharacters("sw_invalid_characters", ".+-*_|!/(){}[]?", 0, "characters to delete when analysing a string");
ConVar g_GreetingsTime("sw_greetings_time", "10", 0, "time in seconds after a player joins within the greetings are valid");
ConVar g_TeamAttackApologyTime("sw_ta_apology_time", "20", 0, "time in seconds after a team attack within the apologies are valid");
ConVar g_TeamFlashApologyTime("sw_tf_apology_time", "20", 0, "time in seconds after a team flash within the apologies are valid");
ConVar g_TeamKillApologyTime("sw_tk_apology_time", "20", 0, "time in seconds after a team kill within the apologies are valid");
ConVar g_InsultApologyTime("sw_insult_apology_time", "20", 0, "time in seconds after an insult within the apologies are valid");

ConVar g_TeamAttackSteps("sw_steps_ta", "5,30,50,70", 0, "damage steps for team attacks");
ConVar g_BehaviourPointsTeamAttack("sw_behav_points_ta", "-1,-2,-3,-4,-5", 0, "behaviour points for a team attack for each step (per shot)");
ConVar g_BanPointsTeamAttack("sw_ban_points_ta", "0,0,0,0,0", 0, "ban points for a team attack for each step (per shot)");
ConVar g_MoneyTeamAttack("sw_money_ta", "0,0,0,0,0", 0, "money for a team attack for each step (per shot)");
ConVar g_HealthTeamAttack("sw_health_ta", "0,0,0,0,0", 0, "health for a team attack for each step (per shot)");

ConVar g_BehaviourPointsTeamKill("sw_behav_points_tk", "-10", 0, "behaviour points for a team kill");
ConVar g_BanPointsTeamKill("sw_ban_points_tk", "-2", 0, "ban points for a team kill");
ConVar g_MoneyTeamKill("sw_money_tk", "0", 0, "money for a team kill");
ConVar g_HealthTeamKill("sw_health_tk", "0", 0, "health for a team kill");

ConVar g_BehaviourPointsInsult("sw_points_behav_insult", "-10,-15,-20", 0, "behaviour points for saying an insult for each insult level");
ConVar g_BanPointsInsult("sw_points_ban_insult", "-2,-3,-5", 0, "ban points for saying an insult for each insult level");
ConVar g_MoneyInsult("sw_money_insult", "0,0,0", 0, "money for saying an insult for each insult level");
ConVar g_HealthInsult("sw_health_insult", "0,0,0", 0, "health for saying an insult for each insult level");

ConVar g_BehaviourPointsTeamAttackApology("sw_behav_points_ta_apology", "1,2,3,4,5", 0, "behaviour points for a team attack apology for each step (per shot)");
ConVar g_BanPointsTeamAttackApology("sw_ban_points_ta_apology", "0,0,0,0,0", 0, "ban points for a team attack apology for each step (per shot)");
ConVar g_MoneyTeamAttackApology("sw_money_ta_apology", "0,0,0,0,0", 0, "money for a team attack apology for each step (per shot)");
ConVar g_HealthTeamAttackApology("sw_health_ta_apology", "0,0,0,0,0", 0, "health for a team attack apology for each step (per shot)");

ConVar g_BehaviourPointsTeamKillApology("sw_behav_points_tk_apology", "5", 0, "behaviour points for a team kill apology");
ConVar g_BanPointsTeamKillApology("sw_ban_points_tk_apology", "0", 0, "ban points for a team kill apology");
ConVar g_MoneyTeamKillApology("sw_money_tk_apology", "0", 0, "money for a team kill apology");
ConVar g_HealthTeamKillApology("sw_health_tk_apology", "0", 0, "health for a team kill apology");

ConVar g_BehaviourPointsInsultApology("sw_points_behav_insult_apology", "10,7,5", 0, "behaviour points for saying an insult apology for each insult level");
ConVar g_BanPointsInsultApology("sw_points_ban_insult_apology", "0,0,0", 0, "ban points for saying an insult apology for each insult level");
ConVar g_MoneyInsultApology("sw_money_insult_apology", "0,0,0", 0, "money for saying an insult apology for each insult level");
ConVar g_HealthInsultApology("sw_health_insult_apology", "0,0,0", 0, "health for saying an insult apology for each insult level");

ConVar g_BehaviourPointsBombPlant("sw_behav_points_bomb_plant", "5", 0, "behaviour points for bomb planting");
ConVar g_BanPointsBombPlant("sw_ban_points_bomb_plant", "0", 0, "ban points for bomb planting");
ConVar g_MoneyBombPlant("sw_money_bomb_plant", "0", 0, "money for bomb planting");
ConVar g_HealthBombPlant("sw_health_bomb_plant", "0", 0, "health for bomb planting");

ConVar g_BehaviourPointsBombDefuse("sw_behav_points_bomb_defuse", "5", 0, "behaviour points for bomb defusing");
ConVar g_BanPointsBombDefuse("sw_ban_points_bomb_defuse", "0", 0, "ban points for bomb defusing");
ConVar g_MoneyBombDefuse("sw_money_bomb_defuse", "0", 0, "money for bomb defusing");
ConVar g_HealthBombDefuse("sw_health_bomb_defuse", "0", 0, "health for bomb defusing");

ConVar g_BehaviourPointsHostageAttack("sw_behav_points_hostage_attack", "-1", 0, "behaviour points for hostage attacking");
ConVar g_BanPointsHostageAttack("sw_ban_points_hostage_attack", "0", 0, "ban points for hostage attacking");
ConVar g_MoneyHostageAttack("sw_money_hostage_attack", "0", 0, "money for hostage attacking");
ConVar g_HealthHostageAttack("sw_health_hostage_attack", "0", 0, "health for hostage attacking");

ConVar g_BehaviourPointsHostageKill("sw_behav_points_hostage_kill", "-4", 0, "behaviour points for hostage killing");
ConVar g_BanPointsHostageKill("sw_ban_points_hostage_kill", "0", 0, "ban points for hostage killing");
ConVar g_MoneyHostageKill("sw_money_hostage_kill", "0", 0, "money for hostage killing");
ConVar g_HealthHostageKill("sw_health_hostage_kill", "0", 0, "health for hostage killing");

ConVar g_BehaviourPointsHostageFollow("sw_behav_points_hostage_follow", "2", 0, "behaviour points for hostage using (sw_max_hostage_follow_in_round times per round)");
ConVar g_BanPointsHostageFollow("sw_ban_points_hostage_follow", "0", 0, "ban points for hostage using (sw_max_hostage_follow_in_round times per round)");
ConVar g_MoneyHostageFollow("sw_money_hostage_follow", "0", 0, "money for hostage using (sw_max_hostage_follow_in_round times per round)");
ConVar g_HealthHostageFollow("sw_health_hostage_follow", "0", 0, "health for hostage using (sw_max_hostage_follow_in_round times per round)");
ConVar g_MaxHostageFollowInRound("sw_max_hostage_follow_in_round", "1", 0, "max hostage using in a round");

ConVar g_BehaviourPointsHostageRescue("sw_behav_points_hostage_rescue", "2", 0, "behaviour points for hostage rescuing");
ConVar g_BanPointsHostageRescue("sw_ban_points_hostage_rescue", "0", 0, "ban points for hostage rescuing");
ConVar g_MoneyHostageRescue("sw_money_hostage_rescue", "0", 0, "money for hostage rescuing");
ConVar g_HealthHostageRescue("sw_health_hostage_rescue", "0", 0, "health for hostage rescuing");

ConVar g_BehaviourPointsGreetings("sw_behav_points_greetings", "6", 0, "behaviour points for greetings");
ConVar g_BanPointsGreetings("sw_ban_points_greetings", "0", 0, "ban points for greetings");
ConVar g_MoneyGreetings("sw_money_greetings", "0", 0, "money for greetings");
ConVar g_HealthGreetings("sw_health_greetings", "0", 0, "health for greetings");

ConVar g_TeamFlashMinimumPlayers("sw_minimum_players_tf", "1", 0, "minimum flashed team players for triggering a team flash");
ConVar g_TeamFlashMinimumDuration("sw_minimum_duration_tf", "3", 0, "minimum flash duration in seconds for triggering a team flash");
ConVar g_BehaviourPointsTeamFlash("sw_behav_points_tf", "-2", 0, "behaviour points lost for a team flash, per player");
ConVar g_BanPointsTeamFlash("sw_ban_points_tf", "0", 0, "ban points lost for a team flash, per player");
ConVar g_MoneyTeamFlash("sw_money_tf", "0", 0, "money lost for a team flash, per player");
ConVar g_HealthTeamFlash("sw_health_tf", "0", 0, "health lost for a team flash, per player");

ConVar g_BehaviourPointsTeamFlashApology("sw_behav_points_tf_apology", "1", 0, "behaviour points for a team flash apology, per player");
ConVar g_BanPointsTeamFlashApology("sw_ban_points_tf_apology", "0", 0, "ban points for a team flash apology, per player");
ConVar g_MoneyTeamFlashApology("sw_money_tf_apology", "0", 0, "money for a team flash apology, per player");
ConVar g_HealthTeamFlashApology("sw_health_tf_apology", "0", 0, "health for a team flash apology, per player");

bool ConCommandAccessor::RegisterConCommandBase(ConCommandBase *pVar)
{
	return META_REGCVAR(pVar);
}

CON_COMMAND(sw_reset, "resets all players points")
{
	if(!CVAR_BOOL(g_ServerWiperEnabled))
		return;

	ServerWiper::getInstance()->resetAllPlayers();
}

CON_COMMAND(sw_reset_steamid, "resets a player points")
{
	if(!CVAR_BOOL(g_ServerWiperEnabled))
		return;

    if(ServerWiper::getInstance()->getEngine()->Cmd_Argc() < 2)
	{
	    META_CONPRINT("Usage: sw_reset_steamid <steam id>");
		return;
	}

	ServerWiper::getInstance()->resetPlayer(ServerWiper::getInstance()->getEngine()->Cmd_Argv(1));
}

CON_COMMAND(sw_reload, "reload all text and language data")
{
	ServerWiper::getInstance()->reload();
}

CON_COMMAND(sw_sm_ban, "command executed by the SourceMod plugin for the SW for detecting admin bans")
{
	if(!CVAR_BOOL(g_ServerWiperEnabled))
		return;

	if(CVAR_BOOL(g_DetectSourceModBans))
		ServerWiper::getInstance()->onSourceModBan();
	else
		META_CONPRINT("Got a sw_sm_ban command but sw_detect_sourcemod_bans is set to 0 !\n");
}
