/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#include "sqlite/sqlite3.h"
#include "database.h"
#include "server_wiper_mm.h"

#ifdef _WIN32
#include <windows.h>
#endif

#include "cvars.h"
//todo: add an auto-clean function to remove old players (not connected since 1 month)
static int callback(void *NotUsed, int argc, char **argv, char **azColName)
{
    DatabaseManager::getInstance()->clearResults();
    for(int i=0; i<argc; i++)
    {
        if(argv[i])
            DatabaseManager::getInstance()->addResult(argv[i]);
        else
            DatabaseManager::getInstance()->addResult("");
    }
    return 0;
}

void* threadHandleDatabase(void* databaseNamePointer)
{
    sqlite3 *database = 0;
    DatabaseManager *databaseManager = DatabaseManager::getInstance();
    bool wait = true;
    shString databaseName = *((shString *)databaseNamePointer);

    int result = sqlite3_open(databaseName.c_str(), &database);
    if(result)
    {
        shString message = "Can't open database: %s (code %d)";
        message += sqlite3_errmsg(database);
        message += itos(result);
        databaseManager->addMessage(message);
        database = 0;

        pthread_exit(NULL);
    }
    else
    {
        wait = false;
    }

	char *errorMessage = 0;

	while(databaseManager->getRunning())
	{
		//Stats requests
		if(databaseManager->getStatsRequestCount() > 0 && database)
		{
			DatabaseStatsRequest actualRequest;
			shString req;
			sqlite3_stmt *pStmt;
			int rc;
			databaseManager->popStatsRequest(actualRequest);

			actualRequest.setKicks(0);
			actualRequest.setBans(0);
			for(int i=0;i<EVENT_TYPE_MAX;i++)
				actualRequest.setEventCount(i, 0);

			if(actualRequest.getType() > DB_STATS_REQUEST_PLAYER || actualRequest.getType() < DB_STATS_REQUEST_SERVER)
				continue;

			if(actualRequest.getType() == DB_STATS_REQUEST_SERVER)
			{
				req = "SELECT * FROM stats WHERE 1;";
			}
			else if(actualRequest.getType() == DB_STATS_REQUEST_PLAYER)
			{
				req = "SELECT * FROM stats WHERE steamid='";
				req += actualRequest.getPlayer();
				req += "';";
			}

			if(sqlite3_exec(database, req.c_str(), callback, 0, &errorMessage) != SQLITE_OK)
			{
				databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + errorMessage);
				sqlite3_free(errorMessage);
			}

			rc = sqlite3_prepare_v2(database, req.c_str(), -1, &pStmt, 0);
			if(rc != SQLITE_OK)
			{
				databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + sqlite3_errmsg(database));
				continue;
			}

			do
			{
				rc = sqlite3_step(pStmt);
				if(rc != SQLITE_DONE && rc != SQLITE_ROW)
				{
					databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + sqlite3_errmsg(database));
					continue;
				}

				actualRequest.addEventCount(EVENT_TYPE_PLAYER_CONNECTED, sqlite3_column_int(pStmt, 1));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_TA, sqlite3_column_int(pStmt, 2));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_TK, sqlite3_column_int(pStmt, 3));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_TEAM_FLASH, sqlite3_column_int(pStmt, 4));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_INSULT, sqlite3_column_int(pStmt, 5));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_GREETINGS, sqlite3_column_int(pStmt, 6));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_APOLOGIES_FOR_TA, sqlite3_column_int(pStmt, 7));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_APOLOGIES_FOR_TK, sqlite3_column_int(pStmt, 8));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_APOLOGIES_FOR_INSULT, sqlite3_column_int(pStmt, 9));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_APOLOGIES_FOR_TF, sqlite3_column_int(pStmt, 10));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_LEAVER, sqlite3_column_int(pStmt, 11));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_PLANTED_BOMB, sqlite3_column_int(pStmt, 12));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_DEFUSED_BOMB, sqlite3_column_int(pStmt, 13));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_KILLED_HOSTAGE, sqlite3_column_int(pStmt, 14));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_USED_HOSTAGE, sqlite3_column_int(pStmt, 15));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_RESCUED_HOSTAGE, sqlite3_column_int(pStmt, 16));
				actualRequest.addEventCount(EVENT_TYPE_PLAYER_ATTACKED_HOSTAGE, sqlite3_column_int(pStmt, 17));
				actualRequest.addKicks(sqlite3_column_int(pStmt, 18));
				actualRequest.addBans(sqlite3_column_int(pStmt, 19));
			}
			while(rc == SQLITE_ROW);
			sqlite3_finalize(pStmt);

			ServerWiper::getInstance()->addDatabaseStatsAnswer(actualRequest);
		}

		//Regular requests
        if(databaseManager->getRequestCount() > 0 && database)
        {
            DatabaseRequest actualRequest;
            databaseManager->popRequest(actualRequest);

            if(actualRequest.getType() == DB_REQUEST_CREATE)
            {
                if(sqlite3_exec(database,
                "CREATE TABLE IF NOT EXISTS users("
                                                    "steamid varchar(32) PRIMARY KEY NOT NULL"
                                                    ",lastconnect datetime DEFAULT CURRENT_TIMESTAMP"
                                                    ",lastip varchar(16) NOT NULL"
                                                    ",behav integer"
                                                    ",ban integer"
                                                    ",playtime integer"
                                                    ",data blob"
                                                    ");",
                                                    0, 0, &errorMessage) != SQLITE_OK)
                {
                    databaseManager->addMessage(shString("SQL error in the users table creation : ") + errorMessage);
                    sqlite3_free(errorMessage);
                }
                if(sqlite3_exec(database,
                "CREATE TABLE IF NOT EXISTS version(version varchar(50) PRIMARY KEY NOT NULL, data_version integer);",
                                                    0, 0, &errorMessage) != SQLITE_OK)
                {
                    databaseManager->addMessage(shString("SQL error in the version table creation : ") + errorMessage);
                    sqlite3_free(errorMessage);
                }
                else
                {
                    shString req = "INSERT INTO version (version, data_version) VALUES('";
                    req += itos(SERVER_WIPER_DATABASE_VERSION);
                    req += "','";
                    req += itos(SERVER_WIPER_PLAYER_DATA_VERSION);
                    req += "');";
                    sqlite3_exec(database, req.c_str(), 0, 0, 0);
                }
				if(sqlite3_exec(database,
					"CREATE TABLE IF NOT EXISTS stats("
					"steamid varchar(32) PRIMARY KEY NOT NULL"
					",connections integer"
					",ta integer"
					",tk integer"
					",tf integer"
					",insult integer"
					",greetings integer"
					",apo_ta integer"
					",apo_tk integer"
					",apo_insult integer"
					",apo_tf integer"
					",leave integer"
					",planted_bomb integer"
					",defused_bomb integer"
					",kill_hostage integer"
					",used_hostage integer"
					",rescued_hostage integer"
					",attacked_hostage integer"
					",kicks integer"
					",bans integer"
					");",
					0, 0, &errorMessage) != SQLITE_OK)
				{
					databaseManager->addMessage(shString("SQL error in the stats table creation : ") + errorMessage);
					sqlite3_free(errorMessage);
				}
            }
			else if(actualRequest.getType() == DB_REQUEST_CLEAN)
			{
				if(CVAR_INT(g_DatabaseCleanDelay) <= 0)
					continue;
				
				int minimumTime = time(NULL) - (CVAR_INT(g_DatabaseCleanDelay) * 24 * 3600);
				shString req = "SELECT steamid FROM users WHERE users.lastconnect<='";
				req += itos(minimumTime);
				req += "';";

				sqlite3_stmt *pStmt;
				int rc;

				rc = sqlite3_prepare_v2(database, req.c_str(), -1, &pStmt, 0);
				if(rc != SQLITE_OK)
				{
					databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + sqlite3_errmsg(database));
					continue;
				}

				shString sreq;

				do
				{
					rc = sqlite3_step(pStmt);
					if(rc != SQLITE_DONE && rc != SQLITE_ROW)
					{
						databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + sqlite3_errmsg(database));
						continue;
					}

					char *steamid = (char*)sqlite3_column_text(pStmt, 0);
					if(!steamid)
						continue;

					sreq = "DELETE FROM users WHERE steamid='";
					sreq += steamid;
					sreq += "';";
					sreq += "DELETE FROM stats WHERE steamid='";
					sreq += steamid;
					sreq += "';";

					if(sqlite3_exec(database, sreq.c_str(), 0, 0, &errorMessage) != SQLITE_OK)
					{
						databaseManager->addMessage(shString("SQL error in statement \"") + sreq + shString("\" : ") + errorMessage);
						sqlite3_free(errorMessage);
					}
				}
				while(rc == SQLITE_ROW);
				sqlite3_finalize(pStmt);
			}
            else if(actualRequest.getType() == DB_REQUEST_PLAYER_SAVE || actualRequest.getType() == DB_REQUEST_PLAYER_ADD)
            {
                Player pl = actualRequest.getPlayer();
                if(pl.getSteamID() == "STEAM_ID_PENDING")
                    continue;

				shString req;

				//Update stats
				if(actualRequest.getType() == DB_REQUEST_PLAYER_SAVE)
				{
					req = "SELECT count(*) FROM stats WHERE steamid='";
					req += pl.getSteamID();
					req += "';";

					if(sqlite3_exec(database, req.c_str(), callback, 0, &errorMessage) != SQLITE_OK)
					{
						databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + errorMessage);
						sqlite3_free(errorMessage);
					}

					if(atoi(databaseManager->getResult(0).c_str()) == 0)
					{
						req = "INSERT INTO stats ";
						req += "VALUES('";
						req += pl.getSteamID();
						req += "', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0'";
						req += ");";
					}
					else
					{
						req = "UPDATE stats SET ";
						req += "connections='";				req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_CONNECTED)->getOccured()) + "', ";
						req += "ta='";						req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_TA)->getOccured()) + "', ";
						req += "tk='";						req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_TK)->getOccured()) + "', ";
						req += "tf='";						req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_TEAM_FLASH)->getOccured()) + "', ";
						req += "insult='";					req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_INSULT)->getOccured()) + "', ";
						req += "greetings='";				req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_GREETINGS)->getOccured()) + "', ";
						req += "apo_ta='";					req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_APOLOGIES_FOR_TA)->getOccured()) + "', ";
						req += "apo_tk='";					req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_APOLOGIES_FOR_TK)->getOccured()) + "', ";
						req += "apo_insult='";				req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_APOLOGIES_FOR_INSULT)->getOccured()) + "', ";
						req += "apo_tf='";					req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_APOLOGIES_FOR_TF)->getOccured()) + "', ";
						req += "leave='";					req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_LEAVER)->getOccured()) + "', ";
						req += "planted_bomb='";			req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_PLANTED_BOMB)->getOccured()) + "', ";
						req += "defused_bomb='";			req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_DEFUSED_BOMB)->getOccured()) + "', ";
						req += "kill_hostage='";			req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_KILLED_HOSTAGE)->getOccured()) + "', ";
						req += "used_hostage='";			req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_USED_HOSTAGE)->getOccured()) + "', ";
						req += "rescued_hostage='";			req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_RESCUED_HOSTAGE)->getOccured()) + "', ";
						req += "attacked_hostage='";		req += itos(pl.getPlayerEvent(EVENT_TYPE_PLAYER_ATTACKED_HOSTAGE)->getOccured()) + "', ";
						req += "kicks='";					req += itos(pl.getKickCount()) + "', ";
						req += "bans='";					req += itos(pl.getBanCount()) + "'";
						req += " WHERE steamid='";
						req += pl.getSteamID();
						req += "';";
					}

					if(sqlite3_exec(database, req.c_str(), 0, 0, &errorMessage) != SQLITE_OK)
					{
						databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + errorMessage);
						sqlite3_free(errorMessage);
					}
				}

				//Update player data
                req = "SELECT count(*) FROM users WHERE steamid='";
                req += pl.getSteamID();
                req += "';";

                if(sqlite3_exec(database, req.c_str(), callback, 0, &errorMessage) != SQLITE_OK)
                {
                    databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + errorMessage);
                    sqlite3_free(errorMessage);
                }

                if(atoi(databaseManager->getResult(0).c_str()) == 0)
                {
                    req = "INSERT INTO users (steamid, lastconnect, lastip, behav, ban, playtime, data)";
                    req += "VALUES('";
                    req += pl.getSteamID();
                    req += "', '";
                    req += itos(pl.getConnectionTime());
                    req += "', '";
                    req += pl.getIP();
                    req += "', '";
                    req += itos(pl.getBehavPoints());
                    req += "', '";
                    req += itos(pl.getBanPoints());
                    req += "', '";
                    req += itos(pl.getPlayTime());
                    req += "', ?";
                    req += ");";
                }
                else
                {
                    req = "UPDATE users SET ";
                    req += "lastconnect='";
                    req += itos(pl.getConnectionTime());
                    req += "', lastip='";
                    req += pl.getIP();
                    if(actualRequest.getType() == DB_REQUEST_PLAYER_SAVE)
                    {
                        req += "', behav='";
                        req += itos(pl.getBehavPoints());
                        req += "', ban='";
                        req += itos(pl.getBanPoints());
                        req += "', playtime='";
                        req += itos(pl.getPlayTime());
                        req += "', data=?";
                    }
                    else
                        req += "'";

                    req += " WHERE steamid='";
                    req += pl.getSteamID();
                    req += "';";
                }

                if(atoi(databaseManager->getResult(0).c_str()) == 0 || actualRequest.getType() == DB_REQUEST_PLAYER_SAVE)
                {
                    sqlite3_stmt *pStmt;
                    int rc;
                    do
                    {
                        rc = sqlite3_prepare_v2(database, req.c_str(), -1, &pStmt, 0);
                        if(rc != SQLITE_OK)
                        {
                            databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + sqlite3_errmsg(database));
                            continue;
                        }
                        sqlite3_bind_blob(pStmt, 1, actualRequest.getData(), sizeof(PlayerDataV1), SQLITE_STATIC);
                        rc = sqlite3_step(pStmt);
                        if(rc != SQLITE_ROW)
                            continue;
                        rc = sqlite3_finalize(pStmt);
                    }
                    while(rc == SQLITE_SCHEMA);
                }
                else
                {
                    if(sqlite3_exec(database, req.c_str(), 0, 0, &errorMessage) != SQLITE_OK)
                    {
                        databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + errorMessage);
                        sqlite3_free(errorMessage);
                    }
                }
            }
            else if(actualRequest.getType() == DB_REQUEST_PLAYER_RESET)
            {
                Player pl = actualRequest.getPlayer();
                shString req = "UPDATE users SET behav='";
                req += itos(CVAR_INT(g_MaxBehavPoints));
                req += "',ban='";
                req += itos(CVAR_INT(g_MaxBanPoints));
                req += "' WHERE steamid='";
                req += pl.getSteamID();
                req += "';";

                if(sqlite3_exec(database, req.c_str(), 0, 0, &errorMessage) != SQLITE_OK)
                {
                    databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + errorMessage);
                    sqlite3_free(errorMessage);
                }
            }
            else if(actualRequest.getType() == DB_REQUEST_RESET)
            {
                shString req = "UPDATE users SET behav='";
                req += itos(CVAR_INT(g_MaxBehavPoints));
                req += "',ban='";
                req += itos(CVAR_INT(g_MaxBanPoints));
                req += "' WHERE 1;";

                if(sqlite3_exec(database, req.c_str(), 0, 0, &errorMessage) != SQLITE_OK)
                {
                    databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + errorMessage);
                    sqlite3_free(errorMessage);
                }
            }
            else if(actualRequest.getType() == DB_REQUEST_PLAYER_GET)
            {
                Player pl = actualRequest.getPlayer();

                shString req;
				sqlite3_stmt *pStmt;
				int rc;
				int eventOccured[EVENT_TYPE_MAX];
				memset(eventOccured, 0, sizeof(int) * EVENT_TYPE_MAX);
				int kicks;
				int bans;
				
				//Get stats
				req = "SELECT * FROM stats WHERE steamid='";
				req += pl.getSteamID();
				req += "';";

				rc = sqlite3_prepare_v2(database, req.c_str(), -1, &pStmt, 0);
				if(rc != SQLITE_OK)
				{
					databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + sqlite3_errmsg(database));
					continue;
				}

				rc = sqlite3_step(pStmt);
				if(rc != SQLITE_DONE && rc != SQLITE_ROW)
				{
					databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + sqlite3_errmsg(database));
					continue;
				}

				eventOccured[EVENT_TYPE_PLAYER_CONNECTED] = sqlite3_column_int(pStmt, 1);
				eventOccured[EVENT_TYPE_PLAYER_TA] = sqlite3_column_int(pStmt, 2);
				eventOccured[EVENT_TYPE_PLAYER_TK] = sqlite3_column_int(pStmt, 3);
				eventOccured[EVENT_TYPE_PLAYER_TEAM_FLASH] = sqlite3_column_int(pStmt, 4);
				eventOccured[EVENT_TYPE_PLAYER_INSULT] = sqlite3_column_int(pStmt, 5);
				eventOccured[EVENT_TYPE_PLAYER_GREETINGS] = sqlite3_column_int(pStmt, 6);
				eventOccured[EVENT_TYPE_PLAYER_APOLOGIES_FOR_TA] = sqlite3_column_int(pStmt, 7);
				eventOccured[EVENT_TYPE_PLAYER_APOLOGIES_FOR_TK] = sqlite3_column_int(pStmt, 8);
				eventOccured[EVENT_TYPE_PLAYER_APOLOGIES_FOR_INSULT] = sqlite3_column_int(pStmt, 9);
				eventOccured[EVENT_TYPE_PLAYER_APOLOGIES_FOR_TF] = sqlite3_column_int(pStmt, 10);
				eventOccured[EVENT_TYPE_PLAYER_LEAVER] = sqlite3_column_int(pStmt, 11);
				eventOccured[EVENT_TYPE_PLAYER_PLANTED_BOMB] = sqlite3_column_int(pStmt, 12);
				eventOccured[EVENT_TYPE_PLAYER_DEFUSED_BOMB] = sqlite3_column_int(pStmt, 13);
				eventOccured[EVENT_TYPE_PLAYER_KILLED_HOSTAGE] = sqlite3_column_int(pStmt, 14);
				eventOccured[EVENT_TYPE_PLAYER_USED_HOSTAGE] = sqlite3_column_int(pStmt, 15);
				eventOccured[EVENT_TYPE_PLAYER_RESCUED_HOSTAGE] = sqlite3_column_int(pStmt, 16);
				eventOccured[EVENT_TYPE_PLAYER_ATTACKED_HOSTAGE] = sqlite3_column_int(pStmt, 17);
				kicks = sqlite3_column_int(pStmt, 18);
				bans = sqlite3_column_int(pStmt, 19);
				sqlite3_finalize(pStmt);

				//Get player info
				req = "SELECT behav,ban,playtime,data FROM users WHERE steamid='";
                req += pl.getSteamID();
                req += "';";

                rc = sqlite3_prepare_v2(database, req.c_str(), -1, &pStmt, 0);
                if(rc != SQLITE_OK)
                {
                    databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + sqlite3_errmsg(database));
                    continue;
                }

                int behav, ban, playtime;
                PlayerDataV1 data;

                do
                {
                    rc = sqlite3_step(pStmt);
                    if(rc != SQLITE_DONE && rc != SQLITE_ROW)
                    {
                        databaseManager->addMessage(shString("SQL error in statement \"") + req + shString("\" : ") + sqlite3_errmsg(database));
                        continue;
                    }

                    behav = sqlite3_column_int(pStmt, 0);
                    ban = sqlite3_column_int(pStmt, 1);
                    playtime = sqlite3_column_int(pStmt, 2);
                    void *tmpData = (void *)sqlite3_column_blob(pStmt, 3);
                    int dataSize = sqlite3_column_bytes(pStmt, 3);
                    memcpy(&data, tmpData, dataSize);

                    if(dataSize > 0)
                        ServerWiper::getInstance()->addPlayerUpdate(pl.getSteamID(), behav, ban, playtime, data, eventOccured, kicks, bans);
                }
                while(rc == SQLITE_ROW);
                sqlite3_finalize(pStmt);
            }
        }
        #ifdef _WIN32
        Sleep(100);
        #else
        usleep(100000);
        #endif
	}

    if(database)
        sqlite3_close(database);

	pthread_exit(NULL);
	return NULL;
}

DatabaseManager::DatabaseManager()
{
    selectResults.clear();

    databaseToLoad = "";

    requests.init();

    #ifdef WIN32
	pthread_win32_process_attach_np();
	pthread_win32_thread_attach_np();
    #endif

    pthread_mutex_init(&runningMutex, NULL);
}

DatabaseManager::~DatabaseManager()
{
    setRunning(false);
	pthread_mutex_lock(&runningMutex);
	pthread_mutex_destroy(&runningMutex);

	requests.destroy();

    #ifdef WIN32
    pthread_win32_thread_detach_np();
    pthread_win32_process_detach_np();
    #endif

    selectResults.clear();
}

void DatabaseManager::loadDatabase(shString const& filename)
{
    char tdir[512];
    ServerWiper::getInstance()->getEngine()->GetGameDir(tdir,512);
    shString pathAndFilename = tdir;
    pathAndFilename += "/cfg/sw/database/";
    pathAndFilename += filename;

    databaseToLoad = pathAndFilename;

    setRunning(true);

	pthread_create(&databaseThread,NULL,&threadHandleDatabase,(void *)&databaseToLoad);
}

void DatabaseManager::setRunning(bool running)
{
    pthread_mutex_lock(&runningMutex);
    run = running;
    pthread_mutex_unlock(&runningMutex);
}

bool DatabaseManager::getRunning()
{
    pthread_mutex_lock(&runningMutex);
    bool isRunning = run;
    pthread_mutex_unlock(&runningMutex);
    return isRunning;
}

void DatabaseManager::clearResults()
{
    pthread_mutex_lock(&runningMutex);
    selectResults.clear();
    pthread_mutex_unlock(&runningMutex);
}

void DatabaseManager::addResult(shString const& newResult)
{
    pthread_mutex_lock(&runningMutex);
    selectResults.push_back(newResult);
    pthread_mutex_unlock(&runningMutex);
}

shString DatabaseManager::getResult(int resultId)
{
    shString result;
    pthread_mutex_lock(&runningMutex);
    result = selectResults[resultId];
    pthread_mutex_unlock(&runningMutex);
    return result;
}
