/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#ifndef _DATABASE_H
#define _DATABASE_H

#include "safeStack.h"
#include "serverWiper.h"
#include "player.h"

const int DB_REQUEST_PLAYER_ADD     =   0;
const int DB_REQUEST_CREATE         =   1;
const int DB_REQUEST_PLAYER_GET     =   2;
const int DB_REQUEST_PLAYER_SAVE    =   3;
const int DB_REQUEST_RESET          =   4;
const int DB_REQUEST_PLAYER_RESET   =   5;
const int DB_REQUEST_CLEAN			=	6;

const int DB_STATS_REQUEST_SERVER	=	0;
const int DB_STATS_REQUEST_PLAYER	=	1;

class DatabaseStatsRequest//Used for both request and answer
{
public:
	DatabaseStatsRequest()					{type = 0;}
	~DatabaseStatsRequest()					{}

    void setType(int const& newValue)        {type = newValue;}
    void setAdmin(int const& newValue)       {admin = newValue;}
    void setPlayer(shString const& newValue) {player = newValue;}
	void setEventCount(int const& eventType, int const& newValue)
	{
		if(eventType >= 0 && eventType < EVENT_TYPE_MAX)
			eventCount[eventType] = newValue;
	}
    void setKicks(int const& newValue)      {kicks = newValue;}
    void setBans(int const& newValue)       {bans = newValue;}

	void addEventCount(int const& eventType, int const& newValue)
	{
		if(eventType >= 0 && eventType < EVENT_TYPE_MAX)
			eventCount[eventType] += newValue;
	}
	void addKicks(int const& newValue)      {kicks += newValue;}
	void addBans(int const& newValue)       {bans += newValue;}

	int getType()                           {return type;}
	int getAdmin()                          {return admin;}
	shString getPlayer()                    {return player;}
	int getEventCount(int const& eventType) {return eventCount[eventType];}
	int getKicks()                          {return kicks;}
	int getBans()							{return bans;}

private:
	int type;
	int admin;
	shString player;
	int eventCount[EVENT_TYPE_MAX];
	int kicks;
	int bans;
};

class DatabaseRequest
{
public:
    DatabaseRequest()                       {type = 0;data.gotKicked = false;data.lastEvent = -1;data.lastEventTime = 0;}
    ~DatabaseRequest()                      {}

    void setType(int const& newType)               {type = newType;}
    void setPlayer(Player const& newPlayer)        {player = newPlayer;}
    void setData(PlayerDataV1 const& newData)          {data = newData;}

    int getType()                           {return type;}
    Player getPlayer()                      {return player;}
    PlayerDataV1 *getData()                 {return &data;}

private:
    int type;
    Player player;
    PlayerDataV1 data;
};

class DatabaseManager : public Singleton <DatabaseManager>
{
    friend class Singleton <DatabaseManager>;

public:
    DatabaseManager();
    ~DatabaseManager();

    void loadDatabase(shString const& filename);

    void setRunning(bool running);
    bool getRunning();

    void clearResults();
    void addResult(shString const& newResult);
    shString getResult(int resultId);

	void addNewRequest(DatabaseRequest const& newRequest){requests.push(newRequest);}
    int getRequestCount()                           {return requests.count();}
    void popRequest(DatabaseRequest & request)      {request = requests.pop();}

    void addNewStatsRequest(DatabaseStatsRequest const& newRequest){statsRequests.push(newRequest);}
	int getStatsRequestCount()                      {return statsRequests.count();}
	void popStatsRequest(DatabaseStatsRequest & request){request = statsRequests.pop();}

    int getMessagesCount()                          {return messages.count();}
    shString popMessage()                           {return messages.pop();}
    void addMessage(shString const& newMessage)     {messages.push(newMessage);}

private:
    pthread_t databaseThread;
    bool run;
    shString databaseToLoad;
    pthread_mutex_t runningMutex;
    safeStack<DatabaseRequest> requests;
    safeStack<DatabaseStatsRequest> statsRequests;
    shVector<shString> selectResults;
    safeStack<shString> messages;
};

void* threadHandleDatabase(void*);

#endif //_DATABASE_H


