/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#include "help.h"
#include "serverWiper.h"
#include "cvars.h"

HelpManager::HelpManager()
{
    srand(time(NULL));
}

HelpManager::~HelpManager()
{
    helpMessages.clear();
}

void HelpManager::loadHelpMessages()
{
    helpMessages.clear();

    shString pathAndFilename = "cfg/sw/text/help";

    if(strcmp(g_Language.GetString(), "") != 0 && strcmp(g_Language.GetString(), "en") != 0)
    {
        pathAndFilename += "_";
        pathAndFilename += g_Language.GetString();
    }
    pathAndFilename += ".txt";

    if(!ServerWiper::getInstance()->getFilesystem()->FileExists(pathAndFilename.c_str(), "MOD"))
    {
        META_LOG(g_PLAPI,"Could not open the help file : %s", pathAndFilename.c_str());
        return;
    }

    FileHandle_t file = ServerWiper::getInstance()->getFilesystem()->Open(pathAndFilename.c_str(), "r", "MOD");

    int helpMessagesCount=0;
    for(;!ServerWiper::getInstance()->getFilesystem()->EndOfFile(file);helpMessagesCount++)
    {
        char text[1024];
        ServerWiper::getInstance()->getFilesystem()->ReadLine(text, sizeof(text), file);
        int text_size = Q_strlen(text);
        text[text_size-1] = '\0';
        if(strcmp(text, "") != 0)
            addHelpMessage(text);
    }

	ServerWiper::getInstance()->getFilesystem()->Close(file);

    META_CONPRINTF("Read %d help messages\n", helpMessagesCount);
}

bool HelpManager::getRandomHelpMessage(shString & message)
{
    if(helpMessages.size() == 0)
        return false;

	int messageId = rand() % helpMessages.size();

	if(messageId < 0)
        messageId = 0;
    if(messageId >= helpMessages.size())
        messageId = helpMessages.size() - 1;

    shString basicMessage = helpMessages[messageId];
    shString finalMessage = "";
    int currentPosition = 0;
    shString buffer = "";

    //First look if we have a prerequisite
    if(basicMessage[0] == '!')
    {
        for(int i=1;i<basicMessage.size();i++)
        {
            if(basicMessage[i] == ' ')
            {
                ConVar *prerequisite = ServerWiper::getInstance()->getConVar(buffer);
                if(!prerequisite)
                {
                    //prerequisite not found, aborting
                    META_LOG(g_PLAPI,"Error in help message file, Prerequisite not found %s", buffer.c_str());
                    return false;
                }
                if(prerequisite && !atoi(prerequisite->GetString()))
                {
                    //prerequisite failed, aborting
                    return false;
                }
                //Ok success, now save the current position
                currentPosition = i+1;
                break;
            }
            else
                buffer += basicMessage[i];
        }
    }

    buffer = "";
    bool isInCVAR = false;

    //Then look for the help message himself
    for(int i=currentPosition;i<basicMessage.size();i++)
    {
        if(basicMessage[i] == '#')//CVAR declaration
        {
            if(isInCVAR)
            {
                META_LOG(g_PLAPI,"Error in help message file, a '#' is in a CVAR name !");
                return false;
            }
            else
            {
                isInCVAR = true;
                buffer = "";
            }
        }
        else if(basicMessage[i] == ' ')//CVAR end / or a simple space if not in CVAR
        {
            if(isInCVAR)
            {
                ConVar *cvar = ServerWiper::getInstance()->getConVar(buffer);
                if(cvar)
                {
                    finalMessage += cvar->GetString();
                    finalMessage += " ";
                }
                else
                {
                    META_LOG(g_PLAPI,"Error in help message file, unable to find cvar named %s", buffer.c_str());
                    return false;
                }
                isInCVAR = false;
            }
            else
            {
                finalMessage += basicMessage[i];
            }
        }
        else
        {
            if(isInCVAR)
                buffer += basicMessage[i];
            else
                finalMessage += basicMessage[i];
        }
    }

    message = finalMessage;

    return true;
}

void HelpManager::addHelpMessage(shString const& helpMessage)
{
    helpMessages.push_back(helpMessage);
}


