/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#include "insults.h"
#include "serverWiper.h"
#include "cvars.h"

#include <stdlib.h>

InsultManager::InsultManager()
{
}

InsultManager::~InsultManager()
{
    reset();
}

void InsultManager::reset()
{
    insults.clear();
}

void InsultManager::loadInsults(shString const& filename, int const& level)
{
    shString pathAndFilename = "cfg/sw/text/";
    pathAndFilename += filename;
    if(!ServerWiper::getInstance()->getFilesystem()->FileExists(pathAndFilename.c_str(), "MOD"))
    {
        META_LOG(g_PLAPI,"Could not open the insult file : %s", filename.c_str());
        return;
    }

    FileHandle_t file = ServerWiper::getInstance()->getFilesystem()->Open(pathAndFilename.c_str(), "r", "MOD");

    int insultCount=0;
    for(;!ServerWiper::getInstance()->getFilesystem()->EndOfFile(file);insultCount++)
    {
        char text[1024];
        ServerWiper::getInstance()->getFilesystem()->ReadLine(text, sizeof(text), file);
        int text_size = Q_strlen(text);
        for(int i=0;i<text_size;i++)
        {
            if(text[i]=='/')
            {
                text[i]='\0';
                i=text_size;
            }
        }
        text[text_size-1] = '\0';
		for(int i=text_size-1;i>0;i--)
		{
			if(text[i] == '\n' || text[i] == '\r' || text[i] == '\0')
				text[i] = '\0';
			else
				break;
		}
        addInsult(getLowerString(text), level);
    }

	ServerWiper::getInstance()->getFilesystem()->Close(file);

    META_CONPRINTF("Read %d insults of level %d\n", insultCount, level);
}

int InsultManager::getInsultLevel(shString const& text, shString &insult)
{
    int maxLevel = 0;

    //Make the shString lowercase
    shString newText = getLowerString(text);

    //Delete all the invalid characters
    shString invalidCharacters = g_InvalidCharacters.GetString();
    for(int i=0;i<invalidCharacters.size();i++)
    {
        newText = deleteFromString(newText, invalidCharacters[i]);
    }

    //Replace double spaces with simple spaces
    shString resultingString = "";
    for(int i = 0;i<newText.size();i++)
    {
        if(!(newText[i] == ' ' && i < newText.size() - 1 && newText[i+1] == ' '))
            resultingString += newText[i];
    }
    newText = resultingString;

    //Put spaces before and after
    shString tmp = " ";
    tmp += newText;
    tmp += " ";
    newText = tmp;

	//Remove all the spaces
	shString textWithoutSpaces = deleteFromString(newText, ' ');

	shString currentInsult, trimedInsult;

    //Search for insults
    for(int i = 0;i<insults.size();i++)
    {
		currentInsult = insults[i].getInsult();
        trimedInsult = currentInsult;
        trimedInsult.trim();

		if(isPatternInString(newText, currentInsult) || newText == trimedInsult || textWithoutSpaces == trimedInsult)
        {
            if(maxLevel < insults[i].getLevel())
            {
                maxLevel = insults[i].getLevel();
                insult = currentInsult;
            }
        }
    }

    return maxLevel;
}

void InsultManager::addInsult(shString const& insult, int const& level)
{
    Insult newInsult;
    newInsult.setInsult(insult);
    newInsult.setLevel(level);
	insults.push_back(newInsult);

    //Make the spaced version only if it's greater than 2 chars and the first char is not a space
    if(insult.size() > 2 && insult[0] != ' ')
    {
        shString spacedInsult = getSpacedString(insult);
        newInsult.setInsult(spacedInsult);
        newInsult.setLevel(level);
        insults.push_back(newInsult);
    }
}

