/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#include "language.h"

LanguageManager::LanguageManager()
{
}

LanguageManager::~LanguageManager()
{
    baseStrings.clear();
    translations.clear();
}

bool LanguageManager::loadLanguageFile(shString const& filename)
{
    if(filename == "en")//Default
        return true;

    baseStrings.clear();
    translations.clear();
    stringCount = 0;

    shString pathAndFilename = "cfg/sw/language/";
    pathAndFilename += filename;
    pathAndFilename += ".txt";
    if(!ServerWiper::getInstance()->getFilesystem()->FileExists(pathAndFilename.c_str(), "MOD"))
    {
        META_LOG(g_PLAPI,"Could not open the language file : %s", filename.c_str());
        return false;
    }

    FileHandle_t file = ServerWiper::getInstance()->getFilesystem()->Open(pathAndFilename.c_str(), "r", "MOD");

    for(int j=0;!ServerWiper::getInstance()->getFilesystem()->EndOfFile(file);j++)
    {
        char text[2048];
        ServerWiper::getInstance()->getFilesystem()->ReadLine(text, sizeof(text), file);
        int text_size = Q_strlen(text);
        shString baseText = "";
        shString translation = "";
        bool isBaseText = true;
        for(int i=0;i<text_size;i++)
        {
            if(isBaseText)
            {
                if(text[i]=='	')
                {
                    isBaseText = false;
                }
                else if(i < text_size-1 && text[i]=='\\' && text[i+1]=='n')
                {
                    baseText += "\n";
                    i++;
                }
                else
                {
                    baseText += text[i];
                }
            }
            else if(i<text_size-1)
            {
                if(i < text_size-1 && text[i]=='\\' && text[i+1]=='n')
                {
                    translation += "\n";
                    i++;
                }
                else
                {
                    translation += text[i];
                }
            }
        }

        if(baseText == "" || translation == "")
        {
            continue;
        }

		baseText.trim();
		translation.trim();

        baseStrings[stringCount] = baseText;
        translations[stringCount] = translation;
        stringCount++;

        baseText = "";
        translation = "";
    }

	ServerWiper::getInstance()->getFilesystem()->Close(file);

	META_CONPRINTF("Read %d language strings\n", stringCount/2);

    return true;
}

shString LanguageManager::translatedString(char *str, ...)
{
    char buffer[4096];
    va_list args;
    shString baseString = str;

    for(shHash<int, shString>::iterator i = baseStrings.begin();i!=baseStrings.end();i++)
    {
        if((*i).val == baseString)
        {
            baseString = translations[(*i).key];
            baseString.trim();
            break;
        }
    }

    va_start(args, str);
    vsprintf(buffer, baseString.c_str(), args);
    va_end(args);
    return shString(buffer);
}

