/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#include "menu.h"
#include "language.h"
#define GAME_DLL 1
#include "cbase.h"
#include "MRecipientFilter.h"
#include "bitbuf.h"
#include "cvars.h"

const int MAX_MESSAGE_DATA =    240;

MenuManager::MenuManager()
{
}

MenuManager::~MenuManager()
{
}

void MenuManager::showAdminMenu(int const& player)
{
    Player *adminPlayer = ServerWiper::getInstance()->getPlayer(player);
    shVector<shString>options;
    //options.push_back(_("Player admin")); //TODO
    options.push_back(_("Server admin"));
    showMenu(_("ServerWiper Admin"), player, options, false, 0, 0, false, false, -1);

    adminPlayer->setActualMenu(MENU_ADMIN);
}

void MenuManager::showTopMenu(int const& player, bool behav)
{
    Player *adminPlayer = ServerWiper::getInstance()->getPlayer(player);
    shVector<shString>options;

    //TODO


    if(behav)
    {
        showMenu(_("Top 10 (behaviour points)"), player, options, false, 0, 0, false, false, -1);
        adminPlayer->setActualMenu(MENU_BEHAV_TOP);
    }
    else
    {
        showMenu(_("Top 10 (ban points)"), player, options, false, 0, 0, false, false, -1);
        adminPlayer->setActualMenu(MENU_BAN_TOP);
    }
}

void MenuManager::showPlayerAdminMenu(int const& player)
{
    Player *playerAdmin = ServerWiper::getInstance()->getPlayer(player);

    if(!playerAdmin->isAdmin())
        return;

    shVector<shString>options;
    options.push_back(_("Give a sanction to a player"));
    options.push_back(_("Reward a player"));
    showMenu(_("ServerWiper Admin"), player, options, false, 0, 0, true, false, -1);

    playerAdmin->setActualMenu(MENU_PLAYER_ADMIN);
}

void MenuManager::showServerAdminMenu(int const& player)
{
    Player *playerAdmin = ServerWiper::getInstance()->getPlayer(player);

    if(!playerAdmin->isAdmin())
        return;

    shVector<shString>options;
    if(CVAR_BOOL(g_ServerWiperEnabled))
        options.push_back(_("Deactivate ServerWiper"));
    else
        options.push_back(_("Activate ServerWiper"));
    options.push_back(_("Reset all points"));
    options.push_back(_("Reload all text data"));
    showMenu(_("ServerWiper Admin"), player, options, false, 0, 0, true, false, -1);

    playerAdmin->setActualMenu(MENU_SERVER_ADMIN);
}

void MenuManager::showForgiveMenu(int const& player, int const& killer, int const& time_left)
{
    Player *playerKilled = ServerWiper::getInstance()->getPlayer(player);

    if(!playerKilled)
        return;

    shVector<shString>options;
    options.push_back(_("Punish"));
    options.push_back(_("Forgive"));

    Player *myKiller = ServerWiper::getInstance()->getPlayer(killer);
    shString killerName = "?";
    if(myKiller)
    {
        killerName = cleanString(myKiller->getName(), MAX_NAME_LENGTH);
    }

    shString title = _("Forgive");
    title += " ";
    title += killerName;
    title += " ?";
    showMenu(title, player, options, false, 0, 0, false, false, time_left);

    playerKilled->setActualMenu(MENU_FORGIVE);
    playerKilled->setActualMenuTimeout(time(NULL) + time_left);
	playerKilled->setLastTKKillerSteamID(myKiller->getSteamID());
}

void MenuManager::showSettingsMenu(int const& player)
{
    //TODO
}

void MenuManager::showPlayerListMenu(int const& player, shString const& title)
{
    //TODO
}

void MenuManager::showMenu(shString const& title, int const& player, shVector<shString> const& options, bool showPage, int const& page, int const& maxpage, bool showPrevious, bool showNext, int const& timeout)
{
    //Send menuselect 10 client command to close any opened menu (Mani Admin, etc.)
    edict_t * entity = ServerWiper::getInstance()->getEngine()->PEntityOfEntIndex(player);
    Player *menuPlayer = ServerWiper::getInstance()->getPlayer(player);
    if(menuPlayer && menuPlayer->getActualMenu() != MENU_FORGIVE)
        ServerWiper::getInstance()->getPlugin()->getHelper()->ClientCommand(entity,"menuselect 10");

    shString menuString = title;
    menuString += "\n";

    short enabledButtons = 0;

    if(showPage)
    {
        menuString += _("Page");
        menuString += " ";
        menuString += itos(page);
        menuString += "/";
        menuString += itos(maxpage);
        menuString += "\n";
    }

    for(int i=0;i<options.size() && i<7;i++)
    {
        menuString += "->";
        menuString += itos(i+1);
        menuString += ". ";
        menuString += options[i];
        menuString += "\n";

        enabledButtons |= (1<<i);
    }

    if(showPrevious)
    {
        menuString += "->8. ";
        menuString += _("Back");
        menuString += "\n";

        enabledButtons |= (1<<7);
    }

    if(showNext)
    {
        menuString += "->9. ";
        menuString += _("Next");
        menuString += "\n";

        enabledButtons |= (1<<8);
    }

    menuString += "0. ";
    menuString += _("Exit");
    enabledButtons |= (1<<9);

    MRecipientFilter filter;
    filter.AddRecipient(player);

    while(true)//Cut the menu message into pieces if it's too big
    {
        bool lastMessage = false;
        shString data = "";
        if(menuString.size() > MAX_MESSAGE_DATA)//Maximum data that can be sent at a time
            data = cutString(menuString, MAX_MESSAGE_DATA);
        else
        {
            lastMessage = true;
            data = menuString;
        }

        bf_write *pWrite = ServerWiper::getInstance()->getEngine()->UserMessageBegin(&filter,10);
        pWrite->WriteWord(enabledButtons);
        pWrite->WriteChar(timeout);
        pWrite->WriteByte(!lastMessage);
        pWrite->WriteString(data.c_str());
        ServerWiper::getInstance()->getEngine()->MessageEnd();

        if(lastMessage)
            break;
    }
}

void MenuManager::onPlayerMenuSelect(int playerindex, int menu)
{
    Player *player = ServerWiper::getInstance()->getPlayer(playerindex);

    if(!player)
        return;

    if(player->getActualMenu() == -1)
        return;

    //Play a nice sound
    edict_t * entity = ServerWiper::getInstance()->getEngine()->PEntityOfEntIndex(playerindex);
    IServerUnknown * pServerUnknown = entity->GetUnknown();
    CBaseEntity * pEntity = pServerUnknown ->GetBaseEntity();
    MRecipientFilter filter;
    filter.AddRecipient(playerindex);
    Vector pos = pEntity->GetCollideable()->GetCollisionOrigin();
    ServerWiper::getInstance()->getPlugin()->getEngineSound()->EmitSound((IRecipientFilter &)filter, playerindex, CHAN_AUTO, "items/flashlight1.wav", 0.7, ATTN_NORM, 0, PITCH_NORM, &pos, 0, 0, true, 0, playerindex);

    //Get the current menu
    if(player->getActualMenu() == MENU_FORGIVE)
    {
        if(menu == 10 || menu == 2)//Close or Forgive
        {
            Event myNewEvent;
            myNewEvent.setType(EVENT_TYPE_MENU_FORGIVE);
            myNewEvent.setAuthor(player->getLastTKKiller());
            myNewEvent.setVictim(playerindex);
			myNewEvent.setText(player->getLastTKKillerSteamID());
            ServerWiper::getInstance()->newEvent(myNewEvent);
            player->setActualMenu(-1);
        }
        else if(menu == 1)//Punish
        {
            Event myNewEvent;
            myNewEvent.setType(EVENT_TYPE_MENU_PUNISH);
            myNewEvent.setAuthor(player->getLastTKKiller());
            myNewEvent.setVictim(playerindex);
			myNewEvent.setText(player->getLastTKKillerSteamID());
            ServerWiper::getInstance()->newEvent(myNewEvent);
            player->setActualMenu(-1);
        }
    }
    else if(player->getActualMenu() == MENU_ADMIN)
    {
        if(menu == 10)//Close
        {
            player->setActualMenu(-1);
        }
        /*else if(menu == 1)//Player admin //TODO
        {
            showPlayerAdminMenu(playerindex);
        }*/
        else if(menu == 1)//Server admin
        {
            showServerAdminMenu(playerindex);
        }
    }
    else if(player->getActualMenu() == MENU_PLAYER_ADMIN)
    {
        if(menu == 10)//Close
        {
            player->setActualMenu(-1);
        }
        else if(menu == 8)//Previous
        {
            showAdminMenu(playerindex);
        }
        else if(menu == 1)//Sanction
        {
            //TODO
        }
        else if(menu == 2)//Reward
        {
            //TODO
        }
    }
    else if(player->getActualMenu() == MENU_SERVER_ADMIN)
    {
        if(menu == 10)//Close
        {
            player->setActualMenu(-1);
        }
        else if(menu == 8)//Previous
        {
            showAdminMenu(playerindex);
        }
        else if(menu == 1)//De/activate SW
        {
            g_ServerWiperEnabled.SetValue(!CVAR_BOOL(g_ServerWiperEnabled));
            if(CVAR_INT(g_DebugLevel))
            {
                META_LOG(g_PLAPI, "ServerWiper has been de/activated by admin %s (%s)", player->getName().c_str(), player->getSteamID().c_str());
            }
            showServerAdminMenu(playerindex);
        }
        else if(menu == 2)//Reset all
        {
            ServerWiper::getInstance()->resetAllPlayers();
            ServerWiper::getInstance()->sendChatText(playerindex, _("The player database has been reset"));
            if(CVAR_INT(g_DebugLevel))
            {
                META_LOG(g_PLAPI, "The player database has been reset by admin %s (%s)", player->getName().c_str(), player->getSteamID().c_str());
            }
            showServerAdminMenu(playerindex);
        }
        else if(menu == 3)//Reload data
        {
            ServerWiper::getInstance()->reload();
            ServerWiper::getInstance()->sendChatText(playerindex, _("Text and language data reloaded"));
            if(CVAR_INT(g_DebugLevel))
            {
                META_LOG(g_PLAPI, "Text and language data reloaded by admin %s (%s)", player->getName().c_str(), player->getSteamID().c_str());
            }
            showServerAdminMenu(playerindex);
        }
    }
    //TODO : Other menu's
}
