/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#ifndef _INCLUDE_PLAYER_H
#define _INCLUDE_PLAYER_H

#include <ISmmPlugin.h>
#include <inetchannelinfo.h>
#include "util.h"
#include "event.h"
#include "server_wiper_mm.h"

const int TEAM_UNASSIGNED              =       0;
const int TEAM_SPECTATOR               =       1;
const int TEAM_TERRORIST               =       2;
const int TEAM_COUNTER_TERRORIST       =       3;

enum PlayerEventStatus
{
    EVENT_STATUS_NEVER_OCCURED = 0,
    EVENT_STATUS_OCCURED,
    EVENT_STATUS_APOLOGIZED,
};

struct PlayerDataV1
{
    int lastEvent;
    bool gotKicked;
    time_t lastEventTime;
};

class PlayerEvent
{
public:
    PlayerEvent();

    void setStatus(PlayerEventStatus const& newValue)   {status = newValue;}
    void setLastOccured(time_t const& newTime)          {lastOccured = newTime;}
    void setBanPointsLost(int const& newValue)          {banPointsLost = newValue;}
    void setBehavPointsLost(int const& newValue)        {behavPointLost = newValue;}
    void setCount(int const& newValue)                  {count = newValue;}
	void setOccured(int const& newValue)				{occured = newValue;}

    PlayerEventStatus getStatus()                       {return status;}
    time_t getLastOccured()                             {return lastOccured;}
    int getBanPointLost()                               {return banPointsLost;}
    int getBehavPointLost()                             {return behavPointLost;}
    int getCount()                                      {return count;}
	int getOccured()									{return occured;}

private:
    PlayerEventStatus status;
    time_t lastOccured;
    int banPointsLost;
    int behavPointLost;
    int count;//Specific stuff, for tf it's the amount of flashed teammates, for a ta the amount of damage
	int occured;
};

class Player
{
public:
    Player(int const& newIndex, edict_t *newEdict);
    Player();
    ~Player();

    void setName(shString const& newValue)              {name = newValue;}
    void setIP(shString const& newValue)                {ip = newValue;}
    void setBehavPoints(int const& newValue)            {behavPoints = newValue;}
    void setBanPoints(int const& newValue)              {banPoints = newValue;}
    void setPlayTime(int const& newTime)                {totalPlaytimeBeforeBanPtGain = newTime;}
    void setSteamID(shString const& newValue)           {steamID = newValue;}
    void setHasSteamID(bool newValue)                   {hasSteamID = newValue;}
    void setTeam(int const& newValue)                   {team = newValue;}
    void setUserID(int const& newValue)                 {userid = newValue;}
    void setUsedHostageCount(int const& newValue)       {usedHostageCount = newValue;}
    void setConnectionTime(time_t const& newTime)       {connectionTime = newTime;}
    void setHadWelcomeMessage(bool newValue)            {hadWelcomeMessage = newValue;}
    void setActualMenu(int const& newMenu)              {actualMenu = newMenu;}
    void setActualPage(int const& newPage)              {actualPage = newPage;}
    void setActualMenuTimeout(time_t const& newTime)    {actualMenuTimeout = newTime;}
    void setLastTKKiller(int const& newValue)           {lastTKKiller = newValue;}
	void setLastTKKillerSteamID(shString const& newValue){lastTKKillerSteamID = newValue;}
    void setAdmin(bool newAdmin)                        {admin = newAdmin;}
    void setData(PlayerDataV1 const& newData)           {data = newData;}
	void setKickCount(int const& newValue)				{kickCount = newValue;}
	void setBanCount(int const& newValue)				{banCount = newValue;}

    shString getName()                                  {return name;}
    shString getSteamID()                               {return steamID;}
    int getBehavPoints()                                {return behavPoints;}
    int getBanPoints()                                  {return banPoints;}
    int getPlayTime()                                   {return totalPlaytimeBeforeBanPtGain;}
    edict_t *getEdict()                                 {return edict;}
    int getIndex()                                      {return index;}
    shString getIP()                                    {return ip;}
    bool getHasSteamID()                                {return hasSteamID;}
    int getTeam()                                       {return team;}
    int getUserID()                                     {return userid;}
    int getUsedHostageCount()                           {return usedHostageCount;}
    time_t getConnectionTime()                          {return connectionTime;}
    bool getHadWelcomeMessage()                         {return hadWelcomeMessage;}
    int getActualMenu()                                 {return actualMenu;}
    int getActualPage()                                 {return actualPage;}
    time_t getActualMenuTimeout()                       {return actualMenuTimeout;}
    int getLastTKKiller()                               {return lastTKKiller;}
	shString getLastTKKillerSteamID()					{return lastTKKillerSteamID;}
    bool isAdmin()                                      {return admin;}
    PlayerEvent *getPlayerEvent(int const& eventType)   {if(eventType >= 0 && eventType < EVENT_TYPE_MAX) return &eventStatus[eventType]; return 0;}
    PlayerDataV1 *getData()                             {return &data;}
	int getKickCount()									{return kickCount;}
	int getBanCount()									{return banCount;}

private:
    bool hasSteamID;
    edict_t *edict;
    int index;
    shString name;
    shString steamID;
    shString ip;
    int team;
    int userid;
    int usedHostageCount;
    PlayerEvent eventStatus[EVENT_TYPE_MAX];
    time_t connectionTime;
    bool hadWelcomeMessage;
    int actualMenu;
    int actualPage;
    time_t actualMenuTimeout;
    int lastTKKiller;
	shString lastTKKillerSteamID;
    bool admin;
    int behavPoints;
    int banPoints;
    int totalPlaytimeBeforeBanPtGain;
    PlayerDataV1 data;
	int kickCount;
	int banCount;
};

#endif //_INCLUDE_PLAYER_H


