/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

/*=================Last Connected======================
*	Copyright (C) 2005-2006 Brian "devicenull" Rak
*   Modified by Jonathan "Jmgr" Mercier-Ganady
*	License: zlib/libpng
*	$Id: safeStack.h 23 2006-06-16 18:33:22Z  $
=====================================================*/

#ifndef _SAFESTACK_H
#define _SAFESTACK_H

#include "util.h"
#include "pthread.h"

template <class T> class safeStack
{
public:
    safeStack()
    {
        #ifdef _WIN32
        mSafe = PTHREAD_MUTEX_INITIALIZER;
        #endif
		init(); // Init here or big badaboom on Debian but not on Ubuntu and Windows... I love computer science.
    }

	void init()
	{
		pthread_mutex_init(&mSafe,NULL);
	}
	void destroy()
	{
		pthread_mutex_lock(&mSafe);
		pthread_mutex_destroy(&mSafe);
	}
	void push(T newItem)
	{
		pthread_mutex_lock(&mSafe);
		sStack.push(newItem);
		pthread_mutex_unlock(&mSafe);
	}
	T& pop()
	{
		pthread_mutex_lock(&mSafe);
		T &temp = sStack.front();
		sStack.pop();
		pthread_mutex_unlock(&mSafe);
		return temp;
	}
	int count()
	{
		pthread_mutex_lock(&mSafe);
		int temp = sStack.size();
		pthread_mutex_unlock(&mSafe);
		return temp;
	}

private:
	shStack<T> sStack;
	pthread_mutex_t mSafe;
};

#endif // _SAFESTACK_H
