/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#include "serverWiper.h"
#include "datamap.h"
#define GAME_DLL 1
#include "cbase.h"
#include "MRecipientFilter.h"
#include "bitbuf.h"
#include "insults.h"
#include "greetings.h"
#include "apologies.h"
#include "url.h"
#include "cvars.h"
#include <time.h>
#include "language.h"
#include "database.h"
#include "menu.h"
#include "admin.h"
#include "help.h"
#include "team.h"

#ifdef _WIN32
#pragma warning(disable: 4018)
#pragma warning(disable: 4244)
#pragma warning(disable: 4005)
#endif

DummyPlugin dummyPluginInstance;

ServerWiper::ServerWiper()
{
    plugin = 0;
    nextPendingCheck = 0.;
    lastAdvert = 0;
    lastHelpMessage = 0;
    players.clear();
    hasUpdates = 0;
    playerUpdates.init();
    lastRegistration = 0;
    lastTerroristIndex = 0;
    lastCounterTerroristIndex = 0;
	waitingForDatabaseStatsAnswer = 0;
}

ServerWiper::~ServerWiper()
{
    playerUpdates.destroy();
    for(int i=0;i<players.GetBuckets();i++)
    {
        if(getPlayer(i))
        {
            delete players[i];
            players[i] = 0;
        }
    }
    players.clear();

    InsultManager::kill();
    GreetingManager::kill();
    ApologyManager::kill();
    LanguageManager::kill();
    DatabaseManager::kill();
    URLManager::kill();
    MenuManager::kill();
    AdminManager::kill();
    HelpManager::kill();
	TeamManager::kill();
}

void ServerWiper::setPluginInstance(ServerWiperPlugin *plugin)
{
    if(!plugin)
    {
        META_LOG(g_PLAPI,"Could not init the ServerWiper instance");
        return;
    }
    this->plugin = plugin;
}

IVEngineServer *ServerWiper::getEngine()
{
    if(!plugin)
    {
        META_LOG(g_PLAPI,"Please call setPluginInstance first");
        return 0;
    }

    return plugin->getEngine();
}

void ServerWiper::playerGotPoints(Event & event, shString const& behavPoints, shString const& banPoints, shString const& health, shString const& money, shString const& reason, int const& coef)
{
    float final_coef = static_cast<float>(coef);

    if( event.getType() == EVENT_TYPE_PLAYER_TA || event.getType() == EVENT_TYPE_PLAYER_TK ||
        event.getType() == EVENT_TYPE_PLAYER_TEAM_FLASH || event.getType() == EVENT_TYPE_PLAYER_ATTACKED_HOSTAGE ||
        event.getType() == EVENT_TYPE_PLAYER_INSULT || event.getType() == EVENT_TYPE_PLAYER_KILLED_HOSTAGE )
    {//Bad events
        if(CVAR_BOOL(g_PointLossRatio) && CVAR_INT(g_PointLossRatioDelay) > 0)
        {
            Player *player = getPlayer(event.getAuthor());
            if(player)
            {
                time_t lastBadThing = getMax(player->getPlayerEvent(EVENT_TYPE_PLAYER_INSULT)->getLastOccured(), player->getPlayerEvent(EVENT_TYPE_PLAYER_TA)->getLastOccured(),
                player->getPlayerEvent(EVENT_TYPE_PLAYER_TK)->getLastOccured(), player->getPlayerEvent(EVENT_TYPE_PLAYER_TEAM_FLASH)->getLastOccured(),
                player->getPlayerEvent(EVENT_TYPE_PLAYER_ATTACKED_HOSTAGE)->getLastOccured(), player->getPlayerEvent(EVENT_TYPE_PLAYER_KILLED_HOSTAGE)->getLastOccured());

                if(time(NULL) - lastBadThing <= CVAR_INT(g_PointLossRatioDelay))
                {
                    time_t timeSinceLastBadThing = time(NULL) - lastBadThing;
                    float ratio_coef = 1.0f - static_cast<float>(timeSinceLastBadThing) / static_cast<float>(CVAR_INT(g_PointLossRatioDelay));
                    float ratio = static_cast<float>(CVAR_INT(g_PointLossRatioMax) - 1) * ratio_coef + 1.0f;
                    final_coef *= ratio;
                }

            }
        }

        if(CVAR_BOOL(g_PunishAtRoundStart) && CVAR_INT(g_PunishAtRoundStartDelay) > 0)
        {
            if(time(NULL) - lastRoundStart <= CVAR_INT(g_PunishAtRoundStartDelay))
            {
                final_coef *= static_cast<float>(CVAR_INT(g_PunishAtRoundStartRatio));
            }
        }
    }

    shString interBehavPoints = behavPoints;
    shString interBanPoints = banPoints;
    shString interHealth = health;
    shString interMoney = money;

    int finalBehavPoints = 0;
    int finalBanPoints = 0;
    int finalHealth = 0;
    int finalMoney = 0;

    Player *player = getPlayer(event.getAuthor());
    int prevBehavPoints = player->getBehavPoints();
    int prevBanPoints = player->getBanPoints();
    int prevHealth = getPlayerHealth(event.getAuthor());
    int prevMoney = getPlayerMoney(event.getAuthor());

    bool isPercentBehav = false;
    int percentBehav = 0;
    bool isPercentBan = false;
    int percentBan = 0;
    bool isPercentHealth = false;
    int percentHealth = 0;
    bool isPercentMoney = false;
    int percentMoney = 0;

    if(event.getType() == EVENT_TYPE_PLAYER_INSULT ||
        event.getType() == EVENT_TYPE_PLAYER_APOLOGIES_FOR_INSULT ||
        event.getType() == EVENT_TYPE_PLAYER_TA ||
        event.getType() == EVENT_TYPE_PLAYER_APOLOGIES_FOR_TA)
    {
        int level = 0;

        switch(event.getType())
        {
            case EVENT_TYPE_PLAYER_INSULT:
                level = event.getIntValue();
                break;
            case EVENT_TYPE_PLAYER_APOLOGIES_FOR_INSULT:
                level = getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_INSULT)->getCount();
                break;
            case EVENT_TYPE_PLAYER_TA:
                level = event.getIntValue();
                break;
            case EVENT_TYPE_PLAYER_APOLOGIES_FOR_TA:
                level = getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_TA)->getCount();
                break;
            default:
                break;
        }

        if(event.getType() == EVENT_TYPE_PLAYER_TA || event.getType() == EVENT_TYPE_PLAYER_APOLOGIES_FOR_TA)
        {
            int *taDamageSteps = new int[5];
            bool found = false;
            for(int i=0;i<5;i++)
            {
                taDamageSteps[i] = atoi(getToken(g_TeamAttackSteps.GetString(), i+1).c_str());

                if(taDamageSteps[i] == 0)
                {
                    META_LOG(g_PLAPI,"Configuration error, please check the sw_steps_ta CVAR");
                    delete [] taDamageSteps;
                    return;
                }

                if(level < taDamageSteps[i])
                {
                    found = true;
                    level = i+1;
                    break;
                }
            }

            if(!found)//Bigger than the greatest step
                level = 5;

            delete [] taDamageSteps;
        }

        interBehavPoints = getToken(behavPoints, level);
        interBanPoints = getToken(banPoints, level);
        interHealth = getToken(health, level);
        interMoney = getToken(money, level);
    }

    if(interBehavPoints[interBehavPoints.size()-1] == '%')
    {
        int actualPoints = prevBehavPoints;
        percentBehav = static_cast<int>(static_cast<float>(atoi(interBehavPoints.substr(0, interBehavPoints.size()-1).c_str())) * final_coef); // Delete the %
        actualPoints += (actualPoints * percentBehav) / 100;
        finalBehavPoints = actualPoints;
        isPercentBehav = true;
    }
    else
    {
        finalBehavPoints = prevBehavPoints + static_cast<int>(static_cast<float>(atoi(interBehavPoints.c_str())) * final_coef);
    }

    if(interBanPoints[interBanPoints.size()-1] == '%')
    {
        int actualPoints = prevBanPoints;
        percentBan = static_cast<int>(static_cast<float>(atoi(interBanPoints.substr(0, interBanPoints.size()-1).c_str())) * final_coef); // Delete the %
        actualPoints += (actualPoints * percentBan) / 100;
        finalBanPoints = actualPoints;
        isPercentBan = true;
    }
    else
    {
        finalBanPoints = prevBanPoints + static_cast<int>(static_cast<float>(atoi(interBanPoints.c_str())) * final_coef);
    }

    if(interHealth[interHealth.size()-1] == '%')
    {
        int actualPoints = prevHealth;
        percentHealth = static_cast<int>(static_cast<float>(atoi(interHealth.substr(0, interHealth.size()-1).c_str())) * final_coef); // Delete the %
        actualPoints += (actualPoints * percentHealth) / 100;
        finalHealth = actualPoints;
        isPercentHealth = true;
    }
    else
    {
        finalHealth = prevHealth + static_cast<int>(static_cast<float>(atoi(interHealth.c_str())) * final_coef);
    }

    if(interMoney[interMoney.size()-1] == '%')
    {
        int actualPoints = prevMoney;
        percentMoney = static_cast<int>(static_cast<float>(atoi(interMoney.substr(0, interMoney.size()-1).c_str())) * final_coef); // Delete the %
        actualPoints += (actualPoints * percentMoney) / 100;
        finalMoney = actualPoints;
        isPercentMoney = true;
    }
    else
    {
        finalMoney = prevMoney + static_cast<int>(static_cast<float>(atoi(interMoney.c_str())) * final_coef);
    }

    if(finalBehavPoints > CVAR_INT(g_MaxBehavPoints))
        finalBehavPoints = CVAR_INT(g_MaxBehavPoints);
    if(finalBanPoints > CVAR_INT(g_MaxBanPoints))
        finalBanPoints = CVAR_INT(g_MaxBanPoints);
    if(finalHealth > 100)
        finalHealth = 100;
    if(finalMoney > 16000)
        finalMoney = 16000;

    if(finalBehavPoints < 0)
        finalBehavPoints = 0;
    if(finalBanPoints < 0)
        finalBanPoints = 0;
    if(finalHealth < 0)
        finalHealth = 0;
    if(finalMoney < 0)
        finalMoney = 0;

    shString message = reason;
    message += " :\n";

    if(finalBehavPoints == prevBehavPoints &&
        finalBanPoints == prevBanPoints &&
        finalHealth == prevHealth &&
        finalMoney == prevMoney)
    {
        message += _("No point gain");
        message += "\n";
        message += _("You are at your maximum");
        sendBoxText(event.getAuthor(), message, CVAR_INT(g_MessageBoxTime));
        return;
    }

    PlayerEvent *authorEvent = getPlayerEvent(event.getAuthor(), event.getType());
    authorEvent->setStatus(EVENT_STATUS_OCCURED);
    authorEvent->setBanPointsLost(prevBanPoints - finalBanPoints);
    authorEvent->setBehavPointsLost(prevBehavPoints - finalBehavPoints);
    authorEvent->setLastOccured(time(NULL));
	authorEvent->setOccured(authorEvent->getOccured() + 1);
    switch(event.getType())
    {
        case EVENT_TYPE_PLAYER_TEAM_FLASH:  //Amount of teammates
        case EVENT_TYPE_PLAYER_TA:          //Damage
        case EVENT_TYPE_PLAYER_INSULT:      //Level
            authorEvent->setCount(event.getIntValue());
            break;
        default:
            authorEvent->setCount(0);
    }

    if(CVAR_INT(g_DebugLevel))
    {
        META_LOG(g_PLAPI,"Player %s (%s) got %d behav. points, %d ban points, %d health and %d money for %s",
        getPlayer(event.getAuthor())->getName().c_str(), getPlayer(event.getAuthor())->getSteamID().c_str(),
        finalBehavPoints - prevBehavPoints, finalBanPoints - prevBanPoints,
        finalHealth - prevHealth, finalMoney - prevMoney,
        reason.c_str());
        META_LOG(g_PLAPI,"He now has %d behaviour points and %d ban points",
        finalBehavPoints, finalBanPoints);
    }

    if(finalBehavPoints != prevBehavPoints)
    {
        message += getIntegerChangeFormatted(finalBehavPoints - prevBehavPoints);
        message += " ";
        message += _("behav. points");
       if(isPercentBehav)
        {
            message += " (";
            message += itos(percentBehav);
            message += "%)";
        }
        message += "\n";
    }
    if(finalBanPoints != prevBanPoints)
    {
        message += getIntegerChangeFormatted(finalBanPoints - prevBanPoints);
        message += " ";
        message += _("ban points");
       if(isPercentBan)
        {
            message += " (";
            message += itos(percentBan);
            message += "%)";
        }
        message += "\n";
    }
    if(finalHealth != prevHealth)
    {
        message += getIntegerChangeFormatted(finalHealth - prevHealth);
        message += " ";
        message += _("health");
       if(isPercentHealth)
        {
            message += " (";
            message += itos(percentHealth);
            message += "%)";
        }
        message += "\n";
    }
    if(finalMoney != prevMoney)
    {
        message += getIntegerChangeFormatted(finalMoney - prevMoney);
        message += " ";
        message += "$";
       if(isPercentMoney)
        {
            message += " (";
            message += itos(percentMoney);
            message += "%)";
        }
        message += "\n";
    }

    player->getData()->lastEvent = event.getType();
    player->getData()->lastEventTime = time(NULL);
    player->setBehavPoints(finalBehavPoints);
    player->setBanPoints(finalBanPoints);
    if(finalHealth < prevHealth)
        slapPlayer(event.getAuthor(), prevHealth - finalHealth);
    else
        setPlayerHealth(event.getAuthor(), finalHealth);
    setPlayerMoney(event.getAuthor(), finalMoney);

    sendBoxText(event.getAuthor(), message, CVAR_INT(g_MessageBoxTime));

    if(CVAR_INT(g_BehavPointsKickSteps)+1 > 0)
    {
        for(int i=1;i<CVAR_INT(g_BehavPointsKickSteps)+1;i++)
        {
            int nextKickStep = CVAR_INT(g_MaxBehavPoints) - (CVAR_INT(g_MaxBehavPoints) / (CVAR_INT(g_BehavPointsKickSteps)+1)) * i;
            if(prevBehavPoints > nextKickStep && finalBehavPoints <= nextKickStep)
            {
                kickPlayer(event.getAuthor(), reason);

                if(CVAR_INT(g_DebugLevel))
                {
                    META_LOG(g_PLAPI,"Player %s (%s) has been kicked for %s",
                        getPlayer(event.getAuthor())->getName().c_str(), getPlayer(event.getAuthor())->getSteamID().c_str(),
                        reason.c_str());
                }

				shString message = _("%s has been {kicked} as a warning", getPlayer(event.getAuthor())->getName().c_str());
                sendChatTextToAll(message, CHAT_TEXT_BAD);
                break;
            }
        }
    }

    if(finalBehavPoints <= 0)
    {
        banPlayer(event.getAuthor(), reason, 0);

        if(CVAR_INT(g_DebugLevel))
        {
            META_LOG(g_PLAPI,"Player %s (%s) has been banned for %s",
                getPlayer(event.getAuthor())->getName().c_str(), getPlayer(event.getAuthor())->getSteamID().c_str(),
                reason.c_str());
        }

        shString banTime = _("for an unlimited time");
		shString message = _("%s has been {banned} %s for having 0 behaviour points",getPlayer(event.getAuthor())->getName().c_str() , banTime.c_str());
        sendChatTextToAll(message, CHAT_TEXT_BAD);
    }

    if(finalBanPoints <= 0)
    {
        banPlayer(event.getAuthor(), reason, 0);

        if(CVAR_INT(g_DebugLevel))
        {
            META_LOG(g_PLAPI,"Player %s (%s) has been banned for %s",
                getPlayer(event.getAuthor())->getName().c_str(), getPlayer(event.getAuthor())->getSteamID().c_str(),
                reason.c_str());
        }

        shString banTime = _("for an unlimited time");
		shString message = _("%s has been {banned} %s for having 0 ban points",getPlayer(event.getAuthor())->getName().c_str() , banTime.c_str());
        sendChatTextToAll(message, CHAT_TEXT_BAD);
    }

    if(finalBehavPoints <= 0 || finalBanPoints <= 0)
    {
        shString bansURL = g_SendBanURL.GetString();
        if(!bansURL.empty())
        {
            Player *player = getPlayer(event.getAuthor());
            URLRequest newRequest;
            newRequest.setURL(bansURL);
            newRequest.addParameter("player", player->getName());
            newRequest.addParameter("steamid", player->getSteamID());
            newRequest.addParameter("behav_points", itos(player->getBehavPoints()));
            newRequest.addParameter("ban_points", itos(player->getBanPoints()));
            shString completeReason = reason;
            completeReason += " (ServerWiper ban : ";
            if(finalBehavPoints <= 0)
                completeReason += _("0 behaviour points");
            else if(finalBanPoints <= 0)
                completeReason += _("0 ban points");
            else
                completeReason += "???";
            completeReason += ")";
            newRequest.addParameter("reason", completeReason);
            shString extendedReason = "";
            switch(event.getType())
            {
                case EVENT_TYPE_PLAYER_TA:
                    extendedReason += _("Damage done to %s : %d", getPlayer(event.getVictim())->getName().c_str(), event.getIntValue());
                    break;
                case EVENT_TYPE_PLAYER_TK:
                    extendedReason += _("Killed %s", getPlayer(event.getVictim())->getName().c_str());
                    break;
                case EVENT_TYPE_PLAYER_TEAM_FLASH:
                    extendedReason += _("Team flash : %d teammates", event.getIntValue());
                    break;
                case EVENT_TYPE_PLAYER_INSULT:
                    extendedReason += _("Insult : '%s' in '%s'", event.getText().c_str(), event.getSecondText().c_str());
                    break;
            }
            newRequest.addParameter("extreason", extendedReason);
            newRequest.addParameter("server", g_ShortServerName.GetString());
            newRequest.addParameter("admin", "ServerWiper");
			//Stats
			newRequest.addParameter("kicks", itos(player->getKickCount()));
			newRequest.addParameter("bans", itos(player->getBanCount()));
			newRequest.addParameter("connections", itos(player->getPlayerEvent(EVENT_TYPE_PLAYER_CONNECTED)->getOccured()));
			newRequest.addParameter("ta", itos(player->getPlayerEvent(EVENT_TYPE_PLAYER_TA)->getOccured()));
			newRequest.addParameter("tk", itos(player->getPlayerEvent(EVENT_TYPE_PLAYER_TK)->getOccured()));
			newRequest.addParameter("tf", itos(player->getPlayerEvent(EVENT_TYPE_PLAYER_TEAM_FLASH)->getOccured()));
			newRequest.addParameter("insults", itos(player->getPlayerEvent(EVENT_TYPE_PLAYER_INSULT)->getOccured()));
			newRequest.addParameter("greetings", itos(player->getPlayerEvent(EVENT_TYPE_PLAYER_GREETINGS)->getOccured()));
            URLManager::getInstance()->addNewRequest(newRequest);
        }
    }
}

void ServerWiper::showStatusToPlayer(int const& player)
{
    if(!CVAR_BOOL(g_AllowStatus))
        return;

    int pointsUntilKickByBehavPoints = 0;
    int pointsUntilBanByBanPoints = getPlayer(player)->getBanPoints();

    for(int i=1;i<CVAR_INT(g_BehavPointsKickSteps)+1;i++)
    {
        int nextKickStep = CVAR_INT(g_MaxBehavPoints) - (CVAR_INT(g_MaxBehavPoints) / (CVAR_INT(g_BehavPointsKickSteps)+1)) * i;
        if(nextKickStep < getPlayer(player)->getBehavPoints())
        {
            pointsUntilKickByBehavPoints = getPlayer(player)->getBehavPoints() - nextKickStep;
            break;
        }
    }

    if(pointsUntilKickByBehavPoints == 0)
        pointsUntilKickByBehavPoints = getPlayer(player)->getBehavPoints();

    shString message = _("Current status :");
    message += "\n";
    message += _("Behaviour points until");
    message += " ";

    if(getPlayer(player)->getBehavPoints() < CVAR_INT(g_MaxBehavPoints) / (CVAR_INT(g_BehavPointsKickSteps)+1))
        message += _("ban");
    else
        message += _("kick");

    message += " : ";
    message += itos(pointsUntilKickByBehavPoints);
    message += " (";
    message += itos(getPlayer(player)->getBehavPoints());
    message += "/";
    message += itos(CVAR_INT(g_MaxBehavPoints));
    message += ")\n";

    message += _("Ban points until ban :");
    message += " ";
    message += itos(pointsUntilBanByBanPoints);
    message += " (";
    message += itos(getPlayer(player)->getBanPoints());
    message += "/";
    message += itos(CVAR_INT(g_MaxBanPoints));
    message += ")";

    if(getPlayer(player)->getBanPoints() < CVAR_INT(g_MaxBanPoints))
    {
        message += "\n";
        message += _("Time until next ban point gain :");
        message += " ";
        message += itos(CVAR_INT(g_BanRecoveryPlaytime) - getPlayer(player)->getPlayTime());
        message += _(" minutes");
    }

    sendBoxText(player, message, CVAR_INT(g_MessageBoxTime));
}

void ServerWiper::showWelcomeMessage(int const& player)
{
    shString message = _("Welcome to a ServerWiper protected server !\nPlease watch your behaviour and play cool :)");
    sendBoxText(player, message, CVAR_INT(g_MessageBoxTime));
}

int ServerWiper::getPlayerIndexByUserID(int const& userID)
{
    if(userIndexByUserID.find(userID) == userIndexByUserID.end())
        return -1;

    return userIndexByUserID[userID];
}

int ServerWiper::getPlayerIndexBySteamID(shString const& steamID)
{
    for(shHash<int, Player *>::iterator i = players.begin();i!=players.end();i++)
    {
        if(!(*i).val)
            continue;

        if((*i).val->getSteamID() == steamID)
        {
            return (*i).key;
        }
    }
    return -1;
}

void ServerWiper::think()
{
	if(waitingForDatabaseStatsAnswer > 0)
	{
		while(databaseStatsAnswer.count() > 0)
		{
			waitingForDatabaseStatsAnswer--;

			DatabaseStatsRequest dsr = databaseStatsAnswer.pop();

			Player *admin = getPlayer(dsr.getAdmin());
			if(!admin)
				continue;

			shString message = "";

			if(dsr.getType() == DB_STATS_REQUEST_SERVER)
			{
				message = "Server stats :\n";
			}
			else if(dsr.getType() == DB_STATS_REQUEST_PLAYER)
			{
				int playerIndex = getPlayerIndexBySteamID(dsr.getPlayer());

				if(playerIndex == -1)
					continue;

				Player *player = getPlayer(playerIndex);
				if(!player)
					continue;

				message = player->getName() + " :\n";
				message += _("Behav. pts : %d, ban pts : %d", player->getBehavPoints(), player->getBanPoints()) + "\n";
			}

			message += _("Kicks : %d, bans : %d", dsr.getKicks(), dsr.getBans()) + "\n";
			message += _("TA : %d, TK : %d", dsr.getEventCount(EVENT_TYPE_PLAYER_TA), dsr.getEventCount(EVENT_TYPE_PLAYER_TK)) + "\n";
			message += _("TF : %d, insults : %d", dsr.getEventCount(EVENT_TYPE_PLAYER_TEAM_FLASH), dsr.getEventCount(EVENT_TYPE_PLAYER_INSULT));
			sendBoxText(admin->getIndex(), message, CVAR_INT(g_MessageBoxTime));

			message = _("Connections : %d, greetings : %d, apologies for ta : %d, apologies for tk : %d, apologies for insult : %d, apologies for tf : %d",
				dsr.getEventCount(EVENT_TYPE_PLAYER_CONNECTED), dsr.getEventCount(EVENT_TYPE_PLAYER_GREETINGS), dsr.getEventCount(EVENT_TYPE_PLAYER_APOLOGIES_FOR_TA),
				dsr.getEventCount(EVENT_TYPE_PLAYER_APOLOGIES_FOR_TK), dsr.getEventCount(EVENT_TYPE_PLAYER_APOLOGIES_FOR_INSULT), dsr.getEventCount(EVENT_TYPE_PLAYER_APOLOGIES_FOR_TF));
			sendChatText(admin->getIndex(), message);

			message = _("Leaves : %d, bomb planted : %d, bomb defused : %d, killed hostage : %d, used hostage : %d, rescued hostages : %d, attacked hostage : %d",
				dsr.getEventCount(EVENT_TYPE_PLAYER_LEAVER), dsr.getEventCount(EVENT_TYPE_PLAYER_PLANTED_BOMB), dsr.getEventCount(EVENT_TYPE_PLAYER_DEFUSED_BOMB),
				dsr.getEventCount(EVENT_TYPE_PLAYER_KILLED_HOSTAGE), dsr.getEventCount(EVENT_TYPE_PLAYER_USED_HOSTAGE), dsr.getEventCount(EVENT_TYPE_PLAYER_RESCUED_HOSTAGE),
				dsr.getEventCount(EVENT_TYPE_PLAYER_ATTACKED_HOSTAGE));
			sendChatText(admin->getIndex(), message);
		}
	}
	if(hasUpdates > 0)
	{
		while(playerUpdates.count() > 0)
		{
			hasUpdates--;
			PlayerUpdate plu = playerUpdates.pop();

			int playerIndex = getPlayerIndexBySteamID(plu.steamID);

			Player *player = getPlayer(playerIndex);
			if(player)
			{
				//Check if the values has been modified before we edit it
				int behavPointsDifference = player->getBehavPoints() - CVAR_INT(g_MaxBehavPoints);
				int banPointsDifference = player->getBanPoints() - CVAR_INT(g_MaxBanPoints);

				player->setBehavPoints(plu.behavPoints + behavPointsDifference);
				player->setBanPoints(plu.banPoints + banPointsDifference);
				player->setPlayTime(plu.playtime);
				player->setData(plu.data);
				player->setKickCount(plu.kicks);
				player->setBanCount(plu.bans);

				for(int i=0;i<EVENT_TYPE_MAX;i++)
				{
					player->getPlayerEvent(i)->setOccured(plu.eventOccured[i]);
				}

				int actualConnections = plu.eventOccured[EVENT_TYPE_PLAYER_CONNECTED];
				player->getPlayerEvent(EVENT_TYPE_PLAYER_CONNECTED)->setOccured(actualConnections + 1);
			}
		}
	}

	if(clientsPending.size() > 0)
	{
		if(getEngine()->Time() > nextPendingCheck)
		{
			nextPendingCheck = getEngine()->Time() + 0.5;

			for(shVector<int>::iterator it = clientsPending.begin(); it != clientsPending.end(); )
			{
				shString steamID = getEngine()->GetPlayerNetworkIDString(getPlayer(*it)->getEdict());

				if(steamID != "STEAM_ID_PENDING")
				{
					Player *player = getPlayer(*it);
					player->setSteamID(steamID);
					player->setHasSteamID(true);
					gotPlayerSteamID(*it);

					it = clientsPending.erase(it);
				}
				else
					it++;
			}
		}
	}
    RETURN_META(MRES_HANDLED);
}

void ServerWiper::lessThink()
{
    if(CVAR_BOOL(g_Advert) && players.GetBuckets() > 0)
    {
        if(time(NULL) - lastAdvert >= CVAR_INT(g_AdvertDelay))
        {
            lastAdvert = time(NULL);
            advert();
        }
    }

    if(CVAR_BOOL(g_HelpMessages) && players.GetBuckets() > 0)
    {
        if(time(NULL) - lastHelpMessage >= CVAR_INT(g_HelpMessagesDelay))
        {
            lastHelpMessage = time(NULL);
            helpMessage();
        }
    }

    while(DatabaseManager::getInstance()->getMessagesCount() > 0)
    {
        META_LOG(g_PLAPI, DatabaseManager::getInstance()->popMessage().c_str());
    }

    while(URLManager::getInstance()->getMessagesCount() > 0)
    {
        META_LOG(g_PLAPI, URLManager::getInstance()->popMessage().c_str());
    }
}

void ServerWiper::gotPlayerSteamID(int const& index)
{
	Player *newPlayer = getPlayer(index);

    DatabaseRequest newRequest;
    newRequest.setType(DB_REQUEST_PLAYER_ADD);
    newRequest.setPlayer(*newPlayer);
    DatabaseManager::getInstance()->addNewRequest(newRequest);

    newRequest.setType(DB_REQUEST_PLAYER_GET);
    newRequest.setPlayer(*newPlayer);
    DatabaseManager::getInstance()->addNewRequest(newRequest);

    if(AdminManager::getInstance()->isAdmin(newPlayer->getSteamID()))
        newPlayer->setAdmin(true);

	if(CVAR_BOOL(g_ServerWiperEnabled) && CVAR_BOOL(g_ShowSteamIdOnConnect))
	{
		sendChatTextToAll(_("New player : {%s} [%s]", newPlayer->getName().c_str(), newPlayer->getSteamID().c_str()));
	}

	TeamManager::getInstance()->checkPlayer(index);
}

void ServerWiper::postInit()
{
    messageSayText = UserMessageIndex("SayText");
    messageVGUI = UserMessageIndex("VGUIMenu");

    InsultManager::getInstance()->loadInsults("insults1.txt", 1);
    InsultManager::getInstance()->loadInsults("insults2.txt", 2);
    InsultManager::getInstance()->loadInsults("insults3.txt", 3);
    GreetingManager::getInstance()->loadGreetings("greetings.txt");
    ApologyManager::getInstance()->loadApologies("apologies.txt");
    URLManager::getInstance();
    DatabaseManager::getInstance()->loadDatabase("players.db");
    MenuManager::getInstance();
    AdminManager::getInstance()->loadAdminFile("admins.txt");
    HelpManager::getInstance()->loadHelpMessages();
	TeamManager::getInstance()->loadTeamFile("team.txt");

    DatabaseRequest newRequest;
    newRequest.setType(DB_REQUEST_CREATE);
    DatabaseManager::getInstance()->addNewRequest(newRequest);
}

unsigned int UTIL_FindInDataMap(datamap_t *pMap, const char *name)
{
	while (pMap)
	{
		for (int i=0; i<pMap->dataNumFields; i++)
		{
			if (pMap->dataDesc[i].fieldName == NULL)
			{
				continue;
			}
			if (strcmp(name, pMap->dataDesc[i].fieldName) == 0)
			{
				return pMap->dataDesc[i].fieldOffset[TD_OFFSET_NORMAL];
			}
			if (pMap->dataDesc[i].td)
			{
				unsigned int offset;
				if ((offset=UTIL_FindInDataMap(pMap->dataDesc[i].td, name)) != 0)
				{
					return offset;
				}
			}
		}
		pMap = pMap->baseMap;
	}

	return 0;
}

int ServerWiper::UserMessageIndex(const char *messageName)
{
	//Returns MessageIndex IDs from Msg String
	char name[128] = "";
	int sizereturn = 0;
	bool boolrtn = false;

	for(int x=1; x<22; x++)
	{
		boolrtn =  plugin->getServerDLL()->GetUserMessageInfo(x, name, 128, sizereturn);
		if(name && strcmp(messageName, name) == 0)
		{
			return x;
		}
	}
	return -1;
}

void ServerWiper::sendChatText(int const& player, shString const& message, ChatTextType const& type)
{
    //Replace { with \x03 and } with \x01
    shString newMessage;
    for(int i=0;i<message.size();i++)
    {
        if(message[i] == '{')
            newMessage += "\x03";
		else if(message[i] == '}')
            newMessage += "\x01";
        else
            newMessage += message[i];
    }

    char pMessage[512];
    Q_snprintf(pMessage,512,"\x04ServerWiper\x01 : %s", newMessage.c_str());

    bf_write *msg;
    MRecipientFilter mrf;
    mrf.AddRecipient(player);
    msg = getEngine()->UserMessageBegin((IRecipientFilter *)&mrf, messageSayText);
    if(type == CHAT_TEXT_NEUTRAL)
        msg->WriteByte(0);//0 = from server
    else if(type == CHAT_TEXT_GOOD)
        msg->WriteByte(lastCounterTerroristIndex);
    else if(type == CHAT_TEXT_BAD)
        msg->WriteByte(lastTerroristIndex);
    msg->WriteString(pMessage);
    msg->WriteByte(true);
    getEngine()->MessageEnd();
}

void ServerWiper::sendChatTextToAll(shString const& message, ChatTextType const& type)
{
    for(shHash<int, Player *>::iterator it = players.begin(); it != players.end(); it++)
    {
        if((*it).val)
            sendChatText((*it).val->getIndex(), message, type);
    }
}

void ServerWiper::sendPlayerChatText(int const& player, shString const& message)
{
    char pMessage[512];
	Q_snprintf(pMessage,512,"{%s} :  %s", getPlayer(player)->getName().c_str(), message.c_str());

    bf_write *msg;
    MRecipientFilter mrf;
    mrf.AddRecipient(player);
    msg = getEngine()->UserMessageBegin((IRecipientFilter *)&mrf, messageSayText);
    msg->WriteByte(player);//0 = from server
    msg->WriteString(pMessage);
    msg->WriteByte(true);
    getEngine()->MessageEnd();
}

void ServerWiper::sendBoxText(int const& player, shString const& message, int const& duration)
{
    //Close any opened menu
    MenuManager::getInstance()->onPlayerMenuSelect(player, 10);

    shString finalMessage = message;
    MRecipientFilter filter;
    filter.AddRecipient(player);
	bf_write *pBuffer = getEngine()->UserMessageBegin(&filter, 10);//messageVGUI); -> messageVGUI does not work ! (13)
	pBuffer->WriteShort( (1<<0) | (1<<1) | (1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7) | (1<<8) | (1<<9) ); //Sets how many options the menu has
	pBuffer->WriteChar( duration ); //Sets how long the menu stays open -1 for stay until option selected
	pBuffer->WriteByte( false );
	finalMessage += "\n0. ";
	finalMessage += _("Exit");
	pBuffer->WriteString( finalMessage.c_str() ); //The text shown on the menu
	getEngine()->MessageEnd();
}

void ServerWiper::sendBoxTextToAll(shString const& message, int const& duration)
{
    for(shHash<int, Player *>::iterator it = players.begin(); it != players.end(); it++)
    {
        if((*it).val)
            sendBoxText((*it).val->getIndex(), message, duration);
    }
}

int ServerWiper::getPlayerInteger(int const& player, shString const& variable)
{
    unsigned int offset = 0;

	IServerUnknown *pUnknown = (IServerUnknown *)getEngine()->PEntityOfEntIndex(player)->GetUnknown();
	if (!pUnknown)
	{
		return -1;
	}

	CBaseEntity *pEntity = pUnknown->GetBaseEntity();

	if (!offset)
	{
		offset = UTIL_FindInDataMap(pEntity->GetDataDescMap(), variable.c_str());
	}

	if(variable == "m_iAccount")
        offset = OFFSET_ACCOUNT;//UGLY HACK...

	return *(int *)((char *)pEntity + offset);
}

void ServerWiper::setPlayerInteger(int const& player, shString const& variable, int const& value)
{
    unsigned int offset = 0;

	IServerUnknown *pUnknown = (IServerUnknown *)getEngine()->PEntityOfEntIndex(player)->GetUnknown();
	if (!pUnknown)
	{
		return;
	}

	CBaseEntity *pEntity = pUnknown->GetBaseEntity();

	if (!offset)
	{
		offset = UTIL_FindInDataMap(pEntity->GetDataDescMap(), variable.c_str());
	}

    if(variable == "m_iAccount")
        offset = OFFSET_ACCOUNT;//UGLY HACK...

	*(int *)((char *)pEntity + offset) = value;
}

int ServerWiper::getPlayerMoney(int const& player)
{
    return getPlayerInteger(player, "m_iAccount");
}

int ServerWiper::getPlayerHealth(int const& player)
{
    return getPlayerInteger(player, "m_iHealth");
}

void ServerWiper::setPlayerMoney(int const& player, int const& money)
{
    int newMoney = money;
    if(newMoney < 0)
        newMoney = 0;
    if(newMoney > 16000)
        newMoney = 16000;
    setPlayerInteger(player, "m_iAccount", newMoney);
}

void ServerWiper::setPlayerHealth(int const& player, int const& health)
{
    if(health <= 0)
    {
        slayPlayer(player);
        return;
    }
    int newHealth = health;
    if(newHealth > 200)
        newHealth = 200;
    setPlayerInteger(player, "m_iHealth", newHealth);
}

void ServerWiper::slapPlayer(int const& player, int const& damage)
{
    int health = getPlayerHealth(player);

    edict_t * entity = getEngine()->PEntityOfEntIndex(player);
    IServerUnknown * pServerUnknown = entity->GetUnknown();
    CBaseEntity * pEntity = pServerUnknown ->GetBaseEntity();

    MRecipientFilter filter;
    for(shHash<int, Player *>::iterator it = players.begin(); it != players.end(); it++)
    {
        if((*it).val)
            filter.AddRecipient((*it).val->getIndex());
    }
    Vector pos = pEntity->GetCollideable()->GetCollisionOrigin();
    plugin->getEngineSound()->EmitSound((IRecipientFilter &)filter, player, CHAN_AUTO, "player/damage2.wav", 0.7, ATTN_NORM, 0, PITCH_NORM, &pos, 0, 0, true, 0, player);

    if(health <= damage)
    {
        plugin->getHelper()->ClientCommand(entity,"kill");
        return;
    }

    setPlayerHealth(player, getPlayerHealth(player) - damage);
}

void ServerWiper::slayPlayer(int const& player)
{
    edict_t * entity = getEngine()->PEntityOfEntIndex(player);

    IServerUnknown * pServerUnknown = entity->GetUnknown();
    CBaseEntity * pEntity = pServerUnknown ->GetBaseEntity();

    plugin->getHelper()->ClientCommand(entity,"kill");

    MRecipientFilter filter;
    for(shHash<int, Player *>::iterator it = players.begin(); it != players.end(); it++)
    {
        if((*it).val)
            filter.AddRecipient((*it).val->getIndex());
    }
    Vector pos = pEntity->GetCollideable()->GetCollisionOrigin();
    plugin->getEngineSound()->EmitSound((IRecipientFilter &)filter, player, CHAN_AUTO, "player/damage3.wav", 0.7, ATTN_NORM, 0, PITCH_NORM, &pos, 0, 0, true, 0, player);
}

void ServerWiper::kickPlayer(int const& player, shString const& reason)
{
	Player *pPlayer = getPlayer(player);

    char buffer[100];
    Q_snprintf(buffer,100,"kickid %s %s\n", pPlayer->getSteamID().c_str(), reason.c_str());
    pPlayer->getData()->gotKicked = true;
	pPlayer->setKickCount(pPlayer->getKickCount() + 1);
    getEngine()->ServerCommand(buffer);
}

void ServerWiper::banPlayer(int const& player, shString const& reason, int const& duration)
{
	Player *pPlayer = getPlayer(player);

    char buffer[100];
    Q_snprintf(buffer,100,"banid %d %s\n", duration, pPlayer->getSteamID().c_str());
    getEngine()->ServerCommand(buffer);
	pPlayer->setBanCount(pPlayer->getBanCount() + 1);
    kickPlayer(player, reason);
    getEngine()->ServerCommand("writeid\n");
}

shString ServerWiper::getServerName()
{
    ConVar* hostname = getConVar("hostname");
    shString back = "";
    if(hostname)
        back = hostname->GetString();
    return back;
}

shString ServerWiper::getServerPort()
{
    ConVar* port = getConVar("hostport");
    shString back = "27015";
    if(port)
        back = port->GetString();
    return back;
}

shString ServerWiper::getMaxplayers()
{
    return itos(plugin->getMaxClients());
}

shString ServerWiper::getMapName()
{
    return STRING(g_SMAPI->pGlobals()->mapname);
}

bool ServerWiper::isPlayerAlive(int const& player)
{
    CPlayerState *playerState = plugin->getGameClients()->GetPlayerState(getEngine()->PEntityOfEntIndex(player));
    if(!playerState)
        return false;

    if(!playerState->deadflag)
        return true;

    return false;
}

void ServerWiper::hideSayMessage()
{
    showSayMessage = false;
}

bool ServerWiper::shouldShowSayMessage()
{
    bool value = showSayMessage;
    showSayMessage = true;
    return value;
}

void ServerWiper::helpMessage()
{
    shString completeMessage = _("[HELP]");
    completeMessage += " ";
    shString message;
    if(HelpManager::getInstance()->getRandomHelpMessage(message))
    {
        completeMessage += message;
        sendChatTextToAll(completeMessage, CHAT_TEXT_GOOD);
    }
}

void ServerWiper::advert()
{
    shString message = "ServerWiper ";
    message += _("version");
    message += " ";
    message += g_ServerWiperVersion.GetString();
    message += " ";
    message += _("coded by");
    message += " ";
	message += "{Jmgr} <\x04www.jmgr.info\x01>";

    sendChatTextToAll(message, CHAT_TEXT_GOOD);
}

void ServerWiper::statusChanged(bool on)
{
	//todo : what is it's deactivated while a map is running ? -> FIXME
    if(on)
		sendChatTextToAll(_("ServerWiper is now {ON}"), CHAT_TEXT_GOOD);
    else
		sendChatTextToAll(_("ServerWiper is now {OFF}"), CHAT_TEXT_BAD);
}

ConVar* ServerWiper::getConVar(shString const& varName)
{
    if(!plugin->getICvar())
        return 0;

    ConVar* var = plugin->getICvar()->FindVar(varName.c_str());
    if(var)
        return var;
    return 0;
}

void ServerWiper::newRound()
{
    if(CVAR_BOOL(g_Register) && time(NULL) - lastRegistration >= 600)
    {
        lastRegistration = time(NULL);
        URLRequest newRequest;
        newRequest.setURL("http://sw.jmgr.info/registration.php");
        newRequest.addParameter("port", getServerPort());
        newRequest.addParameter("hostname", getServerName());
        newRequest.addParameter("maxplayers", getMaxplayers());
        newRequest.addParameter("version", g_ServerWiperVersion.GetString());
        #ifdef _WIN32
        newRequest.addParameter("os", "windows");
        #else
        newRequest.addParameter("os", "linux");
        #endif
        newRequest.addParameter("mapname", getMapName());
        newRequest.addParameter("players", itos(players.GetBuckets()));
        URLManager::getInstance()->addNewRequest(newRequest);
    }

    if(CVAR_BOOL(g_ForgiveTkMenu))
    {
        //Deactivate Mani's forgive menu
        ConVar* mani_tk_forgive = getConVar("mani_tk_forgive");
        if(mani_tk_forgive && atoi(mani_tk_forgive->GetString()))
        {
            mani_tk_forgive->SetValue(0);
            META_LOG(g_PLAPI,"Detected mani_tk_forgive = 1 and sw_forgive_tk_menu = 1, setting mani_tk_forgive to 0");
        }
    }

	if(CVAR_INT(g_DatabaseCleanDelay) > 0)
	{
		//Clean the database
		DatabaseRequest newRequest;
		newRequest.setType(DB_REQUEST_CLEAN);
		DatabaseManager::getInstance()->addNewRequest(newRequest);
	}

    for(shHash<int, Player *>::iterator i = players.begin();i!=players.end();i++)
    {
        if(!(*i).val)
            continue;

        //Autoclose player menus that are timeouted
        if((*i).val->getActualMenu() != -1 && (*i).val->getActualMenuTimeout() != 0)
        {
            if((*i).val->getActualMenuTimeout() <= time(NULL))
            {
                MenuManager::getInstance()->onPlayerMenuSelect((*i).val->getIndex(), 10);//Close event
            }
        }

        //Resend menus that are gone
        if((*i).val->getActualMenu() != -1)
        {
            if((*i).val->getActualMenu() == MENU_FORGIVE)
                MenuManager::getInstance()->showForgiveMenu((*i).val->getIndex(), (*i).val->getLastTKKiller(), (*i).val->getActualMenuTimeout() - time(NULL));
            else if((*i).val->getActualMenu() == MENU_ADMIN)
                MenuManager::getInstance()->showAdminMenu((*i).val->getIndex());
            else if((*i).val->getActualMenu() == MENU_PLAYER_ADMIN)
                MenuManager::getInstance()->showPlayerAdminMenu((*i).val->getIndex());
            else if((*i).val->getActualMenu() == MENU_SERVER_ADMIN)
                MenuManager::getInstance()->showServerAdminMenu((*i).val->getIndex());
            //TODO -> Resend other menus
        }

        (*i).val->setUsedHostageCount(0);

        if(CVAR_BOOL(g_SavePlayersAtRoundStart))
        {
            //Save the player in the database
            DatabaseRequest newRequest;
            newRequest.setType(DB_REQUEST_PLAYER_SAVE);
            newRequest.setPlayer(*getPlayer((*i).val->getIndex()));
            newRequest.setData(*getPlayer((*i).val->getIndex())->getData());
            DatabaseManager::getInstance()->addNewRequest(newRequest);
        }

        int roundTime = (time(NULL) - lastRoundStart) / 60;
        (*i).val->setPlayTime((*i).val->getPlayTime() + roundTime);

        if((*i).val->getBanPoints() < CVAR_INT(g_MaxBanPoints) && (*i).val->getPlayTime() >= CVAR_INT(g_BanRecoveryPlaytime))
        {
            (*i).val->setPlayTime(0);
            (*i).val->setBanPoints((*i).val->getBanPoints() + 1);
			shString message = _("You regain {one ban point} for having played %d minutes on the server", CVAR_INT(g_BanRecoveryPlaytime));
            sendChatText((*i).val->getIndex(), message, CHAT_TEXT_GOOD);

            if(CVAR_INT(g_DebugLevel))
            {
                META_LOG(g_PLAPI,"Player %s (%s) has regained a ban point for playing %d minutes on the server",
                    getPlayer((*i).val->getIndex())->getName().c_str(), getPlayer((*i).val->getIndex())->getSteamID().c_str(),
                    CVAR_INT(g_BanRecoveryPlaytime));
                META_LOG(g_PLAPI,"He now has %d behaviour points and %d ban points",
                    (*i).val->getBehavPoints(), (*i).val->getBanPoints());
            }
        }
    }
    lastRoundStart = time(NULL);
}

void ServerWiper::addPlayerUpdate(shString const& steamid, int const& behav, int const& ban, int const& playtime, PlayerDataV1 const& data, int eventOccured[EVENT_TYPE_MAX], int const& kicks, int const& bans)
{
    PlayerUpdate pl;
    pl.steamID = steamid;
    pl.behavPoints = behav;
    pl.banPoints = ban;
    pl.playtime = playtime;
    pl.data = data;
	pl.kicks = kicks;
	pl.bans = bans;
	memcpy(pl.eventOccured, eventOccured, sizeof(int) * EVENT_TYPE_MAX);
    playerUpdates.push(pl);
    hasUpdates++;
}

void ServerWiper::addDatabaseStatsAnswer(DatabaseStatsRequest const& answer)
{
	databaseStatsAnswer.push(answer);
}

void ServerWiper::addDatabaseStatsRequest(int const& type, int const& admin, shString const& player)
{
	waitingForDatabaseStatsAnswer++;
	
	DatabaseStatsRequest newRequest;
	newRequest.setType(type);
	newRequest.setAdmin(admin);
	newRequest.setPlayer(player);
	DatabaseManager::getInstance()->addNewStatsRequest(newRequest);
}

void ServerWiper::resetAllPlayers()
{
    DatabaseRequest newRequest;
    newRequest.setType(DB_REQUEST_RESET);
    DatabaseManager::getInstance()->addNewRequest(newRequest);

    for(shHash<int, Player *>::iterator i = players.begin();i!=players.end();i++)
    {
        if(!(*i).val)
            continue;

        (*i).val->setBehavPoints(CVAR_INT(g_MaxBehavPoints));
        (*i).val->setBanPoints(CVAR_INT(g_MaxBanPoints));
    }

    META_LOG(g_PLAPI,"Player database has been reset");
}

void ServerWiper::resetPlayer(shString const& steamID)
{
    Player pl;
    pl.setSteamID(steamID);

    META_LOG(g_PLAPI,"Player %s has been reset", steamID.c_str());

    DatabaseRequest newRequest;
    newRequest.setType(DB_REQUEST_PLAYER_RESET);
    newRequest.setPlayer(pl);
    DatabaseManager::getInstance()->addNewRequest(newRequest);

    for(shHash<int, Player *>::iterator i = players.begin();i!=players.end();i++)
    {
        if(!(*i).val)
            continue;

        if((*i).val->getSteamID() == steamID)
        {
            (*i).val->setBehavPoints(CVAR_INT(g_MaxBehavPoints));
            (*i).val->setBanPoints(CVAR_INT(g_MaxBanPoints));
            break;
        }
    }
}

void ServerWiper::checkPlayerName(int const& player)
{
	if(!CVAR_BOOL(g_ServerWiperEnabled))
		return;

    //Check for insults
    shString insult= "";
    if(strstr(getPlayer(player)->getName().c_str(), "STEAM_") || (CVAR_BOOL(g_CheckPlayersName) && InsultManager::getInstance()->getInsultLevel(getPlayer(player)->getName(), insult) > 0))
    {
        kickPlayer(player, _("Bad nickname"));

		shString message = _("%s has been {kicked} for having a bad nickname", getPlayer(player)->getName().c_str());
        sendChatTextToAll(message, CHAT_TEXT_BAD);

        if(CVAR_INT(g_DebugLevel))
        {
            if(strstr(getPlayer(player)->getName().c_str(), "STEAM_"))//Needed for ban detection
                insult = "STEAM_";
            META_LOG(g_PLAPI,"Player %s (%s) has been kicked for having a bad nickname, containing %s",
                getPlayer(player)->getName().c_str(), getPlayer(player)->getSteamID().c_str(),
                insult.c_str());
        }
    }

    //Check for allowed characters percentage
    shString allowedChars = g_AllowedCharactersInName.GetString();
    shString name = getPlayer(player)->getName();
    int numChars = name.size();
    if(CVAR_BOOL(g_AllowedNamesCheck) && allowedChars.size() > 0 && numChars > 0)
    {
        int minAllowedCharsPercentage = CVAR_INT(g_MinAllowedCharactersInName);
        int numAllowedChars = 0;
        for(int i=0;i<numChars;i++)
        {
            for(int j=0;j<allowedChars.size();j++)
            {
                if(name[i] == allowedChars[j])
                {
                    numAllowedChars++;
                    break;
                }
            }
        }

        int percentageAllowedChars = static_cast<int>((static_cast<float>(numAllowedChars) / static_cast<float>(numChars)) * 100.0f);
        if(percentageAllowedChars < minAllowedCharsPercentage)
        {
            kickPlayer(player, _("Bad nickname, put less special chars in it"));

            shString message = getPlayer(player)->getName();
			message += _(" has been {kicked} for having a bad nickname");
            sendChatTextToAll(message, CHAT_TEXT_BAD);

            if(CVAR_INT(g_DebugLevel))
            {
                META_LOG(g_PLAPI,"Player %s (%s) has been kicked for having a bad nickname, less than %d%% of allowed chars : %d%%",
                    getPlayer(player)->getName().c_str(), getPlayer(player)->getSteamID().c_str(),
                    minAllowedCharsPercentage, percentageAllowedChars);
            }
        }
    }
}

bool ServerWiper::gotPlayerCommand(int const& player, shString const& command)
{
    Player *commandPlayer = ServerWiper::getInstance()->getPlayer(player);
    if(!commandPlayer)
        return false;

    bool admin = commandPlayer->isAdmin();

    if(command == "swadmin"|| command == "sw_admin")
    {
        if(!admin)
        {
            sendChatText(player, _("You are not allowed to run this command"));
            return false;
        }
        MenuManager::getInstance()->showAdminMenu(player);
        return true;
    }
    else if(command == "sw_ban_top")
    {
        if(!admin)
        {
            sendChatText(player, _("You are not allowed to run this command"));
            return false;
        }
        MenuManager::getInstance()->showTopMenu(player, false);
        return true;
    }
    else if(command == "sw_behav_top")
    {
        if(!admin)
        {
            sendChatText(player, _("You are not allowed to run this command"));
            return false;
        }
        MenuManager::getInstance()->showTopMenu(player, true);
        return true;
    }
	else if (command == "sw_stats")
	{
		if(!admin)
		{
			sendChatText(player, _("You are not allowed to run this command"));
			return false;
		}
		ServerWiper::getInstance()->addDatabaseStatsRequest(DB_STATS_REQUEST_SERVER, player);
		return true;
	}
	else if (command == "sw_player_stats")
	{
		if(!admin)
		{
			sendChatText(player, _("You are not allowed to run this command"));
			return false;
		}
		char *arg1 = getEngine()->Cmd_Argv(1);
		if(arg1)
			ServerWiper::getInstance()->addDatabaseStatsRequest(DB_STATS_REQUEST_PLAYER, player, arg1);
		else
		{
			sendChatText(player, _("Not enough arguments, steamID is missing"));
			return false;
		}
		return true;
	}

    return false;
}

Player *ServerWiper::getPlayer(int const& player)
{
    if(player < 0)
        return 0;

    if(players.find(player) != players.end())
        return players[player];

    return 0;
}

PlayerEvent *ServerWiper::getPlayerEvent(int const& player, int const& event)
{
    Player *pPlayer = getPlayer(player);

    if(!pPlayer)
        return 0;

    return pPlayer->getPlayerEvent(event);
}

float ServerWiper::getPlayerFlashDuration(int const& player)
{
    unsigned int offset = OFFSET_FLASH_DURATION;

	IServerUnknown *pUnknown = (IServerUnknown *)getEngine()->PEntityOfEntIndex(player)->GetUnknown();
	if(!pUnknown)
	{
		return 0.0f;
	}

	CBaseEntity *pEntity = pUnknown->GetBaseEntity();

    return (float)*(float *)((char *)pEntity + offset);
}

template <class T>
T ServerWiper::getPlayerAttribute(int const& player, int const& offset)
{
	IServerUnknown *pUnknown = (IServerUnknown *)getEngine()->PEntityOfEntIndex(player)->GetUnknown();
	CBaseEntity *pEntity = pUnknown->GetBaseEntity();

    return (T)*(T *)((char *)pEntity + offset);
}

template <class T>
void ServerWiper::setPlayerAttribute(int const& player, int const& offset, T value)
{
	IServerUnknown *pUnknown = (IServerUnknown *)getEngine()->PEntityOfEntIndex(player)->GetUnknown();
	CBaseEntity *pEntity = pUnknown->GetBaseEntity();

    *(T *)((char *)pEntity + offset) = value;
}

void ServerWiper::reload()
{
    InsultManager::getInstance()->reset();
    InsultManager::getInstance()->loadInsults("insults1.txt", 1);
    InsultManager::getInstance()->loadInsults("insults2.txt", 2);
    InsultManager::getInstance()->loadInsults("insults3.txt", 3);
    GreetingManager::getInstance()->loadGreetings("greetings.txt");
    ApologyManager::getInstance()->loadApologies("apologies.txt");
    AdminManager::getInstance()->loadAdminFile("admins.txt");
    LanguageManager::getInstance()->loadLanguageFile(g_Language.GetString());
	HelpManager::getInstance()->loadHelpMessages();
	TeamManager::getInstance()->loadTeamFile("team.txt");

    META_CONPRINTF("Text and language data reloaded\n");
}

void ServerWiper::onSourceModBan()
{
	IVEngineServer *engine = getEngine();

	if(engine->Cmd_Argc() < 6)
	{
		META_CONPRINTF("Invalid sw_sm_ban call !\n");
		return;
	}

	shString playerSteamID = engine->Cmd_Argv(1);
	shString playerName = engine->Cmd_Argv(2);
	shString adminSteamID = engine->Cmd_Argv(3);
	int banTime = atoi(engine->Cmd_Argv(4));
	shString banReason = engine->Cmd_Argv(5);

	shString banDuration = "Admin ban, duration : ";
	if(banTime == 0)
		banDuration += "permanent";
	else
		banDuration += (itos(banTime) + shString(" minutes"));

	shString bansURL = g_SendBanURL.GetString();
	if(!bansURL.empty())
	{
		URLRequest newRequest;
		newRequest.setURL(bansURL);
		newRequest.addParameter("player", playerName);
		newRequest.addParameter("steamid", playerSteamID);
		newRequest.addParameter("behav_points", "");
		newRequest.addParameter("ban_points", "");
		newRequest.addParameter("reason", banDuration);
		newRequest.addParameter("server", g_ShortServerName.GetString());
		newRequest.addParameter("admin", adminSteamID);
		newRequest.addParameter("extreason", banReason);
		URLManager::getInstance()->addNewRequest(newRequest);
	}
}
