/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#ifndef _INCLUDE_SERVER_WIPER_H
#define _INCLUDE_SERVER_WIPER_H

#include <igameevents.h>

#include "server_wiper_mm.h"
#include "singleton.h"
#include "event.h"
#include "player.h"
#include "iserverplugin.h"
#include "safeStack.h"
#include "util.h"

const int OFFSET_FLASH_DURATION =       4768; //m_flFlashDuration
const int OFFSET_ACCOUNT =              3608; //m_iAccount
const int OFFSET_HAS_NIGHT_VISION =     3653; //m_bHasNightVision
const int OFFSET_HAS_NIGHT_VISION_ON =  3654; //m_bNightVisionOn

class DatabaseStatsRequest;

enum ChatTextType
{
    CHAT_TEXT_NEUTRAL,
    CHAT_TEXT_GOOD,
    CHAT_TEXT_BAD,
};

class PlayerUpdate//Sent by the database thread
{
public:
    shString steamID;
    int behavPoints;
    int banPoints;
    int playtime;
    PlayerDataV1 data;
	int eventOccured[EVENT_TYPE_MAX];
	int kicks;
	int bans;
};

class DummyPlugin : public IServerPluginCallbacks
{
public:
	// Initialize the plugin to run
	// Return false if there is an error during startup.
	bool			Load(	CreateInterfaceFn interfaceFactory, CreateInterfaceFn gameServerFactory  )  {return true;}

	// Called when the plugin should be shutdown
	void			Unload( void )  {}

	// called when a plugins execution is stopped but the plugin is not unloaded
	void			Pause( void )  {}

	// called when a plugin should start executing again (sometime after a Pause() call)
	void			UnPause( void )  {}

	// Returns shString describing current plugin.  e.g., Admin-Mod.
	const char     *GetPluginDescription( void )  {return "";}

	// Called any time a new level is started (after GameInit() also on level transitions within a game)
	void			LevelInit( char const *pMapName )  {}

	// The server is about to activate
	void			ServerActivate( edict_t *pEdictList, int edictCount, int clientMax )  {}

	// The server should run physics/think on all edicts
	void			GameFrame( bool simulating )  {}

	// Called when a level is shutdown (including changing levels)
	void			LevelShutdown( void )  {}

	// Client is going active
	void			ClientActive( edict_t *pEntity )  {}

	// Client is disconnecting from server
	void			ClientDisconnect( edict_t *pEntity )  {}

	// Client is connected and should be put in the game
	void			ClientPutInServer( edict_t *pEntity, char const *playername )  {}

	// Sets the client index for the client who typed the command into their console
	void			SetCommandClient( int index )  {}

	// A player changed one/several replicated cvars (name etc)
	void			ClientSettingsChanged( edict_t *pEdict )  {}

	// Client is connecting to server ( set retVal to false to reject the connection )
	//	You can specify a rejection message by writing it into reject
	PLUGIN_RESULT	ClientConnect( bool *bAllowConnect, edict_t *pEntity, const char *pszName, const char *pszAddress, char *reject, int maxrejectlen )  {return PLUGIN_CONTINUE;}

	// The client has typed a command at the console
	PLUGIN_RESULT	ClientCommand( edict_t *pEntity )  {return PLUGIN_CONTINUE;}

	// A user has had their network id setup and validated
	PLUGIN_RESULT	NetworkIDValidated( const char *pszUserName, const char *pszNetworkID )  {return PLUGIN_CONTINUE;}
};

class ServerWiper : public Singleton<ServerWiper>
{
    friend class Singleton<ServerWiper>;

public:
    ServerWiper();
    ~ServerWiper();

    void setPluginInstance(ServerWiperPlugin *plugin);
    IVEngineServer *getEngine();
    IFileSystem *getFilesystem()    {return plugin->getFilesystem();}
    ServerWiperPlugin *getPlugin()  {return plugin;}
    void newEvent(Event & event);
    void think();
    void lessThink(); // Same as think(), but called less often to prevent lag (patent pending :p)
    void gameEvent(shString const& name, IGameEvent *event);
    int getPlayerIndexByUserID(int const& userID);
    int getPlayerIndexBySteamID(shString const& steamID);
    int UserMessageIndex(const char *messageName);
    void postInit();

    void sendChatText(int const& player, shString const& message, ChatTextType const& type = CHAT_TEXT_NEUTRAL);
    void sendChatTextToAll(shString const& message, ChatTextType const& type = CHAT_TEXT_NEUTRAL);
    void sendBoxText(int const& player, shString const& message, int const& duration);
    void sendBoxTextToAll(shString const& message, int const& duration);
    void sendPlayerChatText(int const& player, shString const& message);

    void playerGotPoints(Event & event, shString const& behavPoints, shString const& banPoints, shString const& health, shString const& money, shString const& reason, int const& coef = 1);

    int getPlayerMoney(int const& player);
    int getPlayerHealth(int const& player);
    void setPlayerMoney(int const& player, int const& money);
    void setPlayerHealth(int const& player, int const& health);

    void slapPlayer(int const& player, int const& damage);
    void slayPlayer(int const& player);
    void kickPlayer(int const& player, shString const& reason);
    void banPlayer(int const& player, shString const& reason, int const& duration);

    void showStatusToPlayer(int const& player);
    void showWelcomeMessage(int const& player);

    shString getServerName();
    shString getServerPort();
    shString getMaxplayers();
    shString getMapName();

    bool isPlayerAlive(int const& player);

    void hideSayMessage();
    bool shouldShowSayMessage();

    void advert();
    void helpMessage();
    void statusChanged(bool on);
    void newRound();

    void resetAllPlayers();
    void resetPlayer(shString const& steamID);

    void addPlayerUpdate(shString const& steamid, int const& behav, int const& ban, int const& playtime, PlayerDataV1 const& data, int eventOccured[EVENT_TYPE_MAX], int const& kicks, int const& bans);
	void addDatabaseStatsAnswer(DatabaseStatsRequest const& answer);
	void addDatabaseStatsRequest(int const& type, int const& admin, shString const& player = "");

    void checkPlayerName(int const& player);

    bool gotPlayerCommand(int const& player, shString const& command);

    float getPlayerFlashDuration(int const& player);

    template <class T>
    T getPlayerAttribute(int const& player, int const& offset);
    template <class T>
    void setPlayerAttribute(int const& player, int const& offset, T value);

    Player *getPlayer(int const& player);
    PlayerEvent *getPlayerEvent(int const& player, int const& event);
    ConVar* getConVar(shString const& varName);

    void reload();
	void onSourceModBan();

private:
    void gotPlayerSteamID(int const& index);
    int getPlayerInteger(int const& player, shString const& variable);
    void setPlayerInteger(int const& player, shString const& variable, int const& value);

    int messageSayText;
    int messageVGUI;

    ServerWiperPlugin *plugin;
    shHash<int, Player *> players;
    shHash<int, int> userIndexByUserID;
    shVector<int> clientsPending;
    shVector<int> flashedPlayers;
    float nextPendingCheck;
    time_t lastPlayerConnected;
    time_t lastAdvert;
    time_t lastHelpMessage;
    time_t lastRoundStart;
    int lastPlayerConnectedIndex;
    bool showSayMessage;
    int hasUpdates;
	int waitingForDatabaseStatsAnswer;
    time_t lastRegistration;
    safeStack<PlayerUpdate> playerUpdates;
	safeStack<DatabaseStatsRequest> databaseStatsAnswer;
    int lastTerroristIndex;
    int lastCounterTerroristIndex;
};

class VfuncEmptyClass {};

#endif //_INCLUDE_SERVER_WIPER_H



