/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#include "serverWiper.h"
#include "datamap.h"
#define GAME_DLL 1
#include "cbase.h"
#include "MRecipientFilter.h"
#include "bitbuf.h"
#include "insults.h"
#include "greetings.h"
#include "apologies.h"
#include "url.h"
#include "cvars.h"
#include <time.h>
#include "language.h"
#include "database.h"
#include "menu.h"
#include "dt_send.h"
#include "team.h"

#ifdef _WIN32
#pragma warning(disable: 4018)
#pragma warning(disable: 4244)
#pragma warning(disable: 4005)
#endif

void ServerWiper::newEvent(Event & event)
{
    if(!CVAR_BOOL(g_ServerWiperEnabled) && event.getType() != EVENT_TYPE_PLAYER_CONNECTED
										&& event.getType() != EVENT_TYPE_PLAYER_DISCONNECTED
										&& event.getType() != EVENT_TYPE_PLAYER_JOINTEAM
										&& event.getType() != EVENT_TYPE_PLAYER_CHANGED_NAME)
        return;

    if(event.getAuthor() == -1)
        return;

    if(!getPlayer(event.getAuthor()) && event.getType() != EVENT_TYPE_PLAYER_CONNECTED)
        return;

    if(event.getType() == EVENT_TYPE_PLAYER_CONNECTED)
    {
        Player *newPlayer = new Player(event.getAuthor(), event.getEdict());
        shString steamID = getEngine()->GetPlayerNetworkIDString(event.getEdict());
        bool hasSteamID = true;
        if(steamID == "STEAM_ID_PENDING")
        {
            hasSteamID = false;
            clientsPending.push_back(event.getAuthor());
        }
        else
        {
            newPlayer->setHasSteamID(true);
            newPlayer->setSteamID(steamID);
        }
        userIndexByUserID[getEngine()->GetPlayerUserId(event.getEdict())] = event.getAuthor();
        newPlayer->setUserID(getEngine()->GetPlayerUserId(event.getEdict()));

        if(newPlayer->getUserID() == -1)
        {
            if(CVAR_INT(g_DebugLevel))
            {
                META_LOG(g_PLAPI,"Got a client with an edict which is not in the player list");
            }
        }

        newPlayer->setName(event.getText());
        newPlayer->setIP(event.getSecondText());
        newPlayer->setSteamID(steamID);
        players[event.getAuthor()] = newPlayer;

        if(hasSteamID)
        {
            gotPlayerSteamID(event.getAuthor());
        }

        lastPlayerConnected = time(NULL);
        lastPlayerConnectedIndex = event.getAuthor();

        getPlayer(event.getAuthor())->setConnectionTime(time(NULL));
        getPlayer(event.getAuthor())->setHadWelcomeMessage(false);

        checkPlayerName(event.getAuthor());
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_DISCONNECTED)
    {
        if(CVAR_BOOL(g_ServerWiperEnabled) && CVAR_BOOL(g_PunishLeavers) && CVAR_INT(g_PunishLeaversDelay) > 0 && event.getText() == "Disconnect by user.") // Normal player quit
        {
            Player *player = getPlayer(event.getAuthor());
            if(player)
            {
                time_t lastBadThing = getMax(player->getPlayerEvent(EVENT_TYPE_PLAYER_INSULT)->getLastOccured(), player->getPlayerEvent(EVENT_TYPE_PLAYER_TK)->getLastOccured());
                if(time(NULL) - lastBadThing <= CVAR_INT(g_PunishLeaversDelay))
                {
                    bool ok = true;
                    if(lastBadThing == player->getPlayerEvent(EVENT_TYPE_PLAYER_INSULT)->getLastOccured() && player->getPlayerEvent(EVENT_TYPE_PLAYER_APOLOGIES_FOR_INSULT)->getLastOccured() > player->getPlayerEvent(EVENT_TYPE_PLAYER_INSULT)->getLastOccured())
                        ok = false;
                    if(lastBadThing == player->getPlayerEvent(EVENT_TYPE_PLAYER_TK)->getLastOccured() && player->getPlayerEvent(EVENT_TYPE_PLAYER_APOLOGIES_FOR_TK)->getLastOccured() > player->getPlayerEvent(EVENT_TYPE_PLAYER_TK)->getLastOccured())
                        ok = false;
                    if(ok)
                    {
                        Event myNewEvent;
                        myNewEvent.setType(EVENT_TYPE_PLAYER_LEAVER);
                        myNewEvent.setAuthor(event.getAuthor());
                        playerGotPoints(myNewEvent,
                            "0",
                            g_PunishLeaversBanPoints.GetString(),
                            "0",
                            "0",
                            _("Leaving without apologies"));
                    }
                }
            }
        }

        //Check if a pending player disconnected
        for(shVector<int>::iterator it = clientsPending.begin(); it != clientsPending.end(); )
        {
            if (*it == event.getAuthor())
                it = clientsPending.erase(it);
            else
                ++it;
        }

        //Check if a menu was open
        if(getPlayer(event.getAuthor())->getActualMenu() != -1)
        {
            MenuManager::getInstance()->onPlayerMenuSelect(event.getAuthor(), 10);//Close event
        }

        //Save the player in the database
        DatabaseRequest newRequest;
        newRequest.setType(DB_REQUEST_PLAYER_SAVE);
        newRequest.setPlayer(*getPlayer(event.getAuthor()));
        newRequest.setData(*getPlayer(event.getAuthor())->getData());
        DatabaseManager::getInstance()->addNewRequest(newRequest);

        delete players[event.getAuthor()];

        players.erase(event.getAuthor());
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_JOINTEAM)
    {
        Player *player = getPlayer(event.getAuthor());
        player->setTeam(event.getIntValue());
        player->setUsedHostageCount(0);

        //Save the last T or CT for saying text in red or blue (yes, this is a big hack :) )
        if(event.getIntValue() == TEAM_TERRORIST)
        {
            lastTerroristIndex = event.getAuthor();
            if(lastCounterTerroristIndex == lastTerroristIndex)
                lastCounterTerroristIndex = 0;
        }
        else if(event.getIntValue() == TEAM_COUNTER_TERRORIST)
        {
            lastCounterTerroristIndex = event.getAuthor();
            if(lastTerroristIndex == lastCounterTerroristIndex)
                lastTerroristIndex = 0;
        }
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_SAY)
    {
        shString lowerText = getLowerString(event.getText());
        if(lowerText == shString(g_StatusCommand.GetString()) && CVAR_BOOL(g_AllowStatus))
        {
            showStatusToPlayer(event.getAuthor());
            if(CVAR_BOOL(g_HideSayCommands))
                hideSayMessage();
        }

        if(CVAR_BOOL(g_Insult))
        {
            shString insult;
            int insultLevel = InsultManager::getInstance()->getInsultLevel(event.getText(), insult);
            if(insultLevel > 0)
            {
                if(CVAR_BOOL(g_HideSayInsults))
				{
                    hideSayMessage();
					sendChatTextToAll(_("%s said an {insult} and has been punished", getPlayer(event.getAuthor())->getName().c_str()), CHAT_TEXT_BAD);
				}

                if(CVAR_INT(g_DebugLevel))
                    META_LOG(g_PLAPI,"Player %s (%s) said an insult : \"%s\" in text \"%s\"", getPlayer(event.getAuthor())->getName().c_str(), getPlayer(event.getAuthor())->getSteamID().c_str(), insult.c_str(), event.getText().c_str());

                Event myNewEvent;
                myNewEvent.setType(EVENT_TYPE_PLAYER_INSULT);
                myNewEvent.setAuthor(event.getAuthor());
                myNewEvent.setText(insult);
                myNewEvent.setSecondText(event.getText());
                myNewEvent.setIntValue(insultLevel);
                ServerWiper::getInstance()->newEvent(myNewEvent);
                return;
            }
        }

        if(CVAR_BOOL(g_Greetings) && time(NULL) - getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_GREETINGS)->getLastOccured() >= CVAR_INT(g_GreetingsDelay)
                                && time(NULL) - lastPlayerConnected < CVAR_INT(g_GreetingsTime)
                                && event.getAuthor() != lastPlayerConnectedIndex)
        {
            shString greeting;
            if(GreetingManager::getInstance()->isGreeting(event.getText(), greeting))
            {
                Event myNewEvent;
                myNewEvent.setType(EVENT_TYPE_PLAYER_GREETINGS);
                myNewEvent.setAuthor(event.getAuthor());
                myNewEvent.setText(greeting);
                ServerWiper::getInstance()->newEvent(myNewEvent);
                return;
            }
        }

        if(CVAR_BOOL(g_ApologyTeamAttack) || CVAR_BOOL(g_ApologyTeamKill) || CVAR_BOOL(g_ApologyInsult) || CVAR_BOOL(g_ApologyTeamFlash))
        {
            shString apology;
            if(ApologyManager::getInstance()->isApology(event.getText(), apology))
            {
                Player *player = getPlayer(event.getAuthor());
                time_t timeSinceLastTA = getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_TA)->getLastOccured();
                time_t timeSinceLastTK = getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_TK)->getLastOccured();
                time_t timeSinceLastInsult = getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_INSULT)->getLastOccured();
                time_t timeSinceLastTF = getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_TEAM_FLASH)->getLastOccured();
                time_t timeSinceLastBadEvent = getMax(timeSinceLastTA, timeSinceLastTK, timeSinceLastInsult, timeSinceLastTF);

                if(timeSinceLastTA == timeSinceLastBadEvent
                    && CVAR_BOOL(g_ApologyTeamAttack)
                    && player->getPlayerEvent(EVENT_TYPE_PLAYER_TA)->getStatus() == EVENT_STATUS_OCCURED
                    && (time(NULL) - getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_APOLOGIES_FOR_TA)->getLastOccured() >= CVAR_INT(g_TeamAttackApologyDelay) || !CVAR_BOOL(g_UseDelaysToLimitApologies))
                    && time(NULL) - getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_TA)->getLastOccured() < CVAR_INT(g_TeamAttackApologyTime))
                {//Apology for a TA
                    player->getPlayerEvent(EVENT_TYPE_PLAYER_TA)->setStatus(EVENT_STATUS_APOLOGIZED);
                    Event myNewEvent;
                    myNewEvent.setType(EVENT_TYPE_PLAYER_APOLOGIES_FOR_TA);
                    myNewEvent.setAuthor(event.getAuthor());
                    myNewEvent.setText(apology);
                    ServerWiper::getInstance()->newEvent(myNewEvent);
                }
                if(timeSinceLastTK == timeSinceLastBadEvent
                    && CVAR_BOOL(g_ApologyTeamKill)
                    && player->getPlayerEvent(EVENT_TYPE_PLAYER_TK)->getStatus() == EVENT_STATUS_OCCURED
                    && (time(NULL) - getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_APOLOGIES_FOR_TK)->getLastOccured() >= CVAR_INT(g_TeamKillApologyDelay) || !CVAR_BOOL(g_UseDelaysToLimitApologies))
                    && time(NULL) - getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_TK)->getLastOccured() < CVAR_INT(g_TeamKillApologyTime))
                {//Apology for a TK
                    player->getPlayerEvent(EVENT_TYPE_PLAYER_TK)->setStatus(EVENT_STATUS_APOLOGIZED);
                    Event myNewEvent;
                    myNewEvent.setType(EVENT_TYPE_PLAYER_APOLOGIES_FOR_TK);
                    myNewEvent.setAuthor(event.getAuthor());
                    myNewEvent.setText(apology);
                    ServerWiper::getInstance()->newEvent(myNewEvent);
                }
                if(timeSinceLastInsult == timeSinceLastBadEvent
                    && CVAR_BOOL(g_ApologyInsult)
                    && player->getPlayerEvent(EVENT_TYPE_PLAYER_INSULT)->getStatus() == EVENT_STATUS_OCCURED
                    && (time(NULL) - getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_APOLOGIES_FOR_INSULT)->getLastOccured() >= CVAR_INT(g_InsultApologyDelay) || !CVAR_BOOL(g_UseDelaysToLimitApologies))
                    && time(NULL) - getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_INSULT)->getLastOccured() < CVAR_INT(g_InsultApologyTime))
                {//Apology for an insult
                    player->getPlayerEvent(EVENT_TYPE_PLAYER_INSULT)->setStatus(EVENT_STATUS_APOLOGIZED);
                    Event myNewEvent;
                    myNewEvent.setType(EVENT_TYPE_PLAYER_APOLOGIES_FOR_INSULT);
                    myNewEvent.setAuthor(event.getAuthor());
                    myNewEvent.setText(apology);
                    ServerWiper::getInstance()->newEvent(myNewEvent);
                }
                if(timeSinceLastTF == timeSinceLastBadEvent
                    && CVAR_BOOL(g_ApologyTeamFlash)
                    && player->getPlayerEvent(EVENT_TYPE_PLAYER_TEAM_FLASH)->getStatus() == EVENT_STATUS_OCCURED
                    && (time(NULL) - getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_APOLOGIES_FOR_TF)->getLastOccured() >= CVAR_INT(g_TeamFlashApologyDelay) || !CVAR_BOOL(g_UseDelaysToLimitApologies))
                    && time(NULL) - getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_TEAM_FLASH)->getLastOccured() < CVAR_INT(g_TeamFlashApologyTime))
                {//Apology for a TF
                    player->getPlayerEvent(EVENT_TYPE_PLAYER_TEAM_FLASH)->setStatus(EVENT_STATUS_APOLOGIZED);
                    Event myNewEvent;
                    myNewEvent.setType(EVENT_TYPE_PLAYER_APOLOGIES_FOR_TF);
                    myNewEvent.setAuthor(event.getAuthor());
                    myNewEvent.setText(apology);
                    ServerWiper::getInstance()->newEvent(myNewEvent);
                }
            }
        }
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_ATTACKED)
    {
        if(event.getAuthor() == -1 || event.getVictim() == -1)
            return; // If the victim or the attacker are bots, ignore

        if(event.getAuthor() == event.getVictim())
            return; // Self inflicted

        if(getPlayer(event.getAuthor())->getTeam() != getPlayer(event.getVictim())->getTeam())
            return; // Not a team attack

        Event myNewEvent;
        myNewEvent.setType(EVENT_TYPE_PLAYER_TA);
        myNewEvent.setAuthor(event.getAuthor());
        myNewEvent.setVictim(event.getVictim());
        myNewEvent.setIntValue(event.getIntValue());
        ServerWiper::getInstance()->newEvent(myNewEvent);
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_KILLED)
    {
        if(event.getVictim() != -1)
        {
            if(CVAR_BOOL(g_ShowStatusOnDeath))
            {
                showStatusToPlayer(event.getVictim());
            }
        }

        if(event.getAuthor() == -1 || event.getVictim() == -1)
            return; // If the victim or the attacker are bots, ignore

        if(event.getAuthor() == event.getVictim())
        {
            Event myNewEvent;
            myNewEvent.setType(EVENT_TYPE_PLAYER_SUICIDED);
            myNewEvent.setAuthor(event.getAuthor());
            ServerWiper::getInstance()->newEvent(myNewEvent);
            return;
        }

        if(getPlayer(event.getAuthor())->getTeam() != getPlayer(event.getVictim())->getTeam())
            return; // Not a team attack

        Event myNewEvent;
        myNewEvent.setType(EVENT_TYPE_PLAYER_TK);
        myNewEvent.setAuthor(event.getAuthor());
        myNewEvent.setVictim(event.getVictim());
        myNewEvent.setIntValue(1);//Allow forgive menu
        ServerWiper::getInstance()->newEvent(myNewEvent);
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_TA)
    {
        if(!CVAR_BOOL(g_TeamAttack))
            return;

        if(time(NULL) - getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_TA)->getLastOccured() < CVAR_INT(g_TeamAttackDelay))
            return;

        if(!CVAR_BOOL(g_CountTAWhenDead) && !isPlayerAlive(event.getAuthor()))
            return;

        playerGotPoints(event,
                        g_BehaviourPointsTeamAttack.GetString(),
                        g_BanPointsTeamAttack.GetString(),
                        g_HealthTeamAttack.GetString(),
                        g_MoneyTeamAttack.GetString(),
                        _("Team attack"));
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_TK)
    {
        if(!CVAR_BOOL(g_TeamKill))
            return;

        if(!CVAR_BOOL(g_CountTKWhenDead) && !isPlayerAlive(event.getAuthor()))
            return;

        getPlayer(event.getVictim())->setLastTKKiller(event.getAuthor());

        if(!CVAR_BOOL(g_ForgiveTkMenu) || event.getIntValue() == 0)
        {
            playerGotPoints(event,
                            g_BehaviourPointsTeamKill.GetString(),
                            g_BanPointsTeamKill.GetString(),
                            g_HealthTeamKill.GetString(),
                            g_MoneyTeamKill.GetString(),
                            _("Team kill"));
        }
        else if(event.getIntValue() == 1)//Do not punish now but wait for the answer of the killed player
        {
            MenuManager::getInstance()->showForgiveMenu(event.getVictim(), event.getAuthor(), CVAR_INT(g_ForgiveTkMenuTime));
        }
    }
    else if(event.getType() == EVENT_TYPE_MENU_PUNISH)
    {
		Player *victim = getPlayer(event.getVictim());
		Player *author = getPlayer(event.getAuthor());

        if(!victim)
            return;

		if(author)
		{
			shString message = _("%s has {punished} %s for team killing him", victim->getName().c_str(), author->getName().c_str());
			sendChatTextToAll(message, CHAT_TEXT_BAD);
		}

		if(!author)
		{
			//Author is disconnected or has been kicked, but we have his steamid
			shString authorSteamID = victim->getLastTKKillerSteamID();
			//todo
		}
		else
		{
			Event myNewEvent;
			myNewEvent.setType(EVENT_TYPE_PLAYER_TK);
			myNewEvent.setAuthor(event.getAuthor());
			myNewEvent.setVictim(event.getVictim());
			myNewEvent.setIntValue(0);//Do not accept any forgive menu
			ServerWiper::getInstance()->newEvent(myNewEvent);
		}
    }
    else if(event.getType() == EVENT_TYPE_MENU_FORGIVE)
    {
        if(!getPlayer(event.getVictim()) || !getPlayer(event.getAuthor()))
            return;

		shString message = _("%s has {forgiven} %s for team killing him", getPlayer(event.getVictim())->getName().c_str(), getPlayer(event.getAuthor())->getName().c_str());
        sendChatTextToAll(message, CHAT_TEXT_GOOD);
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_PLANTED_BOMB)
    {
        if(!CVAR_BOOL(g_BombPlant))
            return;

        playerGotPoints(event,
                        g_BehaviourPointsBombPlant.GetString(),
                        g_BanPointsBombPlant.GetString(),
                        g_HealthBombPlant.GetString(),
                        g_MoneyBombPlant.GetString(),
                        _("Bomb planting"));
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_DEFUSED_BOMB)
    {
        if(!CVAR_BOOL(g_BombDefuse))
            return;

        playerGotPoints(event,
                        g_BehaviourPointsBombDefuse.GetString(),
                        g_BanPointsBombDefuse.GetString(),
                        g_HealthBombDefuse.GetString(),
                        g_MoneyBombDefuse.GetString(),
                        _("Bomb defusing"));
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_KILLED_HOSTAGE)
    {
        if(!CVAR_BOOL(g_HostageKill))
            return;

        if(!CVAR_BOOL(g_CountHostageKillWhenDead) && !isPlayerAlive(event.getAuthor()))
            return;

        playerGotPoints(event,
                        g_BehaviourPointsHostageKill.GetString(),
                        g_BanPointsHostageKill.GetString(),
                        g_HealthHostageKill.GetString(),
                        g_MoneyHostageKill.GetString(),
                        _("Hostage killing"));
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_USED_HOSTAGE)
    {
        if(!CVAR_BOOL(g_HostageFollow))
            return;

        if(getPlayer(event.getAuthor())->getUsedHostageCount() >= CVAR_INT(g_MaxHostageFollowInRound))
            return;

        playerGotPoints(event,
                        g_BehaviourPointsHostageFollow.GetString(),
                        g_BanPointsHostageFollow.GetString(),
                        g_HealthHostageFollow.GetString(),
                        g_MoneyHostageFollow.GetString(),
                        _("Hostage using"));

        getPlayer(event.getAuthor())->setUsedHostageCount(getPlayer(event.getAuthor())->getUsedHostageCount() + 1);
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_RESCUED_HOSTAGE)
    {
        if(!CVAR_BOOL(g_HostageRescue))
            return;

        playerGotPoints(event,
                        g_BehaviourPointsHostageRescue.GetString(),
                        g_BanPointsHostageRescue.GetString(),
                        g_HealthHostageRescue.GetString(),
                        g_MoneyHostageRescue.GetString(),
                        _("Hostage rescuing"));
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_ATTACKED_HOSTAGE)
    {
        if(!CVAR_BOOL(g_HostageAttack))
            return;

        if(time(NULL) - getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_ATTACKED_HOSTAGE)->getLastOccured() < CVAR_INT(g_HostageAttackDelay))
            return;

        if(!CVAR_BOOL(g_CountHostageAttackWhenDead) && !isPlayerAlive(event.getAuthor()))
            return;

        playerGotPoints(event,
                        g_BehaviourPointsHostageAttack.GetString(),
                        g_BanPointsHostageAttack.GetString(),
                        g_HealthHostageAttack.GetString(),
                        g_MoneyHostageAttack.GetString(),
                        _("Hostage attacking"));
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_SUICIDED)
    {
        //Not used yet
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_CHANGED_NAME)
    {
        getPlayer(event.getAuthor())->setName(event.getText());
        checkPlayerName(event.getAuthor());
		TeamManager::getInstance()->checkPlayer(event.getAuthor());
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_INSULT)
    {
        if(!CVAR_BOOL(g_Insult))
            return;

        shString message = _("Insult of level");
        message += " ";
        message += itos(event.getIntValue());
        playerGotPoints(event,
                        g_BehaviourPointsInsult.GetString(),
                        g_BanPointsInsult.GetString(),
                        g_HealthInsult.GetString(),
                        g_MoneyInsult.GetString(),
                        message);
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_GREETINGS)
    {
        if(!CVAR_BOOL(g_Greetings))
            return;

        playerGotPoints(event,
                        g_BehaviourPointsGreetings.GetString(),
                        g_BanPointsGreetings.GetString(),
                        g_HealthGreetings.GetString(),
                        g_MoneyGreetings.GetString(),
                        _("New player greeting"));
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_APOLOGIES_FOR_TA)
    {
        if(!CVAR_BOOL(g_ApologyTeamAttack))
            return;

        playerGotPoints(event,
                        g_BehaviourPointsTeamAttackApology.GetString(),
                        g_BanPointsTeamAttackApology.GetString(),
                        g_HealthTeamAttackApology.GetString(),
                        g_MoneyTeamAttackApology.GetString(),
                        _("Apology after team attack"));
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_APOLOGIES_FOR_TK)
    {
        if(!CVAR_BOOL(g_ApologyTeamKill))
            return;

        playerGotPoints(event,
                        g_BehaviourPointsTeamKillApology.GetString(),
                        g_BanPointsTeamKillApology.GetString(),
                        g_HealthTeamKillApology.GetString(),
                        g_MoneyTeamKillApology.GetString(),
                        _("Apology after team kill"));
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_APOLOGIES_FOR_INSULT)
    {
        if(!CVAR_BOOL(g_ApologyInsult))
            return;

        playerGotPoints(event,
                        g_BehaviourPointsInsultApology.GetString(),
                        g_BanPointsInsultApology.GetString(),
                        g_HealthInsultApology.GetString(),
                        g_MoneyInsultApology.GetString(),
                        _("Apology after insult"));
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_TEAM_FLASH)
    {
        if(!CVAR_BOOL(g_TeamFlash))
            return;

        if(event.getIntValue() < CVAR_INT(g_TeamFlashMinimumPlayers))
            return;

        if(!CVAR_BOOL(g_CountTFWhenDead) && !isPlayerAlive(event.getAuthor()))
            return;

        shString reason = _("Team flash (%d teammates)", event.getIntValue());
        playerGotPoints(event,
                        g_BehaviourPointsTeamFlash.GetString(),
                        g_BanPointsTeamFlash.GetString(),
                        g_HealthTeamFlash.GetString(),
                        g_MoneyTeamFlash.GetString(),
                        reason,
                        event.getIntValue());

		shString message = _("%s made a {team flash} ({%d} teammate(s)) and has been punished", getPlayer(event.getAuthor())->getName().c_str(), event.getIntValue());
		sendChatTextToAll(message, CHAT_TEXT_BAD);
    }
    else if(event.getType() == EVENT_TYPE_PLAYER_APOLOGIES_FOR_TF)
    {
        if(!CVAR_BOOL(g_ApologyTeamFlash))
            return;

        playerGotPoints(event,
                        g_BehaviourPointsTeamFlashApology.GetString(),
                        g_BanPointsTeamFlashApology.GetString(),
                        g_HealthTeamFlashApology.GetString(),
                        g_MoneyTeamFlashApology.GetString(),
                        _("Apology after team flash"),
                        getPlayerEvent(event.getAuthor(), EVENT_TYPE_PLAYER_TEAM_FLASH)->getCount());
    }
    else if(CVAR_INT(g_DebugLevel))
    {
        META_LOG(g_PLAPI,"Got an unknown event : %d", event.getType());
    }
}

void ServerWiper::gameEvent(shString const& name, IGameEvent *event)
{
    if(!event)
        return;

    int playerIndex = getPlayerIndexByUserID(event->GetInt("userid"));

    if(name != "round_start" && playerIndex == -1)
        return;

    if(name == "player_team")
    {
        Event myNewEvent;
        myNewEvent.setType(EVENT_TYPE_PLAYER_JOINTEAM);
        myNewEvent.setAuthor(playerIndex);
        myNewEvent.setIntValue(event->GetInt("team"));
        ServerWiper::getInstance()->newEvent(myNewEvent);
    }
    else if(name == "player_disconnect")
    {
        Event myNewEvent;
        myNewEvent.setType(EVENT_TYPE_PLAYER_DISCONNECTED);
        myNewEvent.setAuthor(playerIndex);
        myNewEvent.setText(event->GetString("reason"));
        ServerWiper::getInstance()->newEvent(myNewEvent);
    }
    else if(name == "player_hurt")
    {
        Event myNewEvent;
        myNewEvent.setType(EVENT_TYPE_PLAYER_ATTACKED);
        myNewEvent.setAuthor(getPlayerIndexByUserID(event->GetInt("attacker")));
        myNewEvent.setVictim(playerIndex);
        myNewEvent.setIntValue(event->GetInt("dmg_health"));
        ServerWiper::getInstance()->newEvent(myNewEvent);
    }
    else if(name == "player_death")
    {
        Event myNewEvent;
        myNewEvent.setType(EVENT_TYPE_PLAYER_KILLED);
        myNewEvent.setAuthor(getPlayerIndexByUserID(event->GetInt("attacker")));
        myNewEvent.setVictim(playerIndex);
        ServerWiper::getInstance()->newEvent(myNewEvent);
    }
    else if(name == "bomb_planted")
    {
        Event myNewEvent;
        myNewEvent.setType(EVENT_TYPE_PLAYER_PLANTED_BOMB);
        myNewEvent.setAuthor(playerIndex);
        ServerWiper::getInstance()->newEvent(myNewEvent);
    }
    else if(name == "bomb_defused")
    {
        Event myNewEvent;
        myNewEvent.setType(EVENT_TYPE_PLAYER_DEFUSED_BOMB);
        myNewEvent.setAuthor(playerIndex);
        ServerWiper::getInstance()->newEvent(myNewEvent);
    }
    else if(name == "hostage_killed")
    {
        Event myNewEvent;
        myNewEvent.setType(EVENT_TYPE_PLAYER_KILLED_HOSTAGE);
        myNewEvent.setAuthor(playerIndex);
        ServerWiper::getInstance()->newEvent(myNewEvent);
    }
    else if(name == "hostage_follows")
    {
        Event myNewEvent;
        myNewEvent.setType(EVENT_TYPE_PLAYER_USED_HOSTAGE);
        myNewEvent.setAuthor(playerIndex);
        ServerWiper::getInstance()->newEvent(myNewEvent);
    }
    else if(name == "hostage_rescued")
    {
        Event myNewEvent;
        myNewEvent.setType(EVENT_TYPE_PLAYER_RESCUED_HOSTAGE);
        myNewEvent.setAuthor(playerIndex);
        ServerWiper::getInstance()->newEvent(myNewEvent);
    }
    else if(name == "hostage_hurt")
    {
        Event myNewEvent;
        myNewEvent.setType(EVENT_TYPE_PLAYER_ATTACKED_HOSTAGE);
        myNewEvent.setAuthor(playerIndex);
        ServerWiper::getInstance()->newEvent(myNewEvent);
    }
    else if(name == "player_changename")
    {
        Event myNewEvent;
        myNewEvent.setType(EVENT_TYPE_PLAYER_CHANGED_NAME);
        myNewEvent.setAuthor(playerIndex);
        myNewEvent.setText(event->GetString("newname"));
        ServerWiper::getInstance()->newEvent(myNewEvent);
    }
    else if(name == "flashbang_detonate")
    {
        if(flashedPlayers.size() > 0)
        {
            int playersFlashed = 0;
            int playerWhoFlashed = playerIndex;
            for(int i=0;i<flashedPlayers.size();i++)
            {
                if(flashedPlayers[i] != -1 && flashedPlayers[i] != playerWhoFlashed)
                {
                    if(getPlayer(playerWhoFlashed)->getTeam() == getPlayer(flashedPlayers[i])->getTeam())
                    {
                        playersFlashed++;
                    }
                }
            }

            if(playersFlashed > 0)
            {
                Event myNewEvent;
                myNewEvent.setType(EVENT_TYPE_PLAYER_TEAM_FLASH);
                myNewEvent.setAuthor(playerWhoFlashed);
                myNewEvent.setIntValue(playersFlashed);
                ServerWiper::getInstance()->newEvent(myNewEvent);
            }

            flashedPlayers.clear();
        }
    }
    else if(name == "player_spawn")
    {
        Player *player = getPlayer(playerIndex);
        if(!player)
                return;
        if(player->getData()->gotKicked && CVAR_BOOL(g_RemindPlayersWithBadBehaviour))
        {
            player->getData()->gotKicked = false;
            getPlayer(playerIndex)->setHadWelcomeMessage(true);//Skip welcome message : player_spawn is called 2 times by the engine...
            shString message = _("You have been kicked for having a bad behaviour, please be careful !");
            for(int i=0;i<=4;i++)
            {
                sendChatText(playerIndex, message);
            }
            sendBoxText(playerIndex, message, CVAR_INT(g_MessageBoxTime));

            //Play a sound to warn the player
            if(strcmp(g_SoundToPlayWhenReminding.GetString(), "") != 0)
            {
                edict_t * entity = ServerWiper::getInstance()->getEngine()->PEntityOfEntIndex(playerIndex);
                IServerUnknown * pServerUnknown = entity->GetUnknown();
                CBaseEntity * pEntity = pServerUnknown ->GetBaseEntity();
                MRecipientFilter filter;
                filter.AddRecipient(playerIndex);
                Vector pos = pEntity->GetCollideable()->GetCollisionOrigin();
                getPlugin()->getEngineSound()->PrecacheSound(g_SoundToPlayWhenReminding.GetString(), true); // always precache, because IsSoundPrecached doesn't work ? FIXME
                getPlugin()->getEngineSound()->EmitSound((IRecipientFilter &)filter, playerIndex, CHAN_AUTO, g_SoundToPlayWhenReminding.GetString(), 1.0, ATTN_NORM, 0, PITCH_NORM, &pos, 0, 0, true, 0, playerIndex);
            }
        }
        else
        {
            if(CVAR_BOOL(g_WelcomeMessage))
            {
                if(!getPlayer(playerIndex)->getHadWelcomeMessage())
                {
                    getPlayer(playerIndex)->setHadWelcomeMessage(true);
                    showWelcomeMessage(getPlayer(playerIndex)->getIndex());
                }
            }
        }
    }
    else if(name == "player_blind")
    {
        int victim = playerIndex;
        if(getPlayerFlashDuration(victim) >= (float)CVAR_INT(g_TeamFlashMinimumDuration) && isPlayerAlive(victim))
            flashedPlayers.push_back(victim);
    }
    else if(name == "round_start")
    {
        newRound();
    }
    lessThink(); // We lessThink here, since this is called often, but not every game frame which is too much and will slowdown the server
}
