/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#include <oslink.h>
#include "server_wiper_mm.h"
#include "cvars.h"
#include "serverWiper.h"
#include "sh_string.h"
#define GAME_DLL 1
#include "cbase.h"
#include "icvar.h"
#include "cvars.h"
#include "language.h"
#include "menu.h"
#include "url.h"

#ifdef _WIN32
#pragma warning(disable: 4005)
#endif

ServerWiperPlugin g_ServerWiper;
ConCommandAccessor g_Accessor;
MyListener g_Listener;

PLUGIN_EXPOSE(ServerWiperPlugin, g_ServerWiper);

//This has all of the necessary hook declarations.  Read it!
#include "meta_hooks.h"

#define	FIND_IFACE(func, assn_var, num_var, name, type) \
	do { \
		if ( (assn_var=(type)((ismm->func())(name, NULL))) != NULL ) { \
			num = 0; \
			break; \
		} \
		if (num >= 999) \
			break; \
	} while ( num_var=ismm->FormatIface(name, sizeof(name)-1) ); \
	if (!assn_var) { \
		if (error) \
			snprintf(error, maxlen, "Could not find interface %s", name); \
		return false; \
	}

void ServerWiperPlugin::LogPrintHandler(const char *msg)
{
    if(!CVAR_BOOL(g_SendAdminBans))
        RETURN_META(MRES_IGNORED);

    //Dirty method to get mani admin plugin bans
    if(CVAR_BOOL(g_DetectManiAdminPluginBans) && strncmp(msg, "[MANI_ADMIN_PLUGIN]", 19) == 0 && strstr(msg, "Banned (By Admin)"))
    {
        char *adminSteamIDStart = strstr((char *)msg, "STEAM_");
        char *adminSteamIDEnd = 0;
        int steamIDLength = 0;
        shString adminSteamID;
        shString playerSteamID;
        char buffer[32] = "";
        if(!adminSteamIDStart)
            RETURN_META(MRES_HANDLED);
        adminSteamIDEnd = strchr(adminSteamIDStart,']');
        if(!adminSteamIDEnd)
            RETURN_META(MRES_HANDLED);
        steamIDLength = adminSteamIDEnd - adminSteamIDStart;
        strncpy(buffer, adminSteamIDStart, steamIDLength);

        adminSteamID = buffer;

        char *playerSteamIDStart = strstr(adminSteamIDEnd, "STEAM_");
        char *playerSteamIDEnd = 0;

        if(!playerSteamIDStart)
            RETURN_META(MRES_HANDLED);
        playerSteamIDEnd = strchr(playerSteamIDStart,']');
        if(!playerSteamIDEnd)
            RETURN_META(MRES_HANDLED);
        steamIDLength = playerSteamIDEnd - playerSteamIDStart;
        strncpy(buffer, playerSteamIDStart, steamIDLength);

        playerSteamID = buffer;

        int player = ServerWiper::getInstance()->getPlayerIndexBySteamID(playerSteamID);
        if(player == -1)
            RETURN_META(MRES_HANDLED);

        shString bansURL = g_SendBanURL.GetString();
        if(!bansURL.empty())
        {
            Player *pPlayer = ServerWiper::getInstance()->getPlayer(player);
            URLRequest newRequest;
            newRequest.setURL(bansURL);
            newRequest.addParameter("player", pPlayer->getName());
            newRequest.addParameter("steamid", playerSteamID);
            newRequest.addParameter("behav_points", itos(pPlayer->getBehavPoints()));
            newRequest.addParameter("ban_points", itos(pPlayer->getBanPoints()));
            newRequest.addParameter("reason", "Admin ban");
            newRequest.addParameter("server", g_ShortServerName.GetString());
            newRequest.addParameter("admin", adminSteamID);
            newRequest.addParameter("extreason", "");
            URLManager::getInstance()->addNewRequest(newRequest);
        }

        RETURN_META(MRES_HANDLED);
    }
    RETURN_META(MRES_IGNORED);
}


bool ServerWiperPlugin::Load(PluginId id, ISmmAPI *ismm, char *error, size_t maxlen, bool late)
{
    ServerWiper::getInstance()->setPluginInstance(this);

	PLUGIN_SAVEVARS();

    char iface_buffer[255];
	int num=0;

	strcpy(iface_buffer, INTERFACEVERSION_SERVERGAMEDLL);
	FIND_IFACE(serverFactory, m_ServerDll, num, iface_buffer, IServerGameDLL *)
	strcpy(iface_buffer, INTERFACEVERSION_VENGINESERVER);
	FIND_IFACE(engineFactory, m_Engine, num, iface_buffer, IVEngineServer *)
	strcpy(iface_buffer, INTERFACEVERSION_SERVERGAMECLIENTS);
	FIND_IFACE(serverFactory, m_ServerClients, num, iface_buffer, IServerGameClients *)
	strcpy(iface_buffer, INTERFACEVERSION_GAMEEVENTSMANAGER2);
	FIND_IFACE(engineFactory, m_GameEventManager, num, iface_buffer, IGameEventManager2 *);
	strcpy(iface_buffer, VENGINE_CVAR_INTERFACE_VERSION);
	FIND_IFACE(engineFactory, icvar, num, iface_buffer, ICvar *);
	strcpy(iface_buffer, INTERFACEVERSION_ISERVERPLUGINHELPERS);
	FIND_IFACE(engineFactory, m_Helper, num, iface_buffer, IServerPluginHelpers *);
	strcpy(iface_buffer, INTERFACEVERSION_PLAYERINFOMANAGER);
	FIND_IFACE(serverFactory, m_PlayerInfoManager, num, iface_buffer, IPlayerInfoManager *);
	strcpy(iface_buffer, IENGINESOUND_SERVER_INTERFACE_VERSION);
	FIND_IFACE(engineFactory, pEngineSound, num, iface_buffer, IEngineSound *);
	strcpy(iface_buffer, FILESYSTEM_INTERFACE_VERSION);
	FIND_IFACE(engineFactory, m_FileSystem, num, iface_buffer, IFileSystem *);

	META_LOG(g_PLAPI, "Starting plugin.\n");

	ismm->AddListener(this, &g_Listener);
    ConCommandBaseMgr::OneTimeInit(&g_Accessor);

	SH_ADD_HOOK_MEMFUNC(IServerGameClients, ClientConnect, m_ServerClients, &g_ServerWiper, &ServerWiperPlugin::ClientConnect, false);
	SH_ADD_HOOK_MEMFUNC(IServerGameClients, ClientDisconnect, m_ServerClients, &g_ServerWiper, &ServerWiperPlugin::ClientDisconnect, true);
	SH_ADD_HOOK_MEMFUNC(IServerGameClients, ClientCommand, m_ServerClients, &g_ServerWiper, &ServerWiperPlugin::ClientCommand, false);
	SH_ADD_HOOK_MEMFUNC(IServerGameDLL, GameFrame, m_ServerDll, &g_ServerWiper, &ServerWiperPlugin::GameFrame, true);
	SH_ADD_HOOK_MEMFUNC(IServerGameDLL, ServerActivate, m_ServerDll, &g_ServerWiper, &ServerWiperPlugin::ServerActivate, true);
    SH_ADD_HOOK_MEMFUNC(IVEngineServer, LogPrint, m_Engine, &g_ServerWiper, &ServerWiperPlugin::LogPrintHandler, false);
    m_Engine_CC = SH_GET_CALLCLASS(m_Engine);
    SH_CALL(m_Engine_CC, &IVEngineServer::LogPrint)("All hooks started!\n");

	g_SMAPI->AddListener(g_PLAPI, this);

	pSayCmd = 0;
	pSayTeamCmd = 0;

	return true;
}

bool ServerWiperPlugin::Unload(char *error, size_t maxlen)
{
    m_GameEventManager->RemoveListener(this);

	SH_REMOVE_HOOK_MEMFUNC(IServerGameDLL, ServerActivate, m_ServerDll, &g_ServerWiper, &ServerWiperPlugin::ServerActivate, true);
	SH_REMOVE_HOOK_MEMFUNC(IServerGameClients, ClientDisconnect, m_ServerClients, &g_ServerWiper, &ServerWiperPlugin::ClientDisconnect, true);
	SH_REMOVE_HOOK_MEMFUNC(IServerGameClients, ClientCommand, m_ServerClients, &g_ServerWiper, &ServerWiperPlugin::ClientCommand, true);
	SH_REMOVE_HOOK_MEMFUNC(IServerGameClients, ClientConnect, m_ServerClients, &g_ServerWiper, &ServerWiperPlugin::ClientConnect, false);
	SH_REMOVE_HOOK_MEMFUNC(IServerGameDLL, GameFrame, m_ServerDll, &g_ServerWiper, &ServerWiperPlugin::GameFrame, true);
	SH_REMOVE_HOOK_MEMFUNC(IVEngineServer, LogPrint, m_Engine, &g_ServerWiper, &ServerWiperPlugin::LogPrintHandler, false);

	if(pSayCmd)
		SH_REMOVE_HOOK_MEMFUNC(ConCommand, Dispatch, pSayCmd, &g_ServerWiper, &ServerWiperPlugin::SayHandler, false);
	if(pSayTeamCmd)
		SH_REMOVE_HOOK_MEMFUNC(ConCommand, Dispatch, pSayTeamCmd, &g_ServerWiper, &ServerWiperPlugin::SayTeamHandler, false);
	SH_REMOVE_HOOK_MEMFUNC(IServerGameClients, SetCommandClient, m_ServerClients, &g_ServerWiper, &ServerWiperPlugin::SetCmdIndex, false);

    SH_RELEASE_CALLCLASS(m_Engine_CC);

	return true;
}

void ServerWiperPlugin::SayHandler()
{
    int idx = g_ServerWiper.GetCmdIndex();
    if (!idx)
        return;

    //ask the engine for the first argument
    const char *arg1 = g_ServerWiper.getEngine()->Cmd_Argv(1);

    if (!arg1)
        return;

    Event myNewEvent;
    myNewEvent.setType(EVENT_TYPE_PLAYER_SAY);
    myNewEvent.setAuthor(idx);
    myNewEvent.setIntValue(0);//Said to all
    myNewEvent.setText(arg1);
    ServerWiper::getInstance()->newEvent(myNewEvent);

    if(CVAR_BOOL(g_DisableChat))
    {
        ServerWiper::getInstance()->sendChatText(idx, _("Sorry, chatting is {deactivated}"));
        RETURN_META(MRES_SUPERCEDE);
    }

    if(ServerWiper::getInstance()->shouldShowSayMessage())
        RETURN_META(MRES_HANDLED);
    else
        RETURN_META(MRES_SUPERCEDE);
}

void ServerWiperPlugin::SayTeamHandler()
{
    int idx = g_ServerWiper.GetCmdIndex();
    if (!idx)
        RETURN_META(MRES_IGNORED);

    //ask the engine for the first argument
    const char *arg1 = g_ServerWiper.getEngine()->Cmd_Argv(1);

    if (!arg1)
        RETURN_META(MRES_IGNORED);

    Event myNewEvent;
    myNewEvent.setType(EVENT_TYPE_PLAYER_SAY);
    myNewEvent.setAuthor(idx);
    myNewEvent.setIntValue(1);//Said to team
    myNewEvent.setText(arg1);
    ServerWiper::getInstance()->newEvent(myNewEvent);

    if(CVAR_BOOL(g_DisableChat))
    {
		ServerWiper::getInstance()->sendChatText(idx, _("Sorry, chatting is {deactivated}"));
        RETURN_META(MRES_SUPERCEDE);
    }

    if(ServerWiper::getInstance()->shouldShowSayMessage())
        RETURN_META(MRES_HANDLED);
    else
        RETURN_META(MRES_SUPERCEDE);
}

void ServerWiperPlugin::AllPluginsLoaded()
{
	 m_GameEventManager->AddListener(this, "player_team", true);
	 m_GameEventManager->AddListener(this, "player_disconnect", true);
	 m_GameEventManager->AddListener(this, "player_hurt", true);
	 m_GameEventManager->AddListener(this, "player_death", true);
	 m_GameEventManager->AddListener(this, "bomb_planted", true);
	 m_GameEventManager->AddListener(this, "bomb_defused", true);
	 m_GameEventManager->AddListener(this, "hostage_killed", true);
	 m_GameEventManager->AddListener(this, "hostage_follows", true);
	 m_GameEventManager->AddListener(this, "hostage_rescued", true);
	 m_GameEventManager->AddListener(this, "hostage_hurt", true);
	 m_GameEventManager->AddListener(this, "player_changename", true);
	 m_GameEventManager->AddListener(this, "flashbang_detonate", true);
	 m_GameEventManager->AddListener(this, "player_blind", true);
	 m_GameEventManager->AddListener(this, "round_start", true);
	 m_GameEventManager->AddListener(this, "player_spawn", true);

	 //find the "say" command in the server's CVAR list
    ConCommandBase *pCmd = icvar->GetCommands();

    while (pCmd)
    {
            if(pCmd->IsCommand() && (strcmp(pCmd->GetName(), "say") == 0))
                    break;
            pCmd = const_cast<ConCommandBase *>(pCmd->GetNext());
    }

    if(pCmd)
    {
        pSayCmd = (ConCommand *)pCmd;
        SH_ADD_HOOK_MEMFUNC(ConCommand, Dispatch, pSayCmd, this, &ServerWiperPlugin::SayHandler, false);
    }

    //find the "say_team" command in the server's CVAR list
    pCmd = icvar->GetCommands();

    while (pCmd)
    {
            if(pCmd->IsCommand() && (strcmp(pCmd->GetName(), "say_team") == 0))
                    break;
            pCmd = const_cast<ConCommandBase *>(pCmd->GetNext());
    }

    if(pCmd)
    {
        pSayTeamCmd = (ConCommand *)pCmd;
        SH_ADD_HOOK_MEMFUNC(ConCommand, Dispatch, pSayTeamCmd, this, &ServerWiperPlugin::SayTeamHandler, false);
    }

    //hook the engine's function to our version
    SH_ADD_HOOK_MEMFUNC(IServerGameClients, SetCommandClient, m_ServerClients, this, &ServerWiperPlugin::SetCmdIndex, false);

    ServerWiper::getInstance()->postInit();
}

int ServerWiperPlugin::GetCmdIndex()
{
    return m_CmdIndex;
}

void ServerWiperPlugin::SetCmdIndex(int cmd)
{
    m_CmdIndex = cmd + 1; //HL2 is offset by -1 for some reason...
}

void ServerWiperPlugin::FireGameEvent( IGameEvent *event )
{
	if (!event)
		return;

    ServerWiper::getInstance()->gameEvent(event->GetName(), event);
}

void ServerWiperPlugin::ServerActivate(edict_t *pEdictList, int edictCount, int clientMax)
{
    maxClients = clientMax;
	RETURN_META(MRES_IGNORED);
}

void *MyListener::OnMetamodQuery(const char *iface, int *ret)
{
	if (strcmp(iface, "ServerWiper")==0)
	{
		if (ret)
			*ret = IFACE_OK;
		return static_cast<void *>(&g_ServerWiper);
	}

	if (ret)
		*ret = IFACE_FAILED;

	return NULL;
}

edict_t *EdictOfUserId( int UserId )
{
	int i;
	edict_t *pEnt;
	for  (i=0; MAX_PLAYERS; i++)
	{
		pEnt = g_ServerWiper.m_Engine->PEntityOfEntIndex(i);
		if (pEnt && !pEnt->IsFree())
		{
			if (FStrEq(pEnt->GetClassName(), "player"))
			{
				if (g_ServerWiper.m_Engine->GetPlayerUserId(pEnt)==UserId)
				{
					return pEnt;
				}
			}
		}
	}

	return NULL;
}

// find an offset for PropertyName listed in the server class send table for ClassName
int UTIL_FindOffset(const char *ClassName, const char *PropertyName)
{
	ServerClass *sc = g_ServerWiper.m_ServerDll->GetAllServerClasses();
	while (sc)
	{
		if (FStrEq(sc->GetName(), ClassName))
		{
			int NumProps = sc->m_pTable->GetNumProps();
			for (int i=0; i<NumProps; i++)
			{
				if (stricmp(sc->m_pTable->GetProp(i)->GetName(), PropertyName) == 0)
				{
					return sc->m_pTable->GetProp(i)->GetOffset();
				}
			}
			return 0;
		}
		sc = sc->m_pNext;
	}

	return 0;
}

const char *ServerWiperPlugin::GetAuthor()
{
	return "Jmgr";
}

const char *ServerWiperPlugin::GetName()
{
	return "ServerWiper";
}

const char *ServerWiperPlugin::GetDescription()
{
	return "Watches a server to punish bad behaviour";
}

const char *ServerWiperPlugin::GetURL()
{
	return "http://www.jmgr.info/";
}

const char *ServerWiperPlugin::GetLicense()
{
	return "GPL";
}

const char *ServerWiperPlugin::GetVersion()
{
	return SERVER_WIPER_VERSION;
}

const char *ServerWiperPlugin::GetDate()
{
	return __DATE__;
}

const char *ServerWiperPlugin::GetLogTag()
{
	return "SW";
}

IVEngineServer *ServerWiperPlugin::getEngine()
{
    return m_Engine;
}

void ServerWiperPlugin::ClientDisconnect(edict_t *pEntity)
{
	RETURN_META(MRES_IGNORED);
}

bool ServerWiperPlugin::ClientConnect(edict_t *pEntity, const char *pszName, const char *pszAddress, char *reject, int maxrejectlen)
{
    Event myNewEvent;
	myNewEvent.setType(EVENT_TYPE_PLAYER_CONNECTED);
	myNewEvent.setAuthor(m_Engine->IndexOfEdict(pEntity));
	myNewEvent.setText(pszName);
	myNewEvent.setSecondText(pszAddress);
	myNewEvent.setEdict(pEntity);
	ServerWiper::getInstance()->newEvent(myNewEvent);

	RETURN_META_VALUE(MRES_HANDLED, true);
}

void ServerWiperPlugin::ClientCommand(edict_t *pEntity)
{
	int id = m_Engine->IndexOfEdict(pEntity);
	shString command = m_Engine->Cmd_Argv(0);

	if(command == "menuselect")
	{
		char *arg1 = m_Engine->Cmd_Argv(1);
		if(arg1)
		{
			int arg = atoi(arg1);
			if(arg < 1 || arg > 10)
				RETURN_META(MRES_IGNORED);

			MenuManager::getInstance()->onPlayerMenuSelect(id, arg);
		}
		RETURN_META(MRES_HANDLED);
	}

    if(ServerWiper::getInstance()->gotPlayerCommand(id, command))
        RETURN_META(MRES_SUPERCEDE);

    RETURN_META(MRES_IGNORED);
}

void ServerWiperPlugin::GameFrame(bool simulating)
{
    ServerWiper::getInstance()->think();
}
