/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#ifndef _INCLUDE_SAMPLEPLUGIN_H
#define _INCLUDE_SAMPLEPLUGIN_H

#include <ISmmPlugin.h>
#include <sourcehook/sourcehook.h>
#include <igameevents.h>
#include <iplayerinfo.h>
#include <IEngineSound.h>
#include <filesystem.h>

#define SERVER_WIPER_VERSION	            "1.16"
#define SERVER_WIPER_DATABASE_VERSION       1
#define SERVER_WIPER_PLAYER_DATA_VERSION    1

class ServerWiperPlugin : public ISmmPlugin, public IMetamodListener, public IGameEventListener2
{
public:
	bool Load(PluginId id, ISmmAPI *ismm, char *error, size_t maxlen, bool late);
	bool Unload(char *error, size_t maxlen);
	bool Pause(char *error, size_t maxlen)
	{
		sprintf(error,"This plugin cannot be paused");
		return false;
	}
	bool Unpause(char *error, size_t maxlen)
	{
		sprintf(error,"This plugin cannot be unpaused");
		return false;
	}
	void AllPluginsLoaded();

public:
	const char *GetAuthor();
	const char *GetName();
	const char *GetDescription();
	const char *GetURL();
	const char *GetLicense();
	const char *GetVersion();
	const char *GetDate();
	const char *GetLogTag();
	int GetApiVersion()         { return PLAPI_VERSION; }
	int getMaxClients()         { return maxClients; }

	IVEngineServer *getEngine();
	IServerGameDLL *getServerDLL()  { return m_ServerDll; }
	IServerPluginHelpers *getHelper() { return m_Helper; }
	IServerGameClients *getGameClients() { return m_ServerClients; }
	IPlayerInfoManager *getPlayerInfoManager() { return m_PlayerInfoManager; }
	IEngineSound *getEngineSound()  { return pEngineSound; }
	IFileSystem *getFilesystem()  { return m_FileSystem; }
	ICvar *getICvar()  { return icvar; }

public:
	bool ClientConnect(edict_t *pEntity, const char *pszName, const char *pszAddress, char *reject, int maxrejectlen);
	void ClientDisconnect(edict_t *pEntity);
	void ClientCommand(edict_t *pEntity);
	void GameFrame(bool simulating);
	void ServerActivate(edict_t *pEdictList, int edictCount, int clientMax);
	void LogPrint(const char *msg);
	void FireGameEvent(IGameEvent* event);
    int GetCmdIndex();
    void SetCmdIndex(int cmd);
    void SayHandler();
    void SayTeamHandler();
    void LogPrintHandler(const char *msg);
	bool CacheCmds();

	IGameEventManager2 *m_GameEventManager;
	IServerGameDLL *m_ServerDll;
	IVEngineServer *m_Engine;
	IServerGameClients *m_ServerClients;
	IServerPluginHelpers *m_Helper;
	IPlayerInfoManager *m_PlayerInfoManager;
	IEngineSound *pEngineSound;
	IFileSystem *m_FileSystem;
    SourceHook::CallClass<IVEngineServer> *m_Engine_CC;
    ConCommand *pSayCmd;
    ConCommand *pSayTeamCmd;
    ConCommand *pConPrintCmd;
    ICvar *icvar;
    int m_CmdIndex;
    int maxClients;
};

class MyListener : public IMetamodListener
{
public:
	virtual void *OnMetamodQuery(const char *iface, int *ret);
};

extern ServerWiperPlugin g_ServerWiper;
PLUGIN_GLOBALVARS();

edict_t *EdictOfUserId( int UserId );
int UTIL_FindOffset(const char *ClassName, const char *PropertyName);

#endif //_INCLUDE_SAMPLEPLUGIN_H
