/*
ServerWiper, watch a Source server to punish bad behaviour
Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Contact e-mail : sw@jmgr.info
*/

#include "team.h"
#include "cvars.h"
#include "language.h"

TeamManager::TeamManager()
{
}

TeamManager::~TeamManager()
{
    team.clear();
}

bool TeamManager::loadTeamFile(shString const& filename)
{
	team.clear();

	shString pathAndFilename = "cfg/sw/text/";
	pathAndFilename += filename;
	if(!ServerWiper::getInstance()->getFilesystem()->FileExists(pathAndFilename.c_str(), "MOD"))
	{
		META_LOG(g_PLAPI,"Could not open the team file : %s", filename.c_str());
		return false;
	}

	FileHandle_t file = ServerWiper::getInstance()->getFilesystem()->Open(pathAndFilename.c_str(), "r", "MOD");

	for(int j=0;!ServerWiper::getInstance()->getFilesystem()->EndOfFile(file);j++)
	{
		char text[2048];
		ServerWiper::getInstance()->getFilesystem()->ReadLine(text, sizeof(text), file);
		shString steamID = text;
		steamID = steamID.substr(0, steamID.size() - 1);
		team.push_back(steamID);
	}

	ServerWiper::getInstance()->getFilesystem()->Close(file);

	META_CONPRINTF("Read %d team members\n", team.size());

	return true;
}

bool TeamManager::isTeamMember(shString const& steamID)
{
	for(int i=0;i<team.size();i++)
	{
		if(team[i] == steamID)
			return true;
	}

	return false;
}

void TeamManager::checkPlayer(int const& playerId)
{
	if(!CVAR_BOOL(g_ServerWiperEnabled) || !CVAR_BOOL(g_TeamMembersTagCheck))
		return;

	Player *player = ServerWiper::getInstance()->getPlayer(playerId);
	if(!player)
		return;

	if(strcmp(g_TeamMembersTag.GetString(), "") == 0)
		return;

	if(isTeamMember(player->getSteamID()))
	{
		if(!isPatternInString(player->getName(), g_TeamMembersTag.GetString()))
		{
			if(CVAR_INT(g_DebugLevel))
			{
				META_LOG(g_PLAPI, "Admin %s (%s) was not wearing the team tag", player->getName().c_str(), player->getSteamID().c_str());
			}
			ServerWiper::getInstance()->kickPlayer(playerId, _("Team member without tag %s", g_TeamMembersTag.GetString()));
		}
	}
}
