/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#include "url.h"

size_t write_data(void *buffer, size_t size, size_t nmemb, void *userp)
{
    //Do nothing, since we don't care about the return value
    return 0;
}

void* threadHandleURL(void*)
{
    URLManager *urlManager = URLManager::getInstance();

	while(urlManager->getRunning())
	{
        if(urlManager->getRequestCount() > 0)
        {
            URLRequest actualRequest;
            urlManager->popRequest(actualRequest);

            CURL *url = curl_easy_init();
            if(!url)
            {
                urlManager->addMessage(shString("Cannot create url ") + actualRequest.getURL());
                continue;
            }

            struct curl_httppost *post=NULL;
            struct curl_httppost *last=NULL;

            for(int i=0;i<actualRequest.getNames().size();i++)
            {
                curl_formadd(&post, &last, CURLFORM_COPYNAME, actualRequest.getNames()[i].c_str(),
                    CURLFORM_COPYCONTENTS, actualRequest.getValues()[i].c_str(), CURLFORM_END);
            }

            shString urlString = actualRequest.getURL();

            curl_easy_setopt(url, CURLOPT_URL, urlString.c_str());
            curl_easy_setopt(url, CURLOPT_WRITEFUNCTION, write_data);
            curl_easy_setopt(url, CURLOPT_HTTPPOST, post);

            CURLcode res = curl_easy_perform(url);
            if(res != CURLE_OK)
            {
                urlManager->addMessage(shString("Error performing request : ") + itos(res) + shString(" (url : ") + actualRequest.getURL() + shString(")"));
                continue;
            }

            curl_easy_cleanup(url);
        }
        #ifdef _WIN32
        Sleep(100);
        #else
        usleep(100000);
        #endif
	}

	pthread_exit(NULL);
	return NULL;
}

URLManager::URLManager()
{
    curl_global_init(CURL_GLOBAL_WIN32);//Does nothing on linux
                                        // -> not thread safe !
    requests.init();

    #ifdef _WIN32
	pthread_win32_process_attach_np();
	pthread_win32_thread_attach_np();
    #endif
    pthread_mutex_init(&runningMutex,NULL);

    setRunning(true);

	pthread_create(&urlThread,NULL,&threadHandleURL,NULL);

}

URLManager::~URLManager()
{
    setRunning(false);
	pthread_mutex_lock(&runningMutex);
	pthread_mutex_destroy(&runningMutex);

	requests.destroy();

    #ifdef _WIN32
    pthread_win32_thread_detach_np();
    pthread_win32_process_detach_np();
    #endif

    curl_global_cleanup();
}

void URLManager::addNewRequest(URLRequest const& newRequest)
{
    requests.push(newRequest);
}

void URLManager::setRunning(bool running)
{
    pthread_mutex_lock(&runningMutex);
    run = running;
    pthread_mutex_unlock(&runningMutex);
}

bool URLManager::getRunning()
{
    pthread_mutex_lock(&runningMutex);
    bool isRunning = run;
    pthread_mutex_unlock(&runningMutex);
    return isRunning;
}
