/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#ifndef _URL_H
#define _URL_H

#include "curl/curl.h"
#include <pthread.h>

#include "safeStack.h"
#include "util.h"
#include "serverWiper.h"

class URLRequest
{
public:
    URLRequest()                                   {url = ""; names.clear(); values.clear();}
    ~URLRequest()                                  {names.clear(); values.clear();}

    void setURL(shString const& url)               {this->url = url;}
    void addParameter(shString const& name, shString const& value)    {names.push_back(name); values.push_back(value);}

    shString getURL()                              {return url;}
    shVector<shString> getNames()                  {return names;}
    shVector<shString> getValues()                 {return values;}

private:
    shString url;
    shVector<shString> names;
    shVector<shString> values;
};

class URLManager : public Singleton <URLManager>
{
    friend class Singleton <URLManager>;

public:
    URLManager();
    ~URLManager();

    void setRunning(bool running);
    bool getRunning();

    void addNewRequest(URLRequest const& newRequest);
    int getRequestCount()                           {return requests.count();}
    void popRequest(URLRequest & request)      {request = requests.pop();}

    int getMessagesCount()                          {return messages.count();}
    shString popMessage()                           {return messages.pop();}
    void addMessage(shString const& newMessage)     {messages.push(newMessage);}

private:
    pthread_t urlThread;
    safeStack<shString> messages;
    pthread_mutex_t runningMutex;
    bool run;
    safeStack<URLRequest> requests;
};

void* threadHandleURL(void*);

#endif //_URL_H

