/*
    ServerWiper, watch a Source server to punish bad behaviour
    Copyright (C) 2007-2008  Jonathan "Jmgr" Mercier-Ganady

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed inp the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact e-mail : sw@jmgr.info
*/

#include <ctype.h>

#include "util.h"
#include "language.h"

shString itos(int const& value)
{
    char buffer[4096];
    sprintf(buffer, "%d", value);
    return shString(buffer);
}

shString trimText(shString const& text)
{
    shString back = text;
    int i = 0;
    for(;back[i] == ' ' || back[i] == '\t';i++);
    back.erase(0, i);

    i = back.size();
    for (;i && back[i-1]==' ' || back[i]=='\t';i--);
    back.erase(i, back.size()-i);
    return back;
}

shString getSpacedString(shString const& text)
{
    shString back = "";

    for(int i = 0;i<text.size();i++)
    {
        back += text[i];
        back += ' ';
    }
    back = trimText(back);

    return back;
}

shString deleteFromString(shString const& text, char toDelete)
{
    shString back = "";
    for(int i = 0;i<text.size();i++)
    {
        if(text[i] != toDelete)
            back += text[i];
    }
    return back;
}

shString getCleanedString(shString const& text)
{
    shString back = deleteFromString(text, '\'');
    back = deleteFromString(back, '\"');
    back = deleteFromString(back, '\n');
    back = deleteFromString(back, '\\');
    back = deleteFromString(back, '=');
    back = deleteFromString(back, '');
    back = deleteFromString(back, '}');
    back = deleteFromString(back, '}');
    back = deleteFromString(back, '{');

    return back;
}

shString getLowerString(shString const& text)
{
    shString back = text;
    for(int i=0;i<back.size();i++)
    {
        back.at(i, tolower(back[i]));
    }
    return back;
}

shString getToken(shString const& text, int const& num)
{
    shString buffer = "";
    int tokensFound = 0;
    for(int i=0;;i++)
    {
        if(text[i] == ',' || i>=text.size())
        {
            tokensFound++;

            if(num == tokensFound || i>=text.size())
                return buffer;

            buffer = "";
        }
        else
            buffer += text[i];
    }

    return "";
}

shString getIntegerChangeFormatted(int const& value)
{
    shString back = "";

    if(value < 0)
        back += "-";
    else
        back += "+";
    back += itos(abs(value));

    return back;
}

shString boolToString(bool status)
{
    if(status)
        return _("ON");
    else
        return _("OFF");
}

bool isPatternInString(shString const& string, shString const& pattern)
{
    if(!pattern.size())
        return false;

    for(int i=0;i<string.size();i++)
    {
        if(string[i] == pattern[0])
        {
            for(int j=i,k=0;j<string.size();j++,k++)
            {
                if(string[j] != pattern[k])
                    break;

                if(k == pattern.size()-1)
                    return true;
            }
        }
    }
    return false;
}

shString cleanString(shString const& string, int const& maxlen)
{
    shString back = "";

    for(int i=0;i<string.size() || back.size() == maxlen;i++)
    {
        if(string[i] != '\n')
            back += string[i];
    }

    return back;
}

shString cutString(shString & string, int const& size)
{
    shString back = string.substr(0, size);

    string = string.substr(size);

    return back;
}

